<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

require_once( 'xeroom_indexInit.php' );
require_once( 'xeroom_bulk_action.php' );
require_once( 'xeroom_cancel_order_by_paypal.php' );
require_once( 'vendor/autoload.php' );

$xeroom_bulk = new Xeroom_Batch_Request();

add_action( 'admin_menu', 'xeroom_menu_icon_under_admin' );
add_action( 'admin_head', 'xeroom_display_message_if_licence_not_valid', 40 );

/**
 * Send data to Xero based on Xeroom settings
 */
$send_invoice_method = get_xero_option( 'xero_send_invoice_method' );
$send_payment_method = get_xero_option( 'xero_send_payment_method' );

if ( $send_invoice_method && 'on_creation' == $send_invoice_method ) {
	add_action( 'woocommerce_checkout_order_processed', 'xeroom_sendWooInvoiceToXero', 50 );
}

if ( $send_invoice_method && 'on_processing' == $send_invoice_method ) {
	add_action( 'woocommerce_order_status_processing', 'xeroom_sendWooInvoiceToXero', 50 );
}

add_action( 'woocommerce_order_status_completed', 'xeroom_paymentDoneOnCheckoutManually', 50 );
add_action( 'woocommerce_order_edit_status', 'xeroom_send_payment_on_manually_complete', 10, 2 );
add_action( 'woocommerce_payment_complete', 'xeroom_paymentDoneOnPayment' );

add_action( 'rest_api_init', 'xeroom_register_endpoint' );
/**
 * API End-point
 */
function xeroom_register_endpoint() {
	register_rest_route(
		'xeroom/v2',
		'oauth_callback',
		array(
			'methods'             => 'GET',
			'callback'            => 'xeroom_authorize',
			'permission_callback' => '__return_true',
		)
	);
}

function xeroom_authorize() {
	require_once( XEROOM_ROOT_PATH . 'storage.php' );
	$storage = new StorageClass();

	$xero_oauth_client_id     = get_xero_option( 'xero_oauth_client_id' );
	$xero_oauth_client_secret = get_xero_option( 'xero_oauth_client_secret' );

	$provider = new \League\OAuth2\Client\Provider\GenericProvider(
		[
			'clientId'                => $xero_oauth_client_id,
			'clientSecret'            => $xero_oauth_client_secret,
			'redirectUri'             => get_rest_url( null, 'xeroom/v2/oauth_callback' ),
			'urlAuthorize'            => 'https://login.xero.com/identity/connect/authorize',
			'urlAccessToken'          => 'https://identity.xero.com/connect/token',
			'urlResourceOwnerDetails' => 'https://api.xero.com/api.xro/2.0/Organisation',
		]
	);

	// If we don't have an authorization code then get one
	if ( ! isset( $_GET['code'] ) ) {
		echo 'Something went wrong, no authorization code found';

		update_xero_option( 'xero_connection_status', 'failed' );
		update_xero_option( 'xero_connection_status_message', 'Something went wrong, no authorization code found' );

		exit( 'Something went wrong, no authorization code found' );

		// Check given state against previously stored one to mitigate CSRF attack
	} elseif ( empty( $_GET['state'] ) || ( $_GET['state'] !== $_SESSION['oauth2state'] ) ) {
		echo 'Invalid State';
		unset( $_SESSION['oauth2state'] );

		update_xero_option( 'xero_connection_status', 'failed' );
		update_xero_option( 'xero_connection_status_message', 'Invalid State' );

		exit( 'Invalid state' );
	} else {

		try {
			// Try to get an access token using the authorization code grant.
			$accessToken = $provider->getAccessToken(
				'authorization_code',
				[
					'code' => $_GET['code'],
				]
			);

			update_xero_option( 'xero_access_token_get', (array) $_GET );
			update_xero_option( 'xero_access_token_data', (array) $accessToken );

			$config      = XeroAPI\XeroPHP\Configuration::getDefaultConfiguration()->setAccessToken( (string) $accessToken->getToken() );
			$identityApi = new XeroAPI\XeroPHP\Api\IdentityApi(
				new GuzzleHttp\Client(),
				$config
			);

			$result = $identityApi->getConnections();

			$xero_oauth_options = [
				'token'         => $accessToken->getToken(),
				'expires'       => $accessToken->getExpires(),
				'tenant_id'     => $result[0]->getTenantId(),
				'refresh_token' => $accessToken->getRefreshToken(),
			];
			update_xero_option( 'xero_oauth_options', $xero_oauth_options, false );

			update_xero_option( 'xero_connection_status', 'active' );
			update_xero_option( 'xero_connection_status_message', '' );

			header( 'Location: ' . admin_url( 'admin.php?page=add_xero_api_fields' ) );
			exit();

		} catch ( \League\OAuth2\Client\Provider\Exception\IdentityProviderException $e ) {
			echo 'Callback failed';

			update_xero_option( 'xero_connection_status', 'failed' );
			update_xero_option( 'xero_connection_status_message', 'Callback failed' );

			exit();
		}
	}
}

/**
 * Do token refresh
 *
 * @return mixed
 * @throws Exception
 */
function xeroom_refresh_token() {
	$xero_oauth_client_id     = get_xero_option( 'xero_oauth_client_id' );
	$xero_oauth_client_secret = get_xero_option( 'xero_oauth_client_secret' );
	$xero_oauth_options       = get_xero_option( 'xero_oauth_options' );

	$token_refresh = wp_remote_request(
		'https://identity.xero.com/connect/token',
		array(
			'method'  => 'POST',
			'headers' => array(
				'Authorization'  => 'Basic ' . base64_encode( $xero_oauth_client_id . ':' . $xero_oauth_client_secret ),
				'Xero-tenant-id' => $xero_oauth_options['tenant_id'],
				'Content-Type'   => 'application/x-www-form-urlencoded',
			),
			'timeout' => 70,
			'body'    => 'grant_type=refresh_token&refresh_token=' . $xero_oauth_options['refresh_token'],
		)
	);

	if ( is_wp_error( $token_refresh ) ) {
		update_xero_option( 'xero_connection_status', 'failed' );
		update_xero_option( 'xero_connection_status_message', 'There was a problem connecting to the API' );

		throw new Exception( 'There was a problem connecting to the API.' );
	}

	if ( isset( $token_refresh['body'] ) && 0 === strpos( $token_refresh['body'], 'oauth_problem=' ) ) {

		// Parse error string.
		parse_str( $token_refresh['body'], $oauth_error );

		// Find OAuth advise.
		$oauth_advise = ( ( isset( $oauth_error['oauth_problem_advice'] ) ) ? $oauth_error['oauth_problem_advice'] : '' );

		update_xero_option( 'xero_connection_status', 'failed' );
		update_xero_option( 'xero_connection_status_message', sprintf( 'Request failed due OAuth error: %s | %s', $oauth_error['oauth_problem'], $oauth_advise ) );

		// Throw new exception.
		throw new Exception( sprintf( 'Request failed due OAuth error: %s | %s', $oauth_error['oauth_problem'], $oauth_advise ) );
	}

	$xero_response = json_decode( wp_remote_retrieve_body( $token_refresh ), true );

	if ( isset( $xero_response['access_token'] ) ) {
		$xero_oauth_options_update = [
			'token'         => $xero_response['access_token'],
			'expires'       => time() + $xero_response['expires_in'],
			'tenant_id'     => $xero_oauth_options['tenant_id'],
			'refresh_token' => $xero_response['refresh_token'],
		];

		update_xero_option( 'xero_oauth_options', $xero_oauth_options_update, false );

		update_xero_option( 'xero_connection_status', 'active' );
		update_xero_option( 'xero_connection_status_message', '' );

		return $xero_response['access_token'];
	}

	return '';
}

/**
 * Schedule license check
 */
if ( ! wp_next_scheduled( 'xeroom_check_license' ) ) {
	wp_schedule_event( time(), 'daily', 'xeroom_check_license' );
}

add_action( 'xeroom_check_license', 'xeroom_check_license_method' );
/**
 *
 */
function xeroom_check_license_method() {
	global $wpdb;

	$xeroLicActive     = $wpdb->prefix . 'xeroom_license_key_status';
	$sql               = 'SELECT * FROM ' . $xeroLicActive . ' WHERE id=1';
	$xeroLicensekeyAct = $wpdb->get_results( $sql );

	$xero_plugin_license = sanitize_text_field( $xeroLicensekeyAct[0]->license_key );

	if ( ! empty( $xero_plugin_license ) ) {
		$url          = 'https://www.xeroom.com/apidata.php?license_key=' . $xero_plugin_license;
		$result       = wp_remote_fopen( esc_url( $url ) );
		$validLicense = json_decode( $result, true );

		if ( count( $validLicense ) > 0 ) {
			$license_expire_date = esc_attr( $validLicense[0]['date_expiry'] );
			update_xero_option( 'xero_license_expire_date', $license_expire_date );

			if ( $validLicense[0]['lic_status'] == 'active' ) {
				update_xero_option( 'xero_connection_status', 'active' );
				update_xero_option( 'xero_connection_status_message', '' );
				update_xero_option( 'xero_license_status', 1 );
				// Delete the first check date.
				delete_xero_option( 'xero_license_check_date' );

				$wpdb->update(
					$xeroLicActive,
					array(
						'status' => 'active',
					),
					array( 'id' => 1 )
				);
			} elseif ( $validLicense[0]['lic_status'] == 'expired' ) {
				if ( $license_expire_date ) {
					$after_7_days = strtotime( '+7 day', strtotime( $license_expire_date ) );
					$today        = date( 'Y-m-d' );
					$datetime1    = new DateTime( $today );
					$datetime2    = new DateTime( date( 'Y-m-d', $after_7_days ) );
					$difference   = $datetime2->diff( $datetime1 )->format( '%a' );

					if ( $after_7_days >= strtotime( date( 'Y-m-d' ) ) ) {
						$licenseErrMessage = "Your licence cannot validate.  It has either expired or cannot reach our server.  Xeroom will stop working in $difference days.  Please renew or contact us if this message remains after 2 days.";
						update_xero_option( 'xero_connection_status', 'failed' );
						update_xero_option( 'xero_connection_status_message', $licenseErrMessage );
						update_xero_option( 'xero_license_status', 1 );

						$wpdb->update(
							$xeroLicActive,
							array(
								'status' => 'active',
							),
							array( 'id' => 1 )
						);
					} else {
						$licenseErrMessage = 'Your key is not active or is expired. Please contact Xeroom support for assistance.';
						update_xero_option( 'xero_connection_status', 'expired' );
						update_xero_option( 'xero_connection_status_message', $licenseErrMessage );
						update_xero_option( 'xero_license_status', 0 );

						$wpdb->update(
							$xeroLicActive,
							array(
								'status' => 'expired',
							),
							array( 'id' => 1 )
						);
					}
				} else {
					$first_try = date( 'Y-m-d' );
					update_xero_option( 'xero_license_check_date', $first_try );

					$licenseErrMessage = 'Your licence cannot validate.  It has either expired or cannot reach our server.  Xeroom will stop working in 7 days.  Please renew or contact us if this message remains after 2 days.';
					update_xero_option( 'xero_connection_status', 'failed' );
					update_xero_option( 'xero_connection_status_message', $licenseErrMessage );
					update_xero_option( 'xero_license_status', 0 );

					$wpdb->update(
						$xeroLicActive,
						array(
							'license_key' => $xero_plugin_license,
							'status'      => 'active',
							'xero_method' => 'premium',
						),
						array( 'id' => 1 )
					);
				}
			} else {
				$licenseErrMessage = 'Your key is not active or is expired. Please contact Xeroom support for assistance.';
				update_xero_option( 'xero_connection_status', 'expired' );
				update_xero_option( 'xero_connection_status_message', $licenseErrMessage );
				update_xero_option( 'xero_license_status', 0 );

				$wpdb->update(
					$xeroLicActive,
					array(
						'status' => 'expired',
					),
					array( 'id' => 1 )
				);
			}
		} else {
			$first_try = get_xero_option( 'xero_license_check_date' );
			if ( ! $first_try ) {
				$first_try = date( 'Y-m-d' );
				update_xero_option( 'xero_license_check_date', $first_try );
			}

			$after_7_days = strtotime( '+7 day', strtotime( $first_try ) );
			$today        = date( 'Y-m-d' );
			$datetime1    = new DateTime( $today );
			$datetime2    = new DateTime( date( 'Y-m-d', $after_7_days ) );
			$difference   = $datetime2->diff( $datetime1 )->format( '%a' );

			if ( $after_7_days >= strtotime( date( 'Y-m-d' ) ) ) {
				$licenseErrMessage = "";
				update_xero_option( 'xero_connection_status', 'failed' );
				update_xero_option( 'xero_connection_status_message', $licenseErrMessage );
				update_xero_option( 'xero_license_status', 1 );

				$wpdb->update(
					$xeroLicActive,
					array(
						'status' => 'active',
					),
					array( 'id' => 1 )
				);
			} else {
				$licenseErrMessage = 'Your key is not active or is expired. Please contact Xeroom support for assistance.';
				update_xero_option( 'xero_connection_status', 'expired' );
				update_xero_option( 'xero_connection_status_message', $licenseErrMessage );
				update_xero_option( 'xero_license_status', 0 );

				$wpdb->update(
					$xeroLicActive,
					array(
						'status' => 'expired',
					),
					array( 'id' => 1 )
				);
			}
		}
	}

	/**
	 * Check connection status
	 */
	$xeroCredentialTable    = $wpdb->prefix . 'xeroom_credentials';
	$sql                    = 'SELECT xero_api_key, xero_api_secret FROM ' . $xeroCredentialTable . ' WHERE id=1';
	$xeroCredentialsFromTbl = $wpdb->get_results( $sql );
	if ( $xeroCredentialsFromTbl ) {
		$xeroApiKey    = $xeroCredentialsFromTbl[0]->xero_api_key;
		$xeroApiSecret = $xeroCredentialsFromTbl[0]->xero_api_secret;

		if ( ! defined( 'BASE_PATH' ) ) {
			define( 'BASE_PATH', dirname( __FILE__ ) );
		}

		if ( ! defined( 'PUBLIC_KEY' ) ) {
			define( 'PUBLIC_KEY', BASE_PATH . '/library/certs/publickey.cer' );
		}

		if ( ! defined( 'PRIVATE_KEY' ) ) {
			define( 'PRIVATE_KEY', BASE_PATH . '/library/certs/privatekey.pem' );
		}

		$oauth2 = get_xero_option( 'xero_oauth_options' );
		$xero   = new Xero( $xeroApiKey, $xeroApiSecret, PUBLIC_KEY, PRIVATE_KEY, 'json', $oauth2 );

		$xero_tracking = $xero->TrackingCategories();
		xeroom_check_connection_message( $xero_tracking );
	}
}
