<?php
/**
 * Load Xeroom Process
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

global $wpdb;

/**
 * Xeroom version
 */
define( 'XEROOM_VERSION', '2.2.2' );
/**
 * Xeroom Define Http Path
 */
define( 'XEROOM_HTTP_PATH', plugin_dir_url( __FILE__ ) );
/**
 * Xeroom Define Root Path
 */
define( 'XEROOM_ROOT_PATH', plugin_dir_path( __FILE__ ) );
/**
 * Xeroom Define CSS Path
 */
define( 'XEROOM_CSS_PATH', XEROOM_HTTP_PATH . 'css/' );
/**
 * Xeroom Define JS Path
 */
define( 'XEROOM_JS_PATH', XEROOM_HTTP_PATH . 'js/' );
/**
 * Xeroom Define Image Path
 */
define( 'XEROOM_IMAGE_PATH', XEROOM_HTTP_PATH . 'images/' );
/**
 * Xeroom Define Log Path
 */
define( 'XEROOM_LOG_PATH', XEROOM_ROOT_PATH . 'library/log/' );
/**
 * Xeroom Define Excel Path
 */
define( 'XEROOM_EXCEL_PATH', XEROOM_ROOT_PATH . 'library/excel/' );


if ( ! function_exists( 'is_plugin_active_for_network' ) ) {
	require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
}

if ( is_plugin_active_for_network( 'xeroom/index.php' ) ) {
	/**
	 * Is Network Activated
	 */
	define( 'XERO_PLUGIN_NETWORK_ACTIVATED', true );
} else {
	/**
	 * Activated on Single Blog
	 */
	define( 'XERO_PLUGIN_NETWORK_ACTIVATED', false );
}

/**
 * Get Option
 *
 * @param $option_name
 *
 * @return mixed|void
 */
function get_xero_option( $option_name ) {
	if ( true === XERO_PLUGIN_NETWORK_ACTIVATED ) {
		return get_site_option( $option_name );
	} else {
		return get_option( $option_name );
	}
}

/**
 * Update Option
 *
 * @param $option_name
 * @param $option_value
 *
 * @return bool
 */
function update_xero_option( $option_name, $option_value ) {
	if ( true === XERO_PLUGIN_NETWORK_ACTIVATED ) {
		return update_site_option( $option_name, $option_value );
	} else {
		return update_option( $option_name, $option_value );
	}
}

/**
 * Delete Option
 *
 * @param $option_name
 *
 * @return bool
 */
function delete_xero_option( $option_name ) {
	if ( true === XERO_PLUGIN_NETWORK_ACTIVATED ) {
		return delete_site_option( $option_name );
	} else {
		return delete_option( $option_name );
	}
}

/**
 * Add Xeroom Menu Items
 */
function xeroom_menu_icon_under_admin() {
	global $wpdb;
	$xeroLicActive     = $wpdb->prefix . "xeroom_license_key_status";
	$sql               = "SELECT * FROM " . $xeroLicActive . " WHERE id=1";
	$xeroLicensekeyAct = $wpdb->get_results( $sql );

	if ( $xeroLicensekeyAct ) {
		$active     = sanitize_text_field( $xeroLicensekeyAct[0]->status );
		$lic_key    = sanitize_text_field( $xeroLicensekeyAct[0]->license_key );
		$lic_method = sanitize_text_field( $xeroLicensekeyAct[0]->xero_method );

		add_menu_page( 'Add Xero API', 'Xeroom Setting', 'manage_options', 'add_xero_api_fields', 'xeroom_api_setting', XEROOM_IMAGE_PATH . 'xeroom_admin_menu_icon.png', 71 );
		add_submenu_page( 'add_xero_api_fields', 'Debug Page', 'Xeroom Debug Page', 'administrator', 'xeroom_debug_page', 'xeroom_debug_page' );
		if ( $active == 'active' && $lic_method != "trail" ) {
			add_submenu_page( 'add_xero_api_fields', 'Bulk Export to Xero', 'Bulk Export to Xero', 'administrator', 'xeroom_export_woo_xero', 'xeroom_export_setting' );
			add_submenu_page( 'add_xero_api_fields', 'Log Files', 'Log Files', 'administrator', 'xeroom_log_woo_xero', 'xeroom_log_setting' );
		}
	} else {
		add_action( 'admin_notices', function () {
			$class   = 'notice notice-error';
			$message = __( 'The Xeroom plugin install failed!', 'xeroom' );

			printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
		} );
	}
}

/**
 * Display Admin message if the Xeroom licence is not set or is not active
 */
function xeroom_display_message_if_licence_not_valid() {
	$currentScreen = get_current_screen();

	if ( 'toplevel_page_add_xero_api_fields' != $currentScreen->id ) {
		$active = get_xero_option( 'xero_connection_status' );
		if ( $active != 'active' ) {
			$licenseErrMessage = get_xero_option( 'xero_connection_status_message' );
			$license_status = get_xero_option( 'xero_license_status' );
			if ( ! empty( $licenseErrMessage ) && ! $license_status ) {
				add_action( 'admin_notices', function () use ( $licenseErrMessage ) {
					$class = 'notice notice-error';

					printf( '<div class="%1$s"><p><b>%2$s:</b> %3$s</p></div>', esc_attr( $class ), 'Xeroom', esc_html( $licenseErrMessage ) );
				} );
			}
		}
	}

	$fetch_tax_method = get_xero_option( 'xero_tax_method' );
	if ( $fetch_tax_method && 'xero_complex_tax' == $fetch_tax_method ) {
		$xero_taxes_association = get_xero_option( 'xero_taxes_association' );
		$xeroTime               = date( 'Y-m-d H:i:s' );
		if ( ! $xero_taxes_association ) {
			$message = __( 'Taxes have not been associated to Xero Tax Names. Please review your tax settings and enter the corresponding Xero Tax Names.', 'xeroom' );

			add_action( 'admin_notices', function () use ( $message ) {
				$class = 'notice notice-error';

				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
			} );

//			returnErrorMessageByXero( 0, $message, $xeroTime, '' );
		} else {
			$empty = false;
			foreach ( $xero_taxes_association as $ket => $val ) {
				if ( empty( $val ) ) {
					$empty = true;
					break;
				}
			}

			if ( $empty ) {
				$message = __( 'Some taxes have not been associated to Xero Tax Names. Please review your tax settings and enter the corresponding Xero Tax Names.', 'xeroom' );

				add_action( 'admin_notices', function () use ( $message ) {
					$class = 'notice notice-error';

					printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
				} );

//				returnErrorMessageByXero( 0, $message, $xeroTime, '' );
			}
		}
	}

}

/**
 * Display Xeroom API Settings
 */
function xeroom_api_setting() {
	require_once( XEROOM_ROOT_PATH . 'library/xeroom_api_setting.php' );
}

/**
 * Display Xeroom Debug Page
 */
function xeroom_debug_page() {
	require_once( XEROOM_ROOT_PATH . 'library/xeroom_debug_page.php' );
}

/**
 * Display Xeroom Export Page
 */
function xeroom_export_setting() {
	require_once( XEROOM_ROOT_PATH . 'library/Classes/PHPExcel.php' );
	require_once( XEROOM_ROOT_PATH . 'library/Classes/excel_reader.php' );
	require_once( XEROOM_ROOT_PATH . 'library/xeroom_export_import_functions.php' );
	require_once( XEROOM_ROOT_PATH . 'library/xeroom_export_setting.php' );
}

/**
 * Display Xeroom Logs Page
 */
function xeroom_log_setting() {
	require_once( XEROOM_ROOT_PATH . 'library/Classes/PHPExcel.php' );
	require_once( XEROOM_ROOT_PATH . 'library/xeroom_export_import_functions.php' );
	require_once( XEROOM_ROOT_PATH . 'library/xeroom_log_setting.php' );
}

/**
 * Xeroom Order Duplicate
 *
 * @param $order_id
 *
 * @return int
 */
function xeroom_order_duplicate( $order_id ) {
	global $wpdb;
	$xeroExportTable = $wpdb->prefix . 'xeroom_orders_export';
	$sql             = "SELECT `order_id` FROM `" . $xeroExportTable . "` WHERE `order_id` = " . $order_id;
	$ordersListArr   = $wpdb->get_results( $sql );
	if ( count( $ordersListArr ) > 0 ) {
		return 1;
	} else {
		return 0;
	}
}

add_action( 'wp_ajax_exportDataWooToXero', 'export_woo_to_xero_data' );
/**
 * Export WooCommerce data to Xero
 */
function export_woo_to_xero_data() {
	global $wpdb;

	$xeroExportTable = $wpdb->prefix . 'xeroom_orders_export';
	$xeroomLogExport = $wpdb->prefix . 'xeroom_export_log';
	$xeroom_orderId  = $_REQUEST['orderId'];
	$xeroom_Ignore   = $_REQUEST['ignore'];
	$xeroom_Payment  = $_REQUEST['payment'];
	$spin_time       = $_REQUEST['spinTime'];

	$response = "";
	if ( $spin_time == 1 ) {
		$wpdb->get_results( "TRUNCATE $xeroomLogExport" );
	}
	if ( $xeroom_Ignore == "No repeat" ) {
		if ( xeroom_order_duplicate( $xeroom_orderId ) == 0 ) {
			$response .= xeroom_sendWooInvoiceToXero( $xeroom_orderId, 'manually' );
			if ( $xeroom_Payment == 'Paid' ) {
				$response .= xeroom_paymentDoneOnCheckout( $xeroom_orderId, 'manually' );
			}
		} else {
			$response .= "Order id " . $xeroom_orderId . " already sent to xero account.";
		}
		if ( xeroom_order_duplicate( $xeroom_orderId ) == 0 ) {
			$wpdb->insert( $xeroExportTable, array( 'order_id' => intval( $xeroom_orderId ), 'status' => sanitize_text_field( $xeroom_Payment ) ) );
		}
		$wpdb->insert( $xeroomLogExport,
			array(
				'order_id'     => intval( $xeroom_orderId ),
				'payment_type' => sanitize_text_field( $xeroom_Payment ),
				'ignore_type'  => sanitize_text_field( $xeroom_Ignore ),
				'status'       => sanitize_text_field( $response )
			) );
		wp_die();
	} else if ( $xeroom_Ignore == "Yes repeat" ) {
		$response .= xeroom_sendWooInvoiceToXero( $xeroom_orderId, 'manually' );
		if ( $xeroom_Payment == 'Paid' ) {
			$response .= xeroom_paymentDoneOnCheckout( $xeroom_orderId, 'manually' );
		}
		if ( xeroom_order_duplicate( $xeroom_orderId ) == 0 ) {
			$wpdb->insert( $xeroExportTable, array( 'order_id' => intval( $xeroom_orderId ), 'status' => sanitize_text_field( $xeroom_Payment ) ) );
		}
		$wpdb->insert( $xeroomLogExport,
			array(
				'order_id'     => intval( $xeroom_orderId ),
				'payment_type' => sanitize_text_field( $xeroom_Payment ),
				'ignore_type'  => sanitize_text_field( $xeroom_Ignore ),
				'status'       => sanitize_text_field( $response )
			) );
		wp_die();
	} else {
		$wpdb->insert( $xeroomLogExport,
			array(
				'order_id'     => intval( $xeroom_orderId ),
				'payment_type' => sanitize_text_field( $xeroom_Payment ),
				'ignore_type'  => sanitize_text_field( $xeroom_Ignore ),
				'status'       => sanitize_text_field( 'Ignore type is wrong, it must be "Yes repeat" or "No repeat".' )
			) );
		wp_die();
	}
}

add_action( 'wp_ajax_unlinkxeroomlog', 'unlink_xeroom_log_files' );
/**
 * Unlink Log Files
 */
function unlink_xeroom_log_files() {
	unlink( XEROOM_ROOT_PATH . 'library/log/' . $_REQUEST['fileName'] );
}

add_action( 'wp_ajax_unlinkxeroomexcel', 'unlink_xeroom_excel_files' );
/**
 * Unlink Excel Files
 */
function unlink_xeroom_excel_files() {
	unlink( XEROOM_EXCEL_PATH . 'upload_this_data.xlsx' );
}

/**
 * Create Xeroom tables and add Default Settings on plugin activation
 */
function xeroom_master_install() {
	global $wpdb;
	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	$xeroDebugTable  = $wpdb->prefix . 'xeroom_debug';
	$xeroTaxTable    = $wpdb->prefix . 'xeroom_tax';
	$xeroTable       = $wpdb->prefix . 'xeroom_credentials';
	$licenseTable    = $wpdb->prefix . 'xeroom_license_key_status';
	$xeroomOrderExp  = $wpdb->prefix . 'xeroom_orders_export';
	$xeroomLogExport = $wpdb->prefix . 'xeroom_export_log';

	$query         = "SELECT count(*) as tblExist FROM information_schema.TABLES WHERE (TABLE_SCHEMA = '" . DB_NAME . "') AND (TABLE_NAME = '" . $xeroTable . "')";
	$getTableCount = $wpdb->get_results( $query );
	$CreateTabele  = $getTableCount[0]->tblExist;

	if ( $CreateTabele == 0 ) {
		$wpdb->query( "DROP TABLE IF EXISTS $xeroDebugTable" );
		$wpdb->query( "DROP TABLE IF EXISTS $xeroTaxTable" );
		$wpdb->query( "DROP TABLE IF EXISTS $xeroTable" );
		$wpdb->query( "DROP TABLE IF EXISTS $licenseTable" );
		$wpdb->query( "DROP TABLE IF EXISTS $xeroomOrderExp" );
		$wpdb->query( "DROP TABLE IF EXISTS $xeroomLogExport" );

		$xeroDebugQuery = "CREATE TABLE IF NOT EXISTS $xeroDebugTable(
							`id` int(11) NOT NULL AUTO_INCREMENT,
							`order_id` int(11) NOT NULL,
							`debug` text NOT NULL,
							`created_date` datetime NOT NULL,
							PRIMARY KEY (`id`)
							)";
		dbDelta( $xeroDebugQuery );

		$xeroTaxQuery = "CREATE TABLE IF NOT EXISTS $xeroTaxTable(
							  `id` int(11) NOT NULL AUTO_INCREMENT,
							  `tax_name` varchar(255) NOT NULL,
							  `tax_rate` varchar(255) NOT NULL,
							  `tax_type` varchar(255) NOT NULL,
							  PRIMARY KEY (`id`)
							)";
		dbDelta( $xeroTaxQuery );

		$xeroQuery = "CREATE TABLE IF NOT EXISTS $xeroTable (
						`id` int(11) NOT NULL AUTO_INCREMENT,
						`xero_api_key` varchar(255) NOT NULL,
						`xero_api_secret` varchar(255) NOT NULL,
						`sales_account` varchar(255) NOT NULL,
						`bank_code` varchar(255) NOT NULL,
						`tax_code` varchar(255) NOT NULL,
						`asset_code` varchar(255) NOT NULL,
						`sold_code` varchar(255) NOT NULL,
						`stock_master` varchar(255) NOT NULL,
						`product_master` varchar(255) NOT NULL,
						`payment_master` varchar(255) NOT NULL,
						 PRIMARY KEY (`id`)
						)";
		dbDelta( $xeroQuery );

		$xeroomExpQuery = "CREATE TABLE IF NOT EXISTS $xeroomOrderExp (
						  `id` int(11) NOT NULL AUTO_INCREMENT,
						  `order_id` int(11) NOT NULL,
						  `status` varchar(255) NOT NULL,
							PRIMARY KEY (`id`)
						)";
		dbDelta( $xeroomExpQuery );

		$xeroomLogQuery = "CREATE TABLE IF NOT EXISTS $xeroomLogExport (
							`id` int(11) NOT NULL AUTO_INCREMENT,
							`order_id` int(11) NOT NULL,
							`payment_type` varchar(255) NOT NULL,
							`ignore_type` varchar(255) NOT NULL,
							`status` text NOT NULL,
							PRIMARY KEY (`id`)
						)";
		dbDelta( $xeroomLogQuery );

		$licenseQuery = "CREATE TABLE IF NOT EXISTS $licenseTable (
						  `id` int(11) NOT NULL AUTO_INCREMENT,
						  `license_key` varchar(255) NOT NULL,
						  `status` varchar(255) NOT NULL,
						  `xero_method` varchar(255) NOT NULL,
							PRIMARY KEY (`id`)
						)";
		dbDelta( $licenseQuery );

		if ( empty( $wpdb->last_error ) ) {
			$wpdb->insert( $xeroTable,
				array(
					'xero_api_key'    => "",
					'xero_api_secret' => "",
					'sales_account'   => 200,
					'bank_code'       => 100,
					'tax_code'        => '20% (VAT on Income)',
					'asset_code'      => '630',
					'sold_code'       => '310',
					'stock_master'    => 'w',
					'product_master'  => 'w',
					'payment_master'  => 'n'
				)
			);
			$wpdb->insert( $licenseTable,
				array(
					'license_key' => '123456789',
					'status'      => "deactive",
					'xero_method' => "trail"
				)
			);
		} else {
			add_action( 'admin_notices', function () {
				$class   = 'notice notice-error';
				$message = __( 'The Xeroom installation failed! Please contact Xeroom support for assistance.', 'xeroom' );

				printf( '<div class="%1$s"><p>%2$s</p></div>', esc_attr( $class ), esc_html( $message ) );
			} );
		}
	}

	if ( ! get_xero_option( 'xero_default_invoice_status' ) ) {
		update_xero_option( 'xero_default_invoice_status', 'AUTHORISED' );
	}
	if ( ! get_xero_option( 'xero_use_extra_sales_account' ) ) {
		update_xero_option( 'xero_use_extra_sales_account', 'none' );
	}
	if ( ! get_xero_option( 'xero_generate_credit_note' ) ) {
		update_xero_option( 'xero_generate_credit_note', 0 );
	}
	if ( ! get_xero_option( 'xero_credit_note_status' ) ) {
		update_xero_option( 'xero_credit_note_status', 0 );
	}
	if ( ! get_xero_option( 'xero_tax_method' ) ) {
		update_xero_option( 'xero_tax_method', 'xero_simple_tax' );
	}
	if ( ! get_xero_option( 'xero_invoice_date' ) ) {
		update_xero_option( 'xero_invoice_date', 'order_date' );
	}
	if ( ! get_xero_option( 'xero_send_invoice_method' ) ) {
		update_xero_option( 'xero_send_invoice_method', 'manually' );
	}
	if ( ! get_xero_option( 'xero_set_invoice_duedate' ) ) {
		update_xero_option( 'xero_set_invoice_duedate', 'use_woo_due_date' );
	}
	if ( ! get_xero_option( 'xero_send_payment_method' ) ) {
		update_xero_option( 'xero_send_payment_method', 'manually' );
	}
	if ( ! get_xero_option( 'xero_contact_name' ) ) {
		update_xero_option( 'xero_contact_name', 'xeroom_use_first_name' );
	}
	if ( ! get_xero_option( 'xero_address_info' ) ) {
		update_xero_option( 'xero_address_info', 'xeroom_use_woo_address' );
	}
}

/**
 * Return Xero Error Message
 *
 * @param $order_id
 * @param $errD
 * @param $xeroTime
 * @param $errorType
 */
function returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType ) {
	if ( $errorType == "manually" ) {
		return;
	} else {
		global $wpdb;
		$recordErrorArray = "";
		$xeroDebugTable   = $wpdb->prefix . 'xeroom_debug';

		if ( strpos( $errD, 'oauth_problem' ) !== false ) {
			if ( is_array( $errD ) ) {
				$errD = 'Error while connecting to Xero API. Technical details: ' . print_r( $errD, true );
			} else {
				$errD = 'Error while connecting to Xero API. Technical details: ' . $errD;
			}
			update_xero_option( 'xero_connection_status_message', $errD );
			update_xero_option( 'xero_connection_status', 'failed' );
		}

		$wpdb->insert( $xeroDebugTable,
			array(
				'order_id'     => intval( $order_id ),
				'debug'        => sanitize_text_field( $errD ),
				'created_date' => $xeroTime
			)
		);
	}
}

/**
 * Send data to Xero after successfully checkout
 *
 * @param $order_id
 * @param string $errorType
 *
 * @return string
 */
function xeroom_sendWooInvoiceToXero( $order_id, $errorType = "orderProduct" ) {
	global $wpdb;
	global $product;
	global $order;
	$oId      = intval( $order_id );
	$order    = new WC_Order( $oId );
	$country  = new WC_Countries();
	$orderTax = new WC_Tax();

	//define Tables
	$exportErrorXero = $xero_send_error = "";
	$xeroDebugTable  = $wpdb->prefix . 'xeroom_debug';
	$xeroLicActive   = $wpdb->prefix . "xeroom_license_key_status";
	$taxTableName    = $wpdb->prefix . "xeroom_tax";
	$xeroTime        = date( 'Y-m-d H:i:s' );

	//License Key
	$sql               = "SELECT * FROM " . $xeroLicActive . " WHERE id=1";
	$xeroLicensekeyAct = $wpdb->get_results( $sql );

	if ( $xeroLicensekeyAct ) {
		$active     = sanitize_text_field( $xeroLicensekeyAct[0]->status );
		$lic_key    = sanitize_text_field( $xeroLicensekeyAct[0]->license_key );
		$lic_method = sanitize_text_field( $xeroLicensekeyAct[0]->xero_method );

		$license_checked = get_xero_option( 'xero_license_status' );

		// Invoice Status
		$default_invoice_status = 'AUTHORISED';
		$get_xero_status_option = get_xero_option( 'xero_default_invoice_status' );
		if ( $get_xero_status_option ) {
			$default_invoice_status = esc_attr( $get_xero_status_option );
		}

		//Check License of plugin
		if ( $active == 'active' ) {
//			$url          = "https://www.xeroom.com/apidata.php?license_key=" . $lic_key;
//			$result       = wp_remote_fopen( esc_url( $url ) );
//			$validLicense = json_decode( trim( $result ), true );

			if ( $license_checked != 'expired' ) {
				//Items Details
				$totalItems    = $order->get_item_count();
				$orderCurency  = $order->get_currency();
				$usedCupons    = "Coupon used " . implode( ", ", $order->get_coupon_codes() );
				$shippingPrice = $order->calculate_shipping();
				$totalDiscount = $order->get_total_discount();

				// Prices include tax amount
				$included_tax = false;
				if ( 'yes' === get_option( 'woocommerce_prices_include_tax' ) && wc_tax_enabled() ) {
					$included_tax = true;
				}

				// User Address
				$shipAddress     = apply_filters( 'xeroom_shipping_address', $order->get_address(), $order_id, 'invoice' );
				$allCountry      = $country->get_countries();
				$countryFullName = $allCountry[ $shipAddress['country'] ];
				$allState        = $country->get_states( $shipAddress['country'] );
				$stateFullName   = '';
				if ( $allState ) {
					$stateFullName = $allState[ $shipAddress['state'] ];
				}

				// Tax Rate Used
				$getTaxRate     = array(
					'country'   => $shipAddress['country'],
					'state'     => $shipAddress['state'],
					'city'      => $shipAddress['city'],
					'postcode'  => $shipAddress['postcode'],
					'tax_class' => ''
				);
				$taxRatePercent = $orderTax->find_rates( $getTaxRate );
				if ( count( $taxRatePercent ) != 0 && wc_tax_enabled() ) {
					foreach ( $taxRatePercent as $taxKey => $taxValue ) {
						$taxValue = $taxValue;
					}
					$taxOnWholeCart = $taxValue;
				} else {
					$taxOnWholeCart = array();
				}

				// Xero Connectivity Api Credentials------------
				$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
				$getApiCredentials = $wpdb->get_results( $query );

				if ( $getApiCredentials ) {
					$xeroApiKey    = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
					$xeroApiSecret = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
					$salesAccount  = sanitize_text_field( $getApiCredentials[0]->sales_account );
					$BankCode      = sanitize_text_field( $getApiCredentials[0]->bank_code );
					$TaxCode       = sanitize_text_field( $getApiCredentials[0]->tax_code );
					$AssetCode     = sanitize_text_field( $getApiCredentials[0]->asset_code );
					$SoldCode      = sanitize_text_field( $getApiCredentials[0]->sold_code );
					$StockMaster   = sanitize_text_field( $getApiCredentials[0]->stock_master );
					$productMaster = sanitize_text_field( $getApiCredentials[0]->product_master );

					$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );
					if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
						$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
					}

					if ( 'manually-resend-invoice' != $errorType ) {
						if ( xero_invoice_exists( $order_id, $xeroApiKey, $xeroApiSecret ) && 'manually' != $errorType ) {
							return;
						}
					}

					/**
					 * Get shipping cost code
					 */
					$shippingCode = get_xero_option( 'xero_default_shipping_costs_code' );

					/**
					 * Get Use Extra Sales Accounts - Shipping
					 */
					$use_extra_sales_accounts = get_xero_option( 'xero_use_extra_sales_account' );
					if ( $order->get_shipping_methods() && $use_extra_sales_accounts && 'geography_zones' == $use_extra_sales_accounts ) {
						$zone_id = xero_fetch_order_zone_id( $order );
						if ( ! empty( $zone_id ) ) {
							$shipping_associated_code = xeroom_invoice_geography_zone( $zone_id );
							if ( ! empty( $shipping_associated_code ) ) {
								$salesAccount = $shipping_associated_code;
							}
						}
					}

					include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
					if ( ! defined( 'BASE_PATH' ) ) {
						define( 'BASE_PATH', dirname( __FILE__ ) );
					}

					if ( ! defined( 'XERO_KEY' ) ) {
						define( 'XERO_KEY', $xeroApiKey );
					}

					if ( ! defined( 'XERO_SECRET' ) ) {
						define( 'XERO_SECRET', $xeroApiSecret );
					}

					if ( ! defined( 'PUBLIC_KEY' ) ) {
						define( 'PUBLIC_KEY', BASE_PATH . '/library/certs/publickey.cer' );
					}

					if ( ! defined( 'PRIVATE_KEY' ) ) {
						define( 'PRIVATE_KEY', BASE_PATH . '/library/certs/privatekey.pem' );
					}

					if ( ! defined( 'FORMAT' ) ) {
						define( 'FORMAT', 'json' );
					}
					$oauth2 = get_xero_option( 'xero_oauth_options' );
					$xero   = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );

					$xero_tracking = $xero->TrackingCategories();
					xeroom_check_connection_message( $xero_tracking, $order_id );

					$active_tracking = array();
					if ( is_array( $active_tracking ) ) {
						$active_tracking = xeroom_fetch_tracking_categories( $xero_tracking );
					}
					$country_name = WC()->countries->countries[ $order->get_billing_country() ];

					$add_tracking      = $tracking_category_id = '';
					$tracking_category = get_xero_option( 'xero_tracking_category' );
					if ( ! empty( $tracking_category ) ) {
						if ( is_array( $active_tracking ) && array_key_exists( esc_attr( $tracking_category ), $active_tracking ) && in_array( $country_name, $active_tracking[ $tracking_category ] ) ) {
							$add_tracking = $country_name;
						} else {
							$add_tracking = apply_filters( 'xeroom_not_tracking_name', __( 'Not tracked' ) );
						}
					}

					// Xero Connectivity Api Credentials------------
					$ordersList   = $order->get_items();
					$orderDetails = array();
					$oKeyValue    = array();
					$countOrder   = 0;

					$fetch_tax_method           = get_xero_option( 'xero_tax_method' );
					$fetch_saved_tax            = get_xero_option( 'xero_tax_methods' );
					$fetch_taxes_association    = get_xero_option( 'xero_taxes_association' );
					$invoice_prefix             = get_xero_option( 'xero_invoice_prefix' );
					$invoice_reference          = get_xero_option( 'xero_invoice_reference_prefix' );
					$shipping_price_code        = get_xero_option( 'xero_shipping_price_code' );
					$shipping_price_description = get_xero_option( 'xero_shipping_price_description' );

					$xero_shipping_price_code = 'shipping_price';
					if ( $shipping_price_code ) {
						$xero_shipping_price_code = esc_attr( $shipping_price_code );
					}

					$xero_shipping_price_description = 'Shipping Price';
					if ( $shipping_price_description ) {
						$xero_shipping_price_description = esc_attr( $shipping_price_description );
					}

					$product_tax        = 'standard';
					$item_tax_rate_id   = '';
					$item_tax_rate_rate = 0;
					$products_ids       = array();
					foreach ( $ordersList as $singleorderskey => $singleorders ) {
						$orderDetails[ $countOrder ]['item_price']      = $order->get_item_total( $singleorders, false, true );
						$orderDetails[ $countOrder ]['item_sale_price'] = $order->get_item_subtotal( $singleorders, false, true );
						$orderDetails[ $countOrder ]['name']            = $singleorders['name'];
						$orderDetails[ $countOrder ]['_product_id']     = $singleorders['product_id'];
						$orderDetails[ $countOrder ]['_variation_id']   = $singleorders['variation_id'];
						$orderDetails[ $countOrder ]['_qty']            = $singleorders['quantity'];

						$products_ids[ $singleorders['product_id'] ] = $singleorders['quantity'];

						if ( 0 != $totalDiscount ) {
							$orderDetails[ $countOrder ]['get_subtotal_tax'] = round( $singleorders->get_subtotal_tax(), wc_get_price_decimals() );
						}

						// Fetch Order Item Tax
						if ( $fetch_tax_method && wc_tax_enabled() ) {
							$simple_tax = get_post_meta( $singleorders['product_id'], '_tax_class', true );

							if ( !empty( $simple_tax ) ) {
								$product_tax = $simple_tax;
							} else {
                                $product_tax        = 'standard';
                            }
                            
							switch ( $fetch_tax_method ) {
								case "xero_simple_tax" :
									if ( $fetch_saved_tax && is_array( $fetch_saved_tax ) && array_key_exists( 'xero_' . $product_tax . '_taxmethods', $fetch_saved_tax ) ) {
										$orderDetails[ $countOrder ]['tax_class']     = xero_tax_type_code( $fetch_saved_tax[ 'xero_' . $product_tax . '_taxmethods' ] );
										$orderDetails[ $countOrder ]['tax_class_new'] = xero_tax_type_code( $fetch_saved_tax[ 'xero_' . $product_tax . '_taxmethods' ] );
										$orderDetails[ $countOrder ]['total_tax']     = xero_fetch_tax_complex_amount( $singleorders->get_taxes() );
									}
									break;
								case "xero_complex_tax" :
									$tax_data = $singleorders->get_taxes();
									foreach ( $order->get_taxes() as $tax_item ) {
										$rate_id        = $tax_item->get_rate_id();
										$item_total_tax = $tax_data['total'][ $rate_id ];
                                                                                                       
										if ( isset( $item_total_tax ) && '' !== $item_total_tax ) {
											$item_tax_rate_id = $rate_id;
										}
									}

									if ( $fetch_taxes_association && is_array( $fetch_taxes_association ) && array_key_exists( $item_tax_rate_id, $fetch_taxes_association ) ) {
										$orderDetails[ $countOrder ]['tax_class']     = xero_tax_type_code( $fetch_taxes_association[ $item_tax_rate_id ] );
										$orderDetails[ $countOrder ]['tax_class_new'] = xero_tax_type_code( $fetch_taxes_association[ $item_tax_rate_id ] );
										$orderDetails[ $countOrder ]['total_tax']     = xero_fetch_tax_complex_amount( $singleorders->get_taxes() );
									}
									break;
							}
						}
                        
						if ( 'w' == $productMaster ) {
							if ( wc_tax_enabled() ) {
								if( ! $fetch_tax_method ) {
									$tax_data         = $singleorders->get_taxes();
									$item_tax_rate_id = $tax_name = '';
									foreach ( $order->get_taxes() as $tax_item ) {
										$rate_id = $tax_item->get_rate_id();

										if ( ! empty( $tax_data['total'][ $rate_id ] ) ) {
											$item_tax_rate_rate = $tax_data['total'][ $rate_id ];
											$tax_name           = xero_fetch_tax_rate_name( $rate_id );
											if ( isset( $item_tax_rate_rate ) ) {
												$item_tax_rate_id = xero_tax_type_code( $tax_name );
											}
										}
									}

									if ( ! empty( $item_tax_rate_id ) ) {
										$orderDetails[ $countOrder ]['tax_class'] = $item_tax_rate_id;
										if ( ! empty( $item_tax_rate_id ) ) {
											$orderDetails[ $countOrder ]['total_tax'] = ! empty( $item_tax_rate_rate ) ? $item_tax_rate_rate : 0;
										}
									}
								}
							} else {
								$orderDetails[ $countOrder ]['tax_class'] = 'NONE';
								$orderDetails[ $countOrder ]['total_tax'] = 0;
							}
						}

						$countOrder ++;
					}
                    
					$oderList  = array();
					$uniqueTax = "";
					// Create new tax here
					if ( count( $taxOnWholeCart ) != 0 ) {
						$getAllTax = $xero->TaxRates();
						xeroom_check_connection_message( $getAllTax, $order_id );

						$taxRate = $taxOnWholeCart['rate'];
						if ( isset( $getAllTax['TaxRates']['TaxRate'] ) ) {
							foreach ( $getAllTax['TaxRates']['TaxRate'] as $allTaxesNow ) {
								if ( $allTaxesNow['Name'] == $TaxCode ) {
									$uniqueTax      = $allTaxesNow['TaxType'];
									$displayTaxRate = $allTaxesNow['DisplayTaxRate'];
									break;
								}
							}
						}

						if ( $uniqueTax == "" ) {
							foreach ( $getAllTax['TaxRates']['TaxRate'] as $allTaxesNow ) {
								if ( $allTaxesNow['TaxComponents']['TaxComponent']['Rate'] == $taxRate ) {
									$uniqueTax      = $allTaxesNow['TaxType'];
									$displayTaxRate = $allTaxesNow['DisplayTaxRate'];
									break;
								}
							}
						}
						if ( $uniqueTax == "" ) {
							$taxName     = "Default Xeroom Sales Tax";
							$query       = "SELECT * FROM `" . $taxTableName . "` WHERE `tax_rate` ='" . $taxRate . "'";
							$getTaxRates = $wpdb->get_results( $query );
							if ( count( $getTaxRates ) > 0 ) {
								$uniqueTax = $getTaxRates[0]->tax_type;
							} else {
								$xero_rate = array(
									"TaxRate" => array(
										"Name"          => "$taxName",
										"ReportTaxType" => 'OUTPUT',
										"TaxComponents" => array(
											"TaxComponent" => array(
												"Name" => "VAT",
												"Rate" => $taxRate
											)
										)
									)
								);
								$taxResult = $xero->TaxRates( $xero_rate );

								if ( ! empty( $taxResult ) ) {
									if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
										$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
										returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
										$exportErrorXero .= $errD;
									} else if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
										$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
										for ( $er = 0; $er < count( $errD ); $er ++ ) {
											$errorMessage = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $er ]['Message'];
											returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
											$exportErrorXero .= $errorMessage;
										}
									} else if ( $taxResult['Status'] == "OK" ) {
										$uniqueTax = $taxResult['TaxRates']['TaxRate']['TaxType'];
										$wpdb->insert( $taxTableName,
											array(
												'tax_name' => sanitize_text_field( $taxResult['TaxRates']['TaxRate']['Name'] ),
												'tax_rate' => sanitize_text_field( $taxRate ),
												'tax_type' => sanitize_text_field( $taxResult['TaxRates']['TaxRate']['TaxType'] )
											)
										);
										update_xero_option( 'xero_connection_status', 'active' );
										update_xero_option( 'xero_connection_status_message', '' );
									} else {
										returnErrorMessageByXero( $order_id, $taxResult, $xeroTime, $errorType );
										$exportErrorXero .= $taxResult;
									}
								}
							}
						}
					}
					// End Create new tax here

					for ( $i = 0; $i < count( $orderDetails ); $i ++ ) {

						/**
						 * Add product to associated category
						 */
						if ( $use_extra_sales_accounts && 'product_categories' == $use_extra_sales_accounts ) {
							$product_categories = fetch_product_category_id( $orderDetails[ $i ]['_product_id'] );

							$category_associated_account = xeroom_product_category_associated( $product_categories );
							if ( ! empty( $category_associated_account ) ) {
								$salesAccount = $category_associated_account;
							}
						}

						$allProductAttr = "";
						if ( isset( $orderDetails[ $i ]['_variation_id'] ) && $orderDetails[ $i ]['_variation_id'] != 0 ) {
							$unitPrice = get_post_meta( $orderDetails[ $i ]['_variation_id'] );
							$itemId    = $orderDetails[ $i ]['_variation_id'];
							$product   = wc_get_product( $orderDetails[ $i ]['_variation_id'] );

							if ( ! empty( $product->get_attribute_summary() ) ) {
								$allProductAttr .= ', ' . $product->get_attribute_summary();
							}

							if ( empty( $product->get_attribute_summary() ) ) {
								$productAttributes = get_post_meta( $orderDetails[ $i ]['_product_id'], '_product_attributes' );

								if ( $productAttributes ) {
									foreach ( $productAttributes[0] as $key => $value ) {
										$attributeName  = get_taxonomy( $value['name'] );
										$attributeName  = $attributeName->labels->name;
										$slugValue      = get_term_by( 'slug', $attributeName->name, $value['name'] );
										$allProductAttr .= ', ' . $attributeName . ': ' . $slugValue->name;
									}


								}
							}

						} else {
							$unitPrice = get_post_meta( $orderDetails[ $i ]['_product_id'] );
							$itemId    = $orderDetails[ $i ]['_product_id'];
//							$product   = new WC_Product( $orderDetails[ $i ]['_product_id'] );
							$product = wc_get_product( $orderDetails[ $i ]['_product_id'] );
						}

						if ( $unitPrice['_sku'][0] != "" ) {
							$sku = $unitPrice['_sku'][0];
						} else {
							$sku = $orderDetails[ $i ]['_product_id'];
						}
						if ( isset( $unitPrice['_sale_price'][0] ) && $unitPrice['_sale_price'][0] != "" ) {
							$unitPriceNow = $unitPrice['_sale_price'][0];
							if ( $allProductAttr == "" ) {
								$allProductAttr = ', Full Price: ' . $unitPrice['_regular_price'][0];
							} else {
								$allProductAttr .= ', Full Price: ' . $unitPrice['_regular_price'][0];
							}
						} else {
							$unitPriceNow = $unitPrice['_regular_price'][0];
						}

						$sku = xeroom_reduce_sku_length( $sku );

						// Get item by sku
						$checkwithoutStock = $xero->Items( $sku );
						xeroom_check_connection_message( $checkwithoutStock, $order_id );

						$unitPriceNow = $orderDetails[ $i ]['item_price'];
//						$unitPriceNow = $product->get_price_excluding_tax();
						$unitPriceNow = number_format( (float) $unitPriceNow, 2, '.', '' );
						if ( $allProductAttr != "" ) {
							$descriptionName = $orderDetails[ $i ]['name'] . $allProductAttr;
							if ( $productMaster == "w" ) {
								$oderList[ $i ]['Description'] = $orderDetails[ $i ]['name'] . $allProductAttr;
							}
						} else {
							$descriptionName = $orderDetails[ $i ]['name'];
							if ( $productMaster == "w" ) {
								$oderList[ $i ]['Description'] = $orderDetails[ $i ]['name'];
							}
						}

						$oderList[ $i ]['Quantity'] = $orderDetails[ $i ]['_qty'];
						if ( $productMaster == "w" || $productMaster == "n" ) {
							$oderList[ $i ]['UnitAmount'] = $unitPriceNow;
						}
						$oderList[ $i ]['ItemCode'] = $sku;

						if ( $uniqueTax != "" ) {
							$oderList[ $i ]['TaxType'] = $uniqueTax;
						}

						// Add tax Rate name
						if ( '' != $orderDetails[ $i ]['tax_class'] ) {
							$oderList[ $i ]['TaxType'] = $orderDetails[ $i ]['tax_class'];
							if ( $productMaster == "w" ) {
								$oderList[ $i ]['TaxAmount'] = $orderDetails[ $i ]['total_tax'];
							}
						}

						// Apply discount rate.
						if ( 0 != $totalDiscount ) {
							$percentage = 0;
							foreach ( $order->get_coupon_codes() as $coupon_name ) {
								// Retrieving the coupon ID
								$coupon_post_obj = get_page_by_title( $coupon_name, OBJECT, 'shop_coupon' );
								$coupon_id       = $coupon_post_obj->ID;
								$coupons_obj     = new WC_Coupon( $coupon_id );

								if ( $coupons_obj->get_discount_type() == 'percent' ) {
									$item_subtotal = number_format( (float) $orderDetails[ $i ]['item_sale_price'], 2, '.', '' );
									$percentage    += round( ( ( $item_subtotal - number_format( (float) $orderDetails[ $i ]['item_price'], 2, '.', '' ) ) / $item_subtotal ) * 100 );
								} else {
									$oderList[ $i ]['TaxAmount'] = $orderDetails[ $i ]['get_subtotal_tax'];
								}

							}

							if ( $percentage > 0 ) {
								$oderList[ $i ]['DiscountRate'] = $percentage;
							}

							$unitPriceNow = $orderDetails[ $i ]['item_sale_price'];
							if ( $productMaster == "w" || $productMaster == "n" ) {
								$oderList[ $i ]['UnitAmount'] = $unitPriceNow;
							}
						}

						// Add tracking
						if ( ! empty( $add_tracking ) ) {
							$oderList[ $i ]['Tracking'] = array(
								'TrackingCategory' => array(
									'Name'   => esc_attr( $tracking_category ),
									'Option' => $add_tracking,
								),
							);
						}

						$oderList[ $i ]['AccountCode'] = $salesAccount;
						$post_details                  = get_post( $orderDetails[ $i ]['_product_id'] );
						$xItemName                     = substr( $post_details->post_title, 0, 48 );
						if ( $xItemName == "" ) {
							$xItemName = "No Name_" . rand();
						}
						$xItemDesc  = $post_details->post_content;
						$xItemStock = $unitPrice['_stock'][0];

						/* Stock Master */
						if ( $StockMaster == "w" ) {
							if ( $xItemStock == 0 || $xItemStock == "" ) {
								if ( isset( $checkwithoutStock['Items']['Item']['QuantityOnHand'] ) ) {
									$addNewQuant = $orderDetails[ $i ]['_qty'];
//									$increaseItems = array(
//										"Invoice" => array(
//											"Type"            => "ACCPAY",
//											"Contact"         => array(
//												"Name" => "Inventory Adjustments"
//											),
//											"Date"            => date( 'Y-m-d' ),
//											"DueDate"         => date( 'Y-m-d' ),
//											"LineAmountTypes" => "NoTax",
//											"Status"          => $default_invoice_status,
//											"LineItems"       => array(
//												"LineItem" => array(
//													"ItemCode"    => $sku,
//													"Description" => $descriptionName,
//													"Quantity"    => $addNewQuant,
//													"UnitAmount"  => $unitPriceNow,
//													"AccountCode" => $AssetCode,
//												)
//											)
//										)
//									);
//
//									// not overwriting Xero data
//									if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
//										$xero_desc = '-';
//										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
//											$xero_desc = xero_product_description( $checkwithoutStock );
//										}
//										$increaseItems['Invoice']['LineItems']['LineItem']['Description'] = $xero_desc;
//
//										$increaseItems['Invoice']['LineItems']['LineItem']['UnitAmount'] = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
//									}
//
//									$increaseWithoutStocktest = $xero->Invoices( $increaseItems );
//									if ( isset( $increaseWithoutStocktest['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
//										$errD = $increaseWithoutStocktest['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
//										returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
//										$exportErrorXero .= $errD;
//									} else if ( isset( $increaseWithoutStocktest['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
//										$errD = array();
//										$errD = $increaseWithoutStocktest['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
//										for ( $e = 0; $e < count( $errD ); $e ++ ) {
//											$errorMessage = $increaseWithoutStocktest['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
//											returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
//											$exportErrorXero .= $errorMessage;
//										}
//									} else {
//										update_xero_option( 'xero_connection_status', 'active' );
//										update_xero_option( 'xero_connection_status_message', '' );
//                                    }
								} else {
//									$createItemWithoutStock = array(
//										"Item"            => array(
//											"Code"        => $sku,
//											"Name"        => $xItemName,
//											"Description" => $descriptionName,
//										),
//										"SalesDetails"    => array(
//											"UnitPrice"   => $unitPriceNow,
//											"AccountCode" => $salesAccount
//										),
//										"PurchaseDetails" => array(
//											"UnitPrice"       => $unitPriceNow,
//											"COGSAccountCode" => $SoldCode
//										)
//									);
									$createItemWithoutStock = array(
										"Item" => array(
											"Name"                => $xItemName,
											"Code"                => $sku,
											"Description"         => $descriptionName,
											"PurchaseDescription" => $descriptionName,
											"SalesDetails"        => array(
												"UnitPrice"   => $unitPriceNow,
												"AccountCode" => $salesAccount,
												"TaxType"     => $orderDetails[ $i ]['tax_class_new']
											),
											"PurchaseDetails"     => array(
												"UnitPrice"   => $unitPriceNow,
												"AccountCode" => $SoldCode
											)
										)
									);
									if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
										$xero_desc = $descriptionName;
										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
											$xero_desc = xero_product_description( $checkwithoutStock );
										}
										$createItemWithoutStock['Item']['Description'] = $xero_desc;
										$createItemWithoutStock['Item']['Name']        = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;

										$createItemWithoutStock['Item']['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
										$createItemWithoutStock['Item']['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
									} else {
										if ( $productMaster == "w" ) {
											$createItemWithoutStock['Item']['PurchaseDetails']['UnitPrice'] = 0;
										}
									}

									$withoutStockXero = $xero->Items( $createItemWithoutStock );
									if ( isset( $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
										$errD = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
										returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
										$exportErrorXero .= $errD;
									} else if ( isset( $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
										$errD = array();
										$errD = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
										for ( $e = 0; $e < count( $errD ); $e ++ ) {
											$errorMessage = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
											returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
											$exportErrorXero .= $errorMessage;
										}
									} else {
										update_xero_option( 'xero_connection_status', 'active' );
										update_xero_option( 'xero_connection_status_message', '' );
									}

									$oderList[ $i ]['Description'] = $descriptionName;
									if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
										$xero_desc = '-';
										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
											$xero_desc = xero_product_description( $checkwithoutStock );
										}
										$oderList[ $i ]['Description'] = $xero_desc;
										$oderList[ $i ]['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									}
								}
							} else {
								$newItemsCode = array(
									"Item" => array(
										"Code"                      => $sku,
										"Name"                      => $xItemName,
										"Description"               => $descriptionName,
										"PurchaseDescription"       => $xItemName,
										"IsTrackedAsInventory"      => true,
										"IsSold"                    => true,
										"IsPurchased"               => true,
										"QuantityOnHand"            => $xItemStock,
										"TotalCostPool"             => $xItemStock * $unitPriceNow,
										"InventoryAssetAccountCode" => $AssetCode,
										"SalesDetails"              => array(
											"UnitPrice"   => $unitPriceNow,
											"AccountCode" => $salesAccount,
											"TaxType"     => $orderDetails[ $i ]['tax_class_new']
										),
										"PurchaseDetails"           => array(
											"UnitPrice"       => $unitPriceNow,
											"COGSAccountCode" => $SoldCode
										)
									)
								);

								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$newItemsCode['Item']['Description']                  = $xero_desc;
									$newItemsCode['Item']['PurchaseDescription']          = '' != xero_product_purchase_description( $checkwithoutStock ) ? xero_product_purchase_description( $checkwithoutStock ) : $xItemName;
									$newItemsCode['Item']['Name']                         = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
									$newItemsCode['Item']['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									$newItemsCode['Item']['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
								} else {
									if ( $productMaster == "w" ) {
										$newItemsCode['Item']['PurchaseDetails']['UnitPrice'] = 0;
									}
								}

								$createItem = $xero->Items( $newItemsCode );
								if ( isset( $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
									$exportErrorXero .= $errD;
								} else if ( isset( $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = array();
									$errD = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $e = 0; $e < count( $errD ); $e ++ ) {
										$errorMessage = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
										$exportErrorXero .= $errorMessage;
									}
								} else {
									update_xero_option( 'xero_connection_status', 'active' );
									update_xero_option( 'xero_connection_status_message', '' );
								}

								$itemsDetails = $createItem['Items']['Item'];
								$itemCode     = $itemsDetails['Code'];
								$itemQuantity = $itemsDetails['QuantityOnHand'];
								$itemAction   = "";

								$send_invoice_method = get_xero_option( 'xero_send_invoice_method' );
								if ( 'manually' != $send_invoice_method ) {
									if ( $itemQuantity > $xItemStock ) {
										$newQuantity = $itemQuantity - $xItemStock;
										$itemAction  = "decrease";
									} else if ( $itemQuantity < $xItemStock ) {
										$newQuantity = $xItemStock - $itemQuantity;
										$itemAction  = "increase";
									}
								} else {
									$item_stock_before = $xItemStock + $orderDetails[ $i ]['_qty'];
									if ( $itemQuantity > $item_stock_before ) {
										$newQuantity = $itemQuantity - $item_stock_before;
										$itemAction  = "decrease";
									} else if ( $itemQuantity < $item_stock_before ) {
										$newQuantity = $item_stock_before - $itemQuantity;
										$itemAction  = "increase";
									}
								}

								if ( $itemAction == "decrease" ) {
//									$decreaseItems = array(
//										"CreditNote" => array(
//											"Type"            => "ACCPAYCREDIT",
//											"Contact"         => array(
//												"Name" => "Inventory Adjustments"
//											),
//											"Date"            => date( 'Y-m-d' ),
//											"DueDate"         => date( 'Y-m-d' ),
//											"LineAmountTypes" => "NoTax",
//											"Status"          => $default_invoice_status,
//											"LineItems"       => array(
//												"LineItem" => array(
//													"ItemCode"    => $sku,
//													"Description" => $descriptionName,
//													"Quantity"    => $newQuantity,
//													"UnitAmount"  => $unitPriceNow,
//													"AccountCode" => $AssetCode,
//												)
//											)
//										)
//									);
//
//									if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
//										$xero_desc = '-';
//										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
//											$xero_desc = xero_product_description( $checkwithoutStock );
//										}
//										$decreaseItems['CreditNote']['LineItems']['LineItem']['Description'] = $xero_desc;
//										$decreaseItems['CreditNote']['LineItems']['LineItem']['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
//									}
//
//									$decItemXero = $xero->creditnotes( $decreaseItems );
//									if ( isset( $decItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
//										$errD = $decItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
//										returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
//										$exportErrorXero .= $errD;
//									} else if ( isset( $decItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
//										$errD = array();
//										$errD = $decItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
//										for ( $e = 0; $e < count( $errD ); $e ++ ) {
//											$errorMessage = $decItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
//											returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
//											$exportErrorXero .= $errorMessage;
//										}
//									} else {
//										update_xero_option( 'xero_connection_status', 'active' );
//										update_xero_option( 'xero_connection_status_message', '' );
//                                    }
								}
								if ( $itemAction == "increase" ) {
//									$increaseItems = array(
//										"Invoice" => array(
//											"Type"            => "ACCPAY",
//											"Contact"         => array(
//												"Name" => "Inventory Adjustments"
//											),
//											"Date"            => date( 'Y-m-d' ),
//											"DueDate"         => date( 'Y-m-d' ),
//											"LineAmountTypes" => "NoTax",
//											"Status"          => $default_invoice_status,
//											"LineItems"       => array(
//												"LineItem" => array(
//													"ItemCode"    => $sku,
//													"Description" => $descriptionName,
//													"Quantity"    => $newQuantity,
//													"UnitAmount"  => $unitPriceNow,
//													"AccountCode" => $AssetCode,
//												)
//											)
//										)
//									);
//
//									if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
//										$xero_desc = '-';
//										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
//											$xero_desc = xero_product_description( $checkwithoutStock );
//										}
//										$increaseItems['Invoice']['LineItems']['LineItem']['Description'] = $xero_desc;
//										$increaseItems['Invoice']['LineItems']['LineItem']['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
//									}
//
//									$incrItemXero = $xero->Invoices( $increaseItems );
//									if ( isset( $incrItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
//										$errD = $incrItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
//										returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
//										$exportErrorXero .= $errD;
//									} else if ( isset( $incrItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
//										$errD = array();
//										$errD = $incrItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
//										for ( $e = 0; $e < count( $errD ); $e ++ ) {
//											$errorMessage = $incrItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
//											returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
//											$exportErrorXero .= $errorMessage;
//										}
//									} else {
//										update_xero_option( 'xero_connection_status', 'active' );
//										update_xero_option( 'xero_connection_status_message', '' );
//                                    }
								}
							}
						} else if ( $StockMaster == "x" ) {
							$xeroInventryStock = $xero->Items( $sku );
							xeroom_check_connection_message( $xeroInventryStock, $order_id );

							if ( is_array( $xeroInventryStock ) ) {
								if ( isset( $xeroInventryStock['Items']['Item']['QuantityOnHand'] ) ) {
									$quantityI = $xeroInventryStock['Items']['Item']['QuantityOnHand'];
									$orderPro  = wc_get_product( $itemId );
									/**
									 * Set product Qty. in Woo
									 */
									$newQuantity = $quantityI - $orderDetails[ $i ]['_qty'];
//									$orderPro->set_stock( $quantityI );
									wc_update_product_stock( $orderPro, $newQuantity );

									if ( $productMaster == "x" ) {
										$oderList[ $i ]['Description'] = $xeroInventryStock['Items']['Item']['Description'];

										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
											$oderList[ $i ]['Description'] = xero_product_description( $checkwithoutStock );
										}

										if ( $xeroInventryStock['Items']['Item']['SalesDetails']['UnitPrice'] ) {
											$oderList[ $i ]['UnitAmount'] = $xeroInventryStock['Items']['Item']['SalesDetails']['UnitPrice'];
										}

									}
								} else {
//									$simpleItemCode = array(
//										"Item"            => array(
//											"Code"        => $sku,
//											"Name"        => $xItemName,
//											"Description" => $descriptionName,
//										),
//										"SalesDetails"    => array(
//											"UnitPrice"   => $unitPriceNow,
//											"AccountCode" => $salesAccount
//										),
//										"PurchaseDetails" => array(
//											"UnitPrice"       => $unitPriceNow,
//											"COGSAccountCode" => $SoldCode
//										)
//									);
									$simpleItemCode = array(
										"Item" => array(
											"Name"                => $xItemName,
											"Code"                => $sku,
											"Description"         => $descriptionName,
											"PurchaseDescription" => $descriptionName,
											"SalesDetails"        => array(
												"UnitPrice"   => $unitPriceNow,
												"AccountCode" => $salesAccount
											),
											"PurchaseDetails"     => array(
												"UnitPrice"   => $unitPriceNow,
												"AccountCode" => $SoldCode
											)
										)
									);

									if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
										$xero_desc = '-';
										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
											$xero_desc = xero_product_description( $checkwithoutStock );
										}
										$simpleItemCode['Item']['Description'] = $xero_desc;
										$simpleItemCode['Item']['Name']        = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;

										$simpleItemCode['Item']['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
										$simpleItemCode['Item']['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
									} else {
										if ( $productMaster == "w" ) {
											$simpleItemCode['Item']['PurchaseDetails']['UnitPrice'] = 0;
										}
									}

									$simpleXeroItem = $xero->Items( $simpleItemCode );
									if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
										$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
										returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
										$exportErrorXero .= $errD;
									} else if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
										$errD = array();
										$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
										for ( $e = 0; $e < count( $errD ); $e ++ ) {
											$errorMessage = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
											returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
											$exportErrorXero .= $errorMessage;
										}
									} else {
										update_xero_option( 'xero_connection_status', 'active' );
										update_xero_option( 'xero_connection_status_message', '' );
									}

									$oderList[ $i ]['Description'] = $descriptionName;
									$oderList[ $i ]['UnitAmount']  = $unitPriceNow;
									if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
										$xero_desc = '-';
										if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
											$xero_desc = xero_product_description( $checkwithoutStock );
										}
										$oderList[ $i ]['Description'] = $xero_desc;
										$oderList[ $i ]['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									}
								}
							} else {
//								$simpleItemCode = array(
//									"Item"            => array(
//										"Code"        => $sku,
//										"Name"        => $xItemName,
//										"Description" => $descriptionName,
//									),
//									"SalesDetails"    => array(
//										"UnitPrice"   => $unitPriceNow,
//										"AccountCode" => $salesAccount
//									),
//									"PurchaseDetails" => array(
//										"UnitPrice"       => $unitPriceNow,
//										"COGSAccountCode" => $SoldCode
//									)
//								);
								$simpleItemCode = array(
									"Item" => array(
										"Name"                => $xItemName,
										"Code"                => $sku,
										"Description"         => $descriptionName,
										"PurchaseDescription" => $descriptionName,
										"SalesDetails"        => array(
											"UnitPrice"   => $unitPriceNow,
											"AccountCode" => $salesAccount
										),
										"PurchaseDetails"     => array(
											"UnitPrice"   => $unitPriceNow,
											"AccountCode" => $SoldCode
										)
									)
								);

								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$simpleItemCode['Item']['Description'] = $xero_desc;
									$simpleItemCode['Item']['Name']        = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;

									$simpleItemCode['Item']['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									$simpleItemCode['Item']['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
								} else {
									if ( $productMaster == "w" ) {
										$simpleItemCode['Item']['PurchaseDetails']['UnitPrice'] = 0;
									}
								}

								$simpleXeroItem = $xero->Items( $simpleItemCode );
								if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
									$exportErrorXero .= $errD;
								} else if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = array();
									$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $e = 0; $e < count( $errD ); $e ++ ) {
										$errorMessage = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
										$exportErrorXero .= $errorMessage;
									}
								} else {
									update_xero_option( 'xero_connection_status', 'active' );
									update_xero_option( 'xero_connection_status_message', '' );
								}

								$oderList[ $i ]['Description'] = $descriptionName;
								$oderList[ $i ]['UnitAmount']  = $unitPriceNow;
								if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$oderList[ $i ]['Description'] = $xero_desc;
									$oderList[ $i ]['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
								}
							}
						} else {
							$checkWoNoStoSimpleItem = $xero->Items( $sku );
							xeroom_check_connection_message( $checkWoNoStoSimpleItem, $order_id );
							if ( isset( $checkWoNoStoSimpleItem['Items']['Item']['QuantityOnHand'] ) ) {
								$addNewQuant = $orderDetails[ $i ]['_qty'];
//								$increaseItems = array(
//									"Invoice" => array(
//										"Type"            => "ACCPAY",
//										"Contact"         => array(
//											"Name" => "Inventory Adjustments"
//										),
//										"Date"            => date( 'Y-m-d' ),
//										"DueDate"         => date( 'Y-m-d' ),
//										"LineAmountTypes" => "NoTax",
//										"Status"          => $default_invoice_status,
//										"LineItems"       => array(
//											"LineItem" => array(
//												"ItemCode"    => $sku,
//												"Description" => $descriptionName,
//												"Quantity"    => $addNewQuant,
//												"UnitAmount"  => $unitPriceNow,
//												"AccountCode" => $AssetCode,
//											)
//										)
//									)
//								);
//
//								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
//									$xero_desc = '-';
//									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
//										$xero_desc = xero_product_description( $checkwithoutStock );
//									}
//									$increaseItems['Invoice']['LineItems']['LineItem']['Description'] = $xero_desc;
//									$increaseItems['Invoice']['LineItems']['LineItem']['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
//								}
//
//								$incrWoNoStockTraiItemXero = $xero->Invoices( $increaseItems );
//								if ( isset( $incrWoNoStockTraiItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
//									$errD = $incrWoNoStockTraiItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
//									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
//									$exportErrorXero .= $errD;
//								} else if ( isset( $incrWoNoStockTraiItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
//									$errD = array();
//									$errD = $incrWoNoStockTraiItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
//									for ( $e = 0; $e < count( $errD ); $e ++ ) {
//										$errorMessage = $incrWoNoStockTraiItemXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
//										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
//										$exportErrorXero .= $errorMessage;
//									}
//								} else {
//									update_xero_option( 'xero_connection_status', 'active' );
//									update_xero_option( 'xero_connection_status_message', '' );
//                                }
							} else {
//								$simpleItemsCode = array(
//									"Item"            => array(
//										"Code"        => $sku,
//										"Name"        => $xItemName,
//										"Description" => $descriptionName
//									),
//									"SalesDetails"    => array(
//										"UnitPrice"   => $unitPriceNow,
//										"AccountCode" => $salesAccount
//									),
//									"PurchaseDetails" => array(
//										"UnitPrice"       => $unitPriceNow,
//										"COGSAccountCode" => $SoldCode
//									)
//								);
								$simpleItemsCode = array(
									"Item" => array(
										"Name"                => $xItemName,
										"Code"                => $sku,
										"Description"         => $descriptionName,
										"PurchaseDescription" => $descriptionName,
										"SalesDetails"        => array(
											"UnitPrice"   => $unitPriceNow,
											"AccountCode" => $salesAccount,
											"TaxType"     => $orderDetails[ $i ]['tax_class_new']
										),
										"PurchaseDetails"     => array(
											"UnitPrice"   => $unitPriceNow,
											"AccountCode" => $SoldCode
										)
									)
								);

								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$simpleItemsCode['Item']['Description'] = $xero_desc;

									$simpleItemsCode['Item']['Name'] = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;

									$simpleItemsCode['Item']['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									$simpleItemsCode['Item']['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
								} else {
									if ( $productMaster == "w" ) {
										$simpleItemsCode['Item']['PurchaseDetails']['UnitPrice'] = 0;
									}
								}

								$againSimpleXero = $xero->Items( $simpleItemsCode );
								if ( isset( $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
									$exportErrorXero .= $errD;
								} else if ( isset( $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = array();
									$errD = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $e = 0; $e < count( $errD ); $e ++ ) {
										$errorMessage = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
										$exportErrorXero .= $errorMessage;
									}
								} else {
									update_xero_option( 'xero_connection_status', 'active' );
									update_xero_option( 'xero_connection_status_message', '' );
								}
							}
						}
						if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
							$newItemsCodeShip = array(
								"Item" => array(
									"Code"        => $xero_shipping_price_code,
									"Description" => $xero_shipping_price_description
								)
							);
							$xero->Items( $newItemsCodeShip );
						}

						if ( $totalDiscount != 0 ) {
							$new_discounts_items = array();
							foreach ( $order->get_coupon_codes() as $coupon_name ) {
								$coupon_post_obj = get_page_by_title( $coupon_name, OBJECT, 'shop_coupon' );
								$coupon_id       = $coupon_post_obj->ID;

								// Get an instance of WC_Coupon object in an array(necesary to use WC_Coupon methods)
								$coupons_obj = new WC_Coupon( $coupon_id );

								$coupon_description = esc_html__( 'Cart Fixed Discount Coupon' );
								if ( $coupons_obj->get_discount_type() == 'fixed_product' ) {
									$coupon_description = esc_html__( 'Product Fixed Discount coupon' );
								}
								$new_discounts_items['Item'] = array(
									"Code"        => $coupon_name,
									"Description" => $coupon_description
								);
							}

							$xero->Items( $new_discounts_items );
						}

						if ( $order->get_items( array( 'fee' ) ) ) {
							$newItemsCodeDis = array(
								"Item" => array(
									"Code"        => "fee_price",
									"Description" => "Fee Price"
								)
							);
							$xero->Items( $newItemsCodeDis );
						}
					}

					// Fetch shippinh tax class
					$shipping_tax = array();
					if ( $fetch_tax_method && wc_tax_enabled() ) {
						$shipping_tax_class = 'standard';
						foreach ( $order->get_items( array( 'shipping' ) ) as $item ) {
							if ( '' != $item->get_tax_class() ) {
								$shipping_tax_class = $item->get_tax_class();
							}
						}

						$simple_tax = $order->get_shipping_tax();

						if ( $simple_tax ) {
							$product_tax = $simple_tax;
						} else {
                                $product_tax        = 'standard';
                            }

						switch ( $fetch_tax_method ) {
							case "xero_simple_tax" :
								if ( $fetch_saved_tax && is_array( $fetch_saved_tax ) && array_key_exists( 'xero_' . $shipping_tax_class . '_taxmethods', $fetch_saved_tax ) ) {
									$shipping_tax['tax_class'] = xero_tax_type_code( $fetch_saved_tax[ 'xero_' . $shipping_tax_class . '_taxmethods' ] );
								}
								break;
							case "xero_complex_tax" :
								if ( $fetch_taxes_association && is_array( $fetch_taxes_association ) && array_key_exists( $product_tax, $fetch_taxes_association ) ) {
									$shipping_tax['tax_class'] = xero_tax_type_code( $fetch_taxes_association[ $product_tax ] );
								}
								break;
						}

						if ( 'inherit' == $shipping_tax_class ) {
							$shipping_class            = reset( $orderDetails );
							$shipping_tax['tax_class'] = $shipping_class['tax_class'];
						}

						if ( 0 == $simple_tax ) {
							$shipping_tax['tax_class'] = 'NONE';
						}
					} else {
						$shipping_tax['tax_class'] = 'NONE';
					}

					if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
						$oderList[ count( $orderDetails ) ]['Description'] = $xero_shipping_price_description;
						$oderList[ count( $orderDetails ) ]['Quantity']    = 1;
						$oderList[ count( $orderDetails ) ]['UnitAmount']  = $shippingPrice;
//						$oderList[ count( $orderDetails ) ]['DiscountRate'] = 0;
						$oderList[ count( $orderDetails ) ]['ItemCode'] = $xero_shipping_price_code;
						if ( in_array( 'shipping', $taxOnWholeCart ) && $taxOnWholeCart['shipping'] == "yes" ) {
							// Add tax Rate name
//							if ( $uniqueTax != "" ) {
//								$oderList[ count( $orderDetails ) ]['TaxType'] = $uniqueTax;
//							}
						}
						if ( ! empty( $shipping_tax ) ) {
							$oderList[ count( $orderDetails ) ]['TaxType'] = $shipping_tax['tax_class'];
						}

						$oderList[ count( $orderDetails ) ]['AccountCode'] = $shippingCode && '' != $shippingCode ? $shippingCode : $salesAccount;

						// Add tracking
						if ( ! empty( $add_tracking ) ) {
							$oderList[ count( $orderDetails ) ]['Tracking'] = array(
								'TrackingCategory' => array(
									'Name'   => esc_attr( $tracking_category ),
									'Option' => $add_tracking,
								),
							);
						}
					}

					if ( $lic_method != "trail" ) {
						if ( $totalDiscount != 0 ) {
							$products_tax_type         = reset( $orderDetails );
							$discount_tax['tax_class'] = $products_tax_type['tax_class'];

							if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
								$nextCount = count( $orderDetails ) + 1;
							} else {
								$nextCount = count( $orderDetails );
							}

							$coupon_quantity = 0;

							foreach ( $order->get_coupon_codes() as $coupon_name ) {
								// Retrieving the coupon ID
								$coupon_post_obj = get_page_by_title( $coupon_name, OBJECT, 'shop_coupon' );
								$coupon_id       = $coupon_post_obj->ID;
								$coupons_obj     = new WC_Coupon( $coupon_id );

								$coupon_order_id = wpr_fetch_coupon_item_id( $order_id, $coupon_name );

								$order_discount_amount     = wc_get_order_item_meta( $coupon_order_id, 'discount_amount', true );
								$order_discount_tax_amount = wc_get_order_item_meta( $coupon_order_id, 'discount_amount_tax', true );

//								$oderList[ $nextCount ]['UnitAmount'] = 0;
								$oderList[ $nextCount ]['Quantity'] = 1;
								if ( $coupons_obj->get_discount_type() == 'fixed_cart' ) {
									$oderList[ $nextCount ]['Description'] = esc_html__( 'Cart Fixed Discount coupon' );
									if ( $included_tax ) {
										$oderList[ $nextCount ]['UnitAmount'] = - $order_discount_amount;
									} else {
										$oderList[ $nextCount ]['UnitAmount'] = - $coupons_obj->get_amount();
									}
								} elseif ( $coupons_obj->get_discount_type() == 'fixed_product' ) {
									$coupon_applied                        = array_intersect( array_keys( $products_ids ), $coupons_obj->get_product_ids() );
									$oderList[ $nextCount ]['Description'] = esc_html__( 'Fixed product discount' );
									if ( $included_tax ) {
										$oderList[ $nextCount ]['UnitAmount'] = - $order_discount_amount;
									} else {
										$oderList[ $nextCount ]['UnitAmount'] = - $coupons_obj->get_amount();
									}

									if ( count( $coupon_applied ) > 0 ) {
										foreach ( $products_ids as $product_id => $quantity ) {
											if ( in_array( $product_id, $coupons_obj->get_product_ids() ) ) {
												$coupon_quantity += $quantity;
											}
										}

										$oderList[ $nextCount ]['Quantity'] = $coupon_quantity;
									}
								} else {
									$oderList[ $nextCount ]['Description'] = esc_html__( 'Cart Percent Discount coupon' );
								}

								$oderList[ $nextCount ]['ItemCode'] = $coupon_name;

								// Add tax Rate name
								if ( ! empty( $discount_tax ) ) {
									$oderList[ $nextCount ]['TaxType'] = $discount_tax['tax_class'];
								}
								$oderList[ $nextCount ]['AccountCode'] = $salesAccount;

								// Add tracking
								if ( ! empty( $add_tracking ) && $coupons_obj->get_discount_type() != 'percent' ) {
									$oderList[ $nextCount ]['Tracking'] = array(
										'TrackingCategory' => array(
											'Name'   => esc_attr( $tracking_category ),
											'Option' => $add_tracking,
										),
									);
								}

								$nextCount ++;
							}
						}
					}

					// Fee tax
					$fee_tax = array();
					if ( $fetch_tax_method && wc_tax_enabled() ) {
						$fee_class            = reset( $orderDetails );
						$fee_tax['tax_class'] = $fee_class['tax_class'];
					}

					// Add fee if exists
					if ( $order->get_items( array( 'fee' ) ) ) {
						$fee_count = count( $oderList );
						foreach ( $order->get_items( array( 'fee' ) ) as $item_type => $item ) {
							$oderList[ $fee_count ]['Description']  = "Fee Price";
							$oderList[ $fee_count ]['Quantity']     = 1;
							$oderList[ $fee_count ]['UnitAmount']   = $order->get_item_total( $item, false, true );
							$oderList[ $fee_count ]['DiscountRate'] = 0;
							$oderList[ $fee_count ]['ItemCode']     = "fee_price";
							if ( ! empty( $fee_tax ) ) {
								$oderList[ $fee_count ]['TaxType'] = $fee_tax['tax_class'];
							}

							$oderList[ $fee_count ]['AccountCode'] = $salesAccount;
							$fee_count ++;
						}
					}

					for ( $i = 0; $i < count( $oderList ); $i ++ ) {
						$lineItemsArray[] = array( "LineItem" => array( $oderList[ $i ] ) );
					}

					$currentDate       = xeroom_invoice_date( $order );
					$due_date_settings = get_xero_option( 'xero_set_invoice_duedate' );
					$xero_contact_name = get_xero_option( 'xero_contact_name' );
					$xero_address_info = get_xero_option( 'xero_address_info' );
					$invoice_start_no  = get_xero_option( 'xero_invoice_start_no' );
					$invoice_next_no   = get_xero_option( 'xero_invoice_next_no' );

					if ( 'xeroom_use_company' == $xero_contact_name ) {
						$xero_add_contact_name = $shipAddress['company'];
					} elseif ( 'xeroom_use_email' == $xero_contact_name ) {
						$xero_add_contact_name = $shipAddress['email'];
					} else {
						$xero_add_contact_name = $shipAddress['first_name'] . ' ' . $shipAddress['last_name'];
					}

					$transaction_info = '';
					if ( $order->get_transaction_id() ) {
						$transaction_info = mb_substr( $order->get_transaction_id(), - 7, null, 'UTF-8' );
					}

					if ( $lic_method != "trail" ) {
						$new_invoice = array(
							array(
								"Type"            => "ACCREC",
								"Contact"         => array(
									"Name"         => $xero_add_contact_name,
									"FirstName"    => $shipAddress['first_name'],
									"LastName "    => $shipAddress['last_name'],
									"EmailAddress" => $shipAddress['email'],
//									"Addresses"    => array(
//										"Address" => array(
//											array(
//												"AddressType"  => base64_decode( 'UE9CT1g=' ),
//												"AttentionTo"  => "",
//												"AddressLine1" => $shipAddress['company'],
//												"AddressLine2" => $shipAddress['address_1'],
//												"AddressLine3" => $shipAddress['address_2'],
//												"City"         => $shipAddress['city'],
//												"Region"       => $stateFullName,
//												"Country"      => $countryFullName,
//												"PostalCode"   => $shipAddress['postcode']
//											)
//										)
//									),
									"Phones"       => array(
										"Phone" => array(
											"PhoneType"   => base64_decode( 'TU9CSUxF' ),
											"PhoneNumber" => $shipAddress['phone']
										)
									)
								),
								"Date"            => $currentDate,
								"Status"          => $default_invoice_status,
								"CurrencyCode"    => $orderCurency,
								"LineAmountTypes" => base64_decode( 'RXhjbHVzaXZl' ),
								"Reference"       => $order_id,
								"LineItems"       => $lineItemsArray
							)
						);

						if ( 'xeroom_use_woo_address' == $xero_address_info ) {
							$new_invoice[0]['Contact']['Addresses']['Address'] = array(
								array(
									"AddressType"  => base64_decode( 'UE9CT1g=' ),
									"AttentionTo"  => "",
									"AddressLine1" => ( 'xeroom_use_company' != $xero_contact_name ) ? $shipAddress['company'] : '',
									"AddressLine2" => $shipAddress['address_1'],
									"AddressLine3" => $shipAddress['address_2'],
									"City"         => $shipAddress['city'],
									"Region"       => $stateFullName,
									"Country"      => $countryFullName,
									"PostalCode"   => $shipAddress['postcode']
								)
							);
						}
					} else {
						$new_invoice = array(
							array(
								"Type"            => "ACCREC",
								"Contact"         => array(
									"Name"         => $xero_add_contact_name,
									"FirstName"    => $shipAddress['first_name'],
									"LastName "    => $shipAddress['last_name'],
									"EmailAddress" => $shipAddress['email'],
									"Addresses"    => array(
										"Address" => array(
											array(
												"AddressType"  => base64_decode( 'UE9CT1g=' ),
												"AttentionTo"  => "Created using demo version of Xeroom",
												"AddressLine1" => "",
												"AddressLine2" => "",
												"AddressLine3" => "",
												"City"         => "",
												"Region"       => "",
												"Country"      => "",
												"PostalCode"   => ""
											)
										)
									),
									"Phones"       => array(
										"Phone" => array(
											"PhoneType"   => base64_decode( 'TU9CSUxF' ),
											"PhoneNumber" => $shipAddress['phone']
										)
									)
								),
								"Date"            => $currentDate,
								"Status"          => $default_invoice_status,
								"CurrencyCode"    => $orderCurency,
								"LineAmountTypes" => base64_decode( 'RXhjbHVzaXZl' ),
								"Reference"       => $order_id,
								"LineItems"       => $lineItemsArray
							)
						);
					}

					$xeroom_invoice_no_sent = get_post_meta( $order_id, 'xeroom_invoice_no_sent', true );

					if ( $invoice_start_no || $invoice_prefix ) {
						$xero_invoice_no = $order_id;
						if ( $invoice_start_no && ! empty( $invoice_start_no ) ) {
							if ( $invoice_next_no ) {
								$xero_invoice_no = absint( $invoice_next_no );
							} else {
								$xero_invoice_no = absint( $invoice_start_no );
							}
						}

						if ( $xeroom_invoice_no_sent ) {
							$xero_invoice_no = absint( $xeroom_invoice_no_sent );
						}

						$invoice_meta_no = get_xero_option( 'xeroom_inv_number_meta_key' );
						if ( ! empty( $invoice_meta_no ) ) {
							$custom_order_no_meta = get_post_meta( $order_id, $invoice_meta_no, true );

							if ( $custom_order_no_meta ) {
								$xero_invoice_no = esc_attr( $custom_order_no_meta );
							}
						}

						$new_invoice[0]['InvoiceNumber'] = $xero_invoice_no;

						if ( ! empty( $invoice_prefix ) ) {
							$new_invoice[0]['InvoiceNumber'] = $invoice_prefix . '-' . $xero_invoice_no;
						}
					} else {
						$invoice_meta_no = get_xero_option( 'xeroom_inv_number_meta_key' );
						if ( ! empty( $invoice_meta_no ) ) {
							$custom_order_no_meta = get_post_meta( $order_id, $invoice_meta_no, true );

							if ( $custom_order_no_meta ) {
								$xero_invoice_no = esc_attr( $custom_order_no_meta );
								$new_invoice[0]['InvoiceNumber'] = $xero_invoice_no;
							}
						}
					}

					if ( 'manually-resend-invoice' == $errorType ) {
						if ( $xeroom_invoice_no_sent ) {
							if ( ! empty( $invoice_prefix ) ) {
								$new_invoice[0]['InvoiceNumber'] = $invoice_prefix . '-' . $xeroom_invoice_no_sent . '/' . xeroom_add_invoice_suffix( $order_id );
							} else {
								$new_invoice[0]['InvoiceNumber'] = $xeroom_invoice_no_sent . '/' . xeroom_add_invoice_suffix( $order_id );
							}

						}
					}

					$new_invoice[0]['Reference'] = $order_id;
					if ( $invoice_reference ) {
						$xero_invoice_ref = $invoice_reference . '-' . $order_id;

						if ( ! empty( $order->get_payment_method() ) ) {
							$xero_invoice_ref .= '-' . strtoupper( $order->get_payment_method() );
						}
						if ( ! empty( $transaction_info ) ) {
							$xero_invoice_ref .= '-' . $transaction_info;
						}

						$new_invoice[0]['Reference'] = $xero_invoice_ref;
					}

					$woo_pdf_inv = get_post_meta( $order_id, '_invoice_number_display', true );
					if ( $woo_pdf_inv ) {
//						$new_invoice[0]['Reference'] = esc_html__( $woo_pdf_inv );
					}

					$custom_number_of_days = get_xero_option( 'xero_due_date_custom_days' );
					if ( $due_date_settings && 'use_custom_due_date' == $due_date_settings && $custom_number_of_days ) {
						$new_invoice[0]['DueDate']             = date( 'Y-m-d', strtotime( "+$custom_number_of_days days" ) );
						$new_invoice[0]['ExpectedPaymentDate'] = date( 'Y-m-d', strtotime( "+$custom_number_of_days days" ) );
					} else {
						$new_invoice[0]['DueDate']             = date( 'Y-m-d', strtotime( '+3 day' ) );
						$new_invoice[0]['ExpectedPaymentDate'] = date( 'Y-m-d', strtotime( '+3 day' ) );
					}

					if ( $license_checked != 'expired' ) {
						$new_invoice = apply_filters( 'xeroom_new_invoice_data', $new_invoice, $order_id );
                        
						if ( 'manually-resend-invoice' == $errorType ) {
							$invoiceCreated = $xero->Invoices( $new_invoice, array( 'InvoiceResend' => true ) );
						} else {
							$invoiceCreated = $xero->Invoices( $new_invoice );
						}

						if ( ! empty( $invoiceCreated ) ) {
							if ( isset( $invoiceCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
								$errD = $invoiceCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
								returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
								$exportErrorXero .= $errD;
							} else if ( isset( $invoiceCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
								$errD = array();
								$errD = $invoiceCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
								for ( $e = 0; $e < count( $errD ); $e ++ ) {
									$errorMessage = $invoiceCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
									returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
									$exportErrorXero .= $errorMessage;
								}
							} else if ( $invoiceCreated['Status'] != "OK" ) {
								returnErrorMessageByXero( $order_id, $invoiceCreated, $xeroTime, $errorType );
								$exportErrorXero .= $invoiceCreated;
							}
						} else {
							$mMessage = "Xero Server Response is empty.";
							returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
							$exportErrorXero .= $mMessage;
						}

						if ( isset( $invoiceCreated['Status'] ) && $invoiceCreated['Status'] == "OK" ) {
							$InvoiceNumber = $invoiceCreated['Invoices']['Invoice']['InvoiceNumber'];
							$InvoiceID     = $invoiceCreated['Invoices']['Invoice']['InvoiceID'];
							$AmountDue     = $invoiceCreated['Invoices']['Invoice']['AmountDue'];
							$my_order_post = array(
								'ID'                    => intval( $order_id ),
								'post_content'          => sanitize_text_field( $InvoiceNumber ),
								'post_content_filtered' => sanitize_text_field( $InvoiceID ),
								'post_mime_type'        => sanitize_text_field( $AmountDue )
							);
							wp_update_post( $my_order_post );
							update_post_meta( $order_id, 'xeroom_order_sent', 'Sent to Xero' );

							if ( $invoice_start_no || $invoice_prefix ) {
								if ( ! get_post_meta( $order_id, 'xeroom_invoice_no_sent', true ) ) {
									update_post_meta( $order_id, 'xeroom_invoice_no_sent', absint( $xero_invoice_no ) );
								}

								update_xero_option( 'xero_invoice_next_no', absint( $xero_invoice_no ) + 1 );
							} else {
								if ( ! get_post_meta( $order_id, 'xeroom_invoice_no_sent', true ) ) {
									update_post_meta( $order_id, 'xeroom_invoice_no_sent', sanitize_text_field( $InvoiceNumber ) );
								}
							}

							// Fix Rounding in Xero ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
							$invoice_total = $invoiceCreated['Invoices']['Invoice']['Total'];
							$invoice_items = $invoiceCreated['Invoices']['Invoice']['LineItems'];
                            
							if ( $order->get_total() != $invoice_total ) {
								$items_to_xero = array();
								foreach ( $invoice_items as $xero_items ) {
									$items_to_xero[] = $xero_items;
								}

								$items_to_xero[] = array(
									'Description' => __( 'Rounding adjustment', 'xeroom' ),
									'Quantity'    => 1,
									'UnitAmount'  => $order->get_total() - $invoice_total,
									'AccountCode' => 860,
									//'ItemCode'    => 'rounding_adjustment',
								);

								$grouped_items = array();
								for ( $i = 0; $i < count( $items_to_xero ); $i ++ ) {
									if ( isset( $invoiceCreated['Invoices']['Invoice']['LineItems']['LineItem'][0] ) ) {
										$grouped_items[] = array( "LineItem" => $items_to_xero[ $i ] );
									} else {
										$grouped_items[] = array( "LineItem" => array( $items_to_xero[ $i ] ) );
									}
								}

								$rounding_info = array(
									array(
										"InvoiceNumber" => $InvoiceNumber,
										"LineItems"     => $grouped_items,
									),
								);

								$xero_rounding = $xero->Invoices( $rounding_info );

								if ( ! empty( $xero_rounding ) ) {
									if ( isset( $xero_rounding['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
										$errD = $xero_rounding['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
										returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
									} else if ( isset( $xero_rounding['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
										$errD = $xero_rounding['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
										for ( $e = 0; $e < count( $errD ); $e ++ ) {
											$errorMessage = $xero_rounding['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
											returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
										}
									} else if ( $xero_rounding['Status'] != "OK" ) {
										returnErrorMessageByXero( $order_id, $xero_rounding, $xeroTime, $errorType );
									}
								} else {
									$mMessage = "Xero Server Response is empty.";
									returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
								}

								// Update Amount Due
								if ( $xero_rounding['Status'] == "OK" ) {
									$AmountDue     = $xero_rounding['Invoices']['Invoice']['AmountDue'];
									$my_order_post = array(
										'ID'             => intval( $order_id ),
										'post_mime_type' => sanitize_text_field( $AmountDue )
									);
									wp_update_post( $my_order_post );
								}
							}

							update_xero_option( 'xero_connection_status', 'active' );
							update_xero_option( 'xero_connection_status_message', '' );
							// Fix Rounding in Xero ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
						}
					}
				} else {
					$mMessage = "Xeroom credentials are not set or invalid, Please contact Xeroom support team.";
					returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
					$exportErrorXero .= $mMessage;
				}
			} else {
				$mMessage = "Your License expired, Please contact xeroom support team.";
				returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
				$exportErrorXero .= $mMessage;
			}
		} else {
			//			$mMessage = "Please use premium or trail license!";
			$mMessage = esc_html__( 'Order posting to Xero by Xeroom failed. Your licence has expired.', 'xeroom' );
			returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
			$exportErrorXero          .= $mMessage;
			$license_validation_error = esc_html__( 'Order posting to Xero by Xeroom failed. Your licence has expired.', 'xeroom' );
		}

		// Add order note
		if ( '' != $exportErrorXero ) {
			$success = 0;
			// Add order note for failing to send data Xero
			if ( ! isset( $license_validation_error ) && ! empty( $license_validation_error ) ) {
				$xero_order_note = __( 'Order posting to Xero by Xeroom failed', 'xeroom' );
				$exportErrorXero .= $xero_order_note;

				$xero_order_note = apply_filters(
					'xero_add_fail_message_order',
					$xero_order_note,
					$order
				);

				$order->add_order_note( $xero_order_note );
			}
		} else {
			$success = 1;
			// Add order note for successfully sent data to Xero
			$xero_order_note = __( 'The order has been successfully sent to Xero using Xeroom', 'xeroom' );

			$xero_order_note = apply_filters(
				'xero_add_success_message_order',
				$xero_order_note,
				$order
			);

			$order->add_order_note( $xero_order_note );
			update_post_meta( $order_id, 'xeroom_order_sent', 'Sent to Xero' );
		}

		if ( ! $success ) {
			return $exportErrorXero;
		} elseif ( $success ) {
			return esc_html__( 'Successful', 'xeroom' );
		} else {
			return $exportErrorXero;
		}
	} else {
		$mMessage = "Xeroom credentials are not set or invalid, Please contact Xeroom support team.";
		returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
	}
}

/**
 * Return Coupon as item id
 *
 * @param $order_id
 * @param $coupon_name
 *
 * @return mixed
 */
function wpr_fetch_coupon_item_id( $order_id, $coupon_name ) {
	global $wpdb;

	$table   = $wpdb->prefix . "woocommerce_order_items";
	$item_id = $wpdb->get_row( "SELECT order_item_id FROM {$table} WHERE order_id = {$order_id} AND order_item_name = '{$coupon_name}'" );

	return $item_id->order_item_id;
}

/**
 * Set Invoice Date
 *
 * @param $order
 *
 * @return false|string
 */
function xeroom_invoice_date( $order ) {
	$invoice_date = get_xero_option( 'xero_invoice_date' );
	if ( 'order_date' == $invoice_date ) {
		$currentDate = $order->get_date_created()->format( 'Y-m-d H:i:s' );
	} else {
		$currentDate = date( 'Y-m-d H:i:s' );
	}

	return $currentDate;
}

/**
 * Send Payment Success to Xero after Successfully Checkout Payment
 *
 * @param $order_id
 * @param string $errorType
 *
 * @return string
 * @throws XeroException
 */
function xeroom_paymentDoneOnCheckout( $order_id, $errorType = "productOrder" ) {
	include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
	global $wpdb;
	$exportErrorXero   = "";
	$xeroTime          = date( 'Y-m-d H:i:s' );
	$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
	$getApiCredentials = $wpdb->get_results( $query );

	if ( $getApiCredentials ) {
		$xeroApiKey    = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
		$xeroApiSecret = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
		$salesAccount  = sanitize_text_field( $getApiCredentials[0]->sales_account );
		$BankCode      = sanitize_text_field( $getApiCredentials[0]->bank_code );

		$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );
		if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
			$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
		}

		$woo_gateway_payment_sending = get_xero_option( 'xero_woo_gateway_payment_send' );
		if ( $woo_gateway_payment_sending && ! array_key_exists( 'xero_' . $oder_gateway_code . '_payment_auto_send', $woo_gateway_payment_sending ) ) {
			return;
		}

		if ( $errorType == "manually" ) {
			if ( ! defined( 'BASE_PATH' ) ) {
				define( 'BASE_PATH', dirname( __FILE__ ) );
			}

			if ( ! defined( 'XERO_KEY' ) ) {
				define( 'XERO_KEY', $xeroApiKey );
			}

			if ( ! defined( 'XERO_SECRET' ) ) {
				define( 'XERO_SECRET', $xeroApiSecret );
			}

			if ( ! defined( 'PUBLIC_KEY' ) ) {
				define( 'PUBLIC_KEY', '' );
			}

			if ( ! defined( 'PRIVATE_KEY' ) ) {
				define( 'PRIVATE_KEY', '' );
			}

			if ( ! defined( 'FORMAT' ) ) {
				define( 'FORMAT', 'json' );
			}

			$xero_timezone = new DateTimeZone( 'UTC' );
			$xero_time     = date( 'Y-m-d H:i:s' );
			$xero_date     = new DateTime( $xero_time, $xero_timezone );

			$oauth2        = get_xero_option( 'xero_oauth_options' );
			$xero          = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );
			$order_details = get_post( $order_id );

			$order            = new WC_Order( $order_id );
			$transaction_info = '';
			if ( $order->get_transaction_id() ) {
				$transaction_info = mb_substr( $order->get_transaction_id(), - 7, null, 'UTF-8' );
			}

			$new_payment = array(
				array(
					"Invoice"   => array(
						"InvoiceNumber" => $order_details->post_content
					),
					"Account"   => array(
						"Code" => $BankCode
					),
					"Date"      => $xero_date,
					"Amount"    => $order_details->post_mime_type,
					"Reference" => strtoupper( $oder_gateway_code ) . $transaction_info
				)
			);
			$paymentXero = $xero->Payments( $new_payment );

			$order = new WC_Order( $order_id );

			if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
				$errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] . ". Please Create your bank account on xero and update bank code in xeroom setting's bank code.";
				returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
				$exportErrorXero .= $errD;

				// Add order note for failing to send data Xero
				$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

				$xero_order_note = apply_filters(
					'xero_add_fail_message_order',
					$xero_order_note,
					$order
				);

				$order->add_order_note( $xero_order_note );
			} else if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
				$errD = array();
				$errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
				for ( $e = 0; $e < count( $errD ); $e ++ ) {
					$errorMessage = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
					returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
					$exportErrorXero .= $errorMessage;
				}

				// Add order note for failing to send data Xero
				$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

				$xero_order_note = apply_filters(
					'xero_add_fail_message_order',
					$xero_order_note,
					$order
				);

				$order->add_order_note( $xero_order_note );
			} else {
				update_post_meta( $order_id, 'xeroom_payment_sent', 'Sent to Xero' );

				// Add order note for successfully sent data to Xero
				$xero_order_note = __( 'The Payment has been successfully sent to Xero using Xeroom', 'xeroom' );

				$xero_order_note = apply_filters(
					'xero_add_success_message_order',
					$xero_order_note,
					$order
				);

				$order->add_order_note( $xero_order_note );
				update_post_meta( $order_id, 'xeroom_order_sent', 'Sent to Xero' );

				update_xero_option( 'xero_connection_status', 'active' );
				update_xero_option( 'xero_connection_status_message', '' );
			}

			return $exportErrorXero;
		}
	} else {
		$mMessage = "Xeroom credentials are not set or invalid, Please contact Xeroom support team.";
		returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
	}

	return $exportErrorXero;
}

/**
 * Send Payment Success to Xero after Payment is Done
 *
 * @param $order_id
 */
function xeroom_paymentDoneOnPayment( $order_id ) {
	include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
	global $wpdb;
	$errorType = "productOrder";

	$xeroTime          = date( 'Y-m-d H:i:s' );
	$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
	$getApiCredentials = $wpdb->get_results( $query );

	if ( $getApiCredentials ) {
		$xeroApiKey    = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
		$xeroApiSecret = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
		$salesAccount  = sanitize_text_field( $getApiCredentials[0]->sales_account );
		$BankCode      = sanitize_text_field( $getApiCredentials[0]->bank_code );
		$sendOrNot     = $getApiCredentials[0]->payment_master;

		$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );
		if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
			$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
		}

		$woo_gateway_payment_sending = get_xero_option( 'xero_woo_gateway_payment_send' );
		if ( $woo_gateway_payment_sending && ! array_key_exists( 'xero_' . $oder_gateway_code . '_payment_auto_send', $woo_gateway_payment_sending ) ) {
			return;
		}

		$send_payment_method = get_xero_option( 'xero_send_payment_method' );

		if ( 'automatically' == $send_payment_method ) {
			if ( ! defined( 'BASE_PATH' ) ) {
				define( 'BASE_PATH', dirname( __FILE__ ) );
			}

			if ( ! defined( 'XERO_KEY' ) ) {
				define( 'XERO_KEY', $xeroApiKey );
			}

			if ( ! defined( 'XERO_SECRET' ) ) {
				define( 'XERO_SECRET', $xeroApiSecret );
			}

			if ( ! defined( 'PUBLIC_KEY' ) ) {
				define( 'PUBLIC_KEY', '' );
			}

			if ( ! defined( 'PRIVATE_KEY' ) ) {
				define( 'PRIVATE_KEY', '' );
			}

			if ( ! defined( 'FORMAT' ) ) {
				define( 'FORMAT', 'json' );
			}

			$xero_timezone = new DateTimeZone( 'UTC' );
			$xero_time     = date( 'Y-m-d H:i:s' );
			$xero_date     = new DateTime( $xero_time, $xero_timezone );

			$oauth2 = get_xero_option( 'xero_oauth_options' );
			$xero   = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );

			// Check if Invoice exists in Xero
			if ( ! xero_invoice_exists( $order_id, $xeroApiKey, $xeroApiSecret ) ) {
				xeroom_sendWooInvoiceToXero( $order_id, 'orderProduct' );
			}

			$order_details = get_post( $order_id );

			$order            = new WC_Order( $order_id );
			$transaction_info = '';
			if ( $order->get_transaction_id() ) {
				$transaction_info = mb_substr( $order->get_transaction_id(), - 7, null, 'UTF-8' );
			}

			$new_payment = array(
				array(
					"Invoice"   => array(
						"InvoiceNumber" => $order_details->post_content
					),
					"Account"   => array(
						"Code" => $BankCode
					),
					"Date"      => $xero_date,
					"Amount"    => $order_details->post_mime_type,
					"Reference" => strtoupper( $oder_gateway_code ) . $transaction_info
				)
			);
			$paymentXero = $xero->Payments( $new_payment );

			if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
				$errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] . ". Please Create your bank account on xero and update bank code in xeroom setting's bank code.";
				returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );

				// Add order note for failing to send data Xero
				$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

				$xero_order_note = apply_filters(
					'xero_add_fail_message_order',
					$xero_order_note,
					$order
				);

				$order->add_order_note( $xero_order_note );
			} else if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
				$errD = array();
				$errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
				for ( $e = 0; $e < count( $errD ); $e ++ ) {
					$errorMessage = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
					returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
				}

				// Add order note for failing to send data Xero
				$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

				$xero_order_note = apply_filters(
					'xero_add_fail_message_order',
					$xero_order_note,
					$order
				);

				$order->add_order_note( $xero_order_note );
			} else {
				update_post_meta( $order_id, 'xeroom_payment_sent', 'Sent to Xero' );

				// Add order note for successfully sent data to Xero
				$xero_order_note = __( 'The Payment has been successfully sent to Xero using Xeroom', 'xeroom' );

				$xero_order_note = apply_filters(
					'xero_add_success_message_order',
					$xero_order_note,
					$order
				);

				$order->add_order_note( $xero_order_note );
				update_post_meta( $order_id, 'xeroom_order_sent', 'Sent to Xero' );

				update_xero_option( 'xero_connection_status', 'active' );
				update_xero_option( 'xero_connection_status_message', '' );
			}
		}
	} else {
		$mMessage = "Xeroom credentials are not set or invalid, Please contact Xeroom support team.";
		returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
	}
}

/**
 * Send Payment Success to Xero on Order status changed to Complete
 *
 * @param $order_id
 */
function xeroom_paymentDoneOnCheckoutManually( $order_id ) {
	// Send Invoice to Xero when order status changed to Complete
	$send_invoice_method = get_xero_option( 'xero_send_invoice_method' );
	if ( 'on_completion' == $send_invoice_method ) {
		xeroom_sendWooInvoiceToXero( $order_id, 'orderProduct' );
	}

	$send_payment_method = get_xero_option( 'xero_send_payment_method' );
	if ( 'manually' == $send_payment_method ) {
		return;
	}

	include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
	global $wpdb;
	$xeroTime          = date( 'Y-m-d H:i:s' );
	$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
	$getApiCredentials = $wpdb->get_results( $query );

	if ( $getApiCredentials ) {
		$xeroApiKey    = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
		$xeroApiSecret = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
		$salesAccount  = sanitize_text_field( $getApiCredentials[0]->sales_account );
		$BankCode      = sanitize_text_field( $getApiCredentials[0]->bank_code );

		$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );
		if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
			$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
		}

		$woo_gateway_payment_sending = get_xero_option( 'xero_woo_gateway_payment_send' );
		if ( $woo_gateway_payment_sending && ! array_key_exists( 'xero_' . $oder_gateway_code . '_payment_auto_send', $woo_gateway_payment_sending ) ) {
			return;
		}

		if ( ! defined( 'BASE_PATH' ) ) {
			define( 'BASE_PATH', dirname( __FILE__ ) );
		}

		if ( ! defined( 'XERO_KEY' ) ) {
			define( 'XERO_KEY', $xeroApiKey );
		}

		if ( ! defined( 'XERO_SECRET' ) ) {
			define( 'XERO_SECRET', $xeroApiSecret );
		}

		if ( ! defined( 'PUBLIC_KEY' ) ) {
			define( 'PUBLIC_KEY', '' );
		}

		if ( ! defined( 'PRIVATE_KEY' ) ) {
			define( 'PRIVATE_KEY', '' );
		}

		if ( ! defined( 'FORMAT' ) ) {
			define( 'FORMAT', 'json' );
		}

		$xero_timezone = new DateTimeZone( 'UTC' );
		$xero_time     = date( 'Y-m-d H:i:s' );
		$xero_date     = new DateTime( $xero_time, $xero_timezone );

		$oauth2        = get_xero_option( 'xero_oauth_options' );
		$xero          = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );
		$order_details = get_post( $order_id );

		$order            = new WC_Order( $order_id );
		$transaction_info = '';
		if ( $order->get_transaction_id() ) {
			$transaction_info = mb_substr( $order->get_transaction_id(), - 7, null, 'UTF-8' );
		}

		$new_payment = array(
			array(
				"Invoice"   => array(
					"InvoiceNumber" => $order_details->post_content
				),
				"Account"   => array(
					"Code" => $BankCode
				),
				"Date"      => $xero_date,
				"Amount"    => $order_details->post_mime_type,
				"Reference" => strtoupper( $oder_gateway_code ) . $transaction_info
			)
		);
		$paymentXero = $xero->Payments( $new_payment );

		if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
			$errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] . ". Please Create your bank account on xero and update bank code in xeroom setting's bank code.";
			returnErrorMessageByXero( $order_id, $errD, $xeroTime, 'productOrder' );

			// Add order note for failing to send data Xero
			$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

			$xero_order_note = apply_filters(
				'xero_add_fail_message_order',
				$xero_order_note,
				$order
			);

			$order->add_order_note( $xero_order_note );
		} else if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
			$errD = array();
			$errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
			for ( $e = 0; $e < count( $errD ); $e ++ ) {
				$errorMessage = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
				returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, 'productOrder' );
			}

			// Add order note for failing to send data Xero
			$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

			$xero_order_note = apply_filters(
				'xero_add_fail_message_order',
				$xero_order_note,
				$order
			);

			$order->add_order_note( $xero_order_note );
		} else {
			update_post_meta( $order_id, 'xeroom_payment_sent', 'Sent to Xero' );

			// Add order note for successfully sent data to Xero
			$xero_order_note = __( 'The Payment has been successfully sent to Xero using Xeroom', 'xeroom' );

			$xero_order_note = apply_filters(
				'xero_add_success_message_order',
				$xero_order_note,
				$order
			);

			$order->add_order_note( $xero_order_note );
			update_post_meta( $order_id, 'xeroom_order_sent', 'Sent to Xero' );

			update_xero_option( 'xero_connection_status', 'active' );
			update_xero_option( 'xero_connection_status_message', '' );
		}
	} else {
		$mMessage = "Xeroom credentials are not set or invalid, Please contact Xeroom support team.";
		returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, 'productOrder' );
	}
}

add_action( 'admin_enqueue_scripts', 'xero_enqueue_admin_js' );
/**
 * Enqueue admin script
 *
 * @param $hook
 */
function xero_enqueue_admin_js() {
	if ( ! wp_script_is( 'select2', 'enqueued' ) ) {
		wp_enqueue_style( 'select2', plugins_url( 'css/select2.css', __FILE__ ) );
		wp_register_script( 'select2', plugin_dir_url( __FILE__ ) . 'js/select2.full.js', array( 'jquery' ), '2.0.0', true );
	}

	wp_register_script( 'xero-js', plugin_dir_url( __FILE__ ) . 'js/xero-js.js', array( 'select2' ), XEROOM_VERSION, true );

	$scheme = 'http';
	if ( is_ssl() ) {
		$scheme = 'https';
	}

	$screen = get_current_screen();

	$args = array(
		'nonce'        => wp_create_nonce( 'xero-ajax' ),
		'xeroajax'     => admin_url( 'admin-ajax.php', $scheme ),
		'xero_orders'  => esc_attr( $screen->id ),
		'xero_loading' => sprintf( '%s %s <a href="%s" target="_blank">%s</a>', __( 'Sync job in progress!', 'xeroom' ), __( 'Please', 'xeroom' ), admin_url( 'admin.php?page=xeroom_log_woo_xero' ), __( 'check Xeroom logs here', 'xeroom' ) ),
	);
	wp_localize_script( 'xero-js', 'xero_ajax_object', $args );
	wp_enqueue_script( 'xero-js' );
}

add_action( 'add_meta_boxes', 'xero_add_order_meta_boxe' );
/**
 * Register Xero order custom meta box
 */
function xero_add_order_meta_boxe() {
	add_meta_box( 'xero_meta_box', __( 'Xero invoice status', 'xeroom' ), 'xero_invoice_status_meta_box', 'shop_order', 'side', 'low' );
}

/**
 * Display Xero buttons
 *
 * @param $post
 */
function xero_invoice_status_meta_box( $post ) {
	echo sprintf( '<span>%s</span><br />', __( 'Send Invoice to Xero' ) );
	echo sprintf( '<button data-id="%d" data-call="invoice" type="button" class="button button-primary xero-send-as-invoice" id="xero-send-invoice">%s</button><br /><br />', $post->ID, __( 'Send Invoice to Xero', 'xeroom' ) );
	echo sprintf( '<span>%s</span><br />', __( 'Send Payment to Xero' ) );
	echo sprintf( '<button data-id="%d" data-call="payment" type="button" class="button button-primary xero-send-as-invoice" id="xero-send-invoice-payment">%s</button><br /><br />', $post->ID, __( 'Send Payment to Xero', 'xeroom' ) );
	echo sprintf( '<span>%s</span><br />', __( 'Resend Invoice to Xero' ) );
	echo sprintf( '<button data-id="%d" data-call="resend-invoice" type="button" class="button button-primary xero-send-as-invoice" id="xero-resend-invoice">%s</button>', $post->ID, __( 'Resend Invoice to Xero', 'xeroom' ) );
}

add_action( 'wp_ajax_xero_send_invoice', 'xero_resend_invoice_data' );
/**
 * Resend data to Xero
 */
function xero_resend_invoice_data() {
	check_ajax_referer( 'xero-ajax', 'nonce' );

	$order_id     = absint( $_POST['invoice'] );
	$request_type = esc_attr( $_POST['call_type'] );

	if ( ! $order_id && empty( $request_type ) ) {
		wp_die( __( 'Data sending failed, please try again!', 'xeroom' ) );
	}

	$response = xero_send_invoice_data( $order_id, $request_type );

	wp_die( $response );
}

add_action( 'wp_ajax_xero_sync_stock', 'xero_sync_stock_data' );
/**
 * Sync Stock Settings
 */
function xero_sync_stock_data() {
	check_ajax_referer( 'xero-ajax', 'nonce' );

	$sync_master     = esc_attr( $_POST['master'] );
	$sync_schedule   = esc_attr( $_POST['schedule'] );
	$batch_sync_size = absint( $_POST['size'] );

	if ( empty( $batch_sync_size ) ) {
		wp_die( __( 'Please fill Batch Sync Size!', 'xeroom' ) );
	}

	$sync_data = array();
	if ( ! empty( $sync_master ) ) {
		$sync_data['sync_master'] = $sync_master;
	}

	if ( ! empty( $sync_schedule ) ) {
		$sync_data['sync_schedule'] = $sync_schedule;

		if ( ! empty( $batch_sync_size ) ) {
			$sync_data['batch_sync_size'] = $batch_sync_size;
		}

		update_xero_option( 'sync_stock_data', $sync_data );

		if ( 'n' != $sync_schedule ) {
			// Remove first, if exists
			wp_clear_scheduled_hook( 'xeroom_sync_schedule' );

			// Create new cron job
			$message = __( 'Sync job started!', 'xeroom' );
			switch ( $sync_schedule ) {
				case 'h':
					$recurrence = 'hourly';
					break;
				case 'd':
					$recurrence = 'daily';
					break;
			}

			wp_schedule_event( time(), $recurrence, 'xeroom_sync_schedule' );
		} else {
			wp_clear_scheduled_hook( 'xeroom_sync_schedule' );

			xeroom_reset_products_sync();
			xeroom_sync_products_stock();
			$message = sprintf( '%s %s <a href="%s" target="_blank">%s</a>', __( 'Manual Sync is completed.', 'xeroom' ), __( 'Please', 'xeroom' ), admin_url( 'admin.php?page=xeroom_log_woo_xero' ), __( 'check Log Files for the report.', 'xeroom' ) );
		}
	}

	wp_die( $message );
}

/**
 * Reset products sync to not sent
 */
function xeroom_reset_products_sync() {
	global $wpdb;

	$wpdb->query(
		$wpdb->prepare( "
			UPDATE $wpdb->postmeta
			SET meta_value = %d
			WHERE meta_key = %s
			",
			0, 'wpr_stock_updated'
		)
	);
}

/**
 * Send invoice data to Xero
 *
 * @param $order_id
 */
function xero_send_invoice_data( $order_id, $type ) {
	global $wpdb;

	$display_message = '';
	$xeroTime        = date( 'Y-m-d H:i:s' );
	// Xero Connectivity Api Credentials------------
	$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
	$getApiCredentials = $wpdb->get_results( $query );

	if ( $getApiCredentials ) {
		$xeroApiKey    = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
		$xeroApiSecret = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
		$salesAccount  = intval( $getApiCredentials[0]->sales_account );
		$BankCode      = intval( $getApiCredentials[0]->bank_code );

		$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );
		if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
			$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
		}

		include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
		if ( ! defined( 'BASE_PATH' ) ) {
			define( 'BASE_PATH', dirname( __FILE__ ) );
		}

		if ( ! defined( 'XERO_KEY' ) ) {
			define( 'XERO_KEY', $xeroApiKey );
		}

		if ( ! defined( 'XERO_SECRET' ) ) {
			define( 'XERO_SECRET', $xeroApiSecret );
		}

		if ( ! defined( 'PUBLIC_KEY' ) ) {
			define( 'PUBLIC_KEY', '' );
		}

		if ( ! defined( 'PRIVATE_KEY' ) ) {
			define( 'PRIVATE_KEY', '' );
		}

		if ( ! defined( 'FORMAT' ) ) {
			define( 'FORMAT', 'json' );
		}
		$oauth2 = get_xero_option( 'xero_oauth_options' );
		$xero   = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );

		$order_details = get_post( $order_id );

		if ( '' != trim( $order_details->post_content ) ) {
			$invoice_info = array(
				array(
					"InvoiceNumber" => $order_details->post_content
				)
			);

			$xero_check = $xero->Invoices( $invoice_info );

			if ( $xero_check && ! isset( $xero_check['ErrorNumber'] ) ) {
				if ( 'resend-invoice' == $type ) {
					$display_message = xeroom_sendWooInvoiceToXero( $order_id, 'manually-resend-invoice' );
				} else {
					if ( count( $xero_check['Invoices'] ) > 0 ) {
						if ( 'invoice' == $type ) {
							$display_message = __( 'The invoice is already registered in Xero.', 'xeroom' );
						}

						if ( 'payment' == $type && ( isset( $xero_check['Invoices'][0]['Invoice']['AmountPaid'] ) && 0 == $xero_check['Invoices'][0]['Invoice']['AmountPaid'] ) ) {
							$display_message = xero_send_manually_payment( $order_id );
						}
					} else {
						$display_message = xeroom_sendWooInvoiceToXero( $order_id, 'orderProduct' );
					}
				}
			} else {
				if ( isset( $xero_check['Elements'] ) && $xero_check['Elements']['DataContractBase']['InvoiceNumber'] != $order_details->post_content && 'invoice' == $type ) {
					$display_message = xeroom_sendWooInvoiceToXero( $order_id, 'orderProduct' );
				} else {
					$display_message = $xero_check['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0]['Message'];
				}

				if ( empty( $display_message ) ) {
					$display_message = 'Invoice not sent to Xero, please try again later.';
				}
			}
		} else {
			$display_message = xeroom_sendWooInvoiceToXero( $order_id, 'orderProduct' );
		}

		returnErrorMessageByXero( $order_id, $display_message, $xeroTime, 'productOrder' );
	} else {
		$mMessage = "Xeroom credentials are not set or invalid, Please contact Xeroom support team.";
		returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, 'productOrder' );
	}

	return $display_message;
}

/**
 * Send Order Payment to Xero on status Change to Complete
 *
 * @param $order_id
 * @param $status
 */
function xeroom_send_payment_on_manually_complete( $order_id, $status ) {
	$send_payment_method = get_xero_option( 'xero_send_payment_method' );
	if ( 'manually' == $send_payment_method ) {
		return;
	}

	if ( 'completed' == $status ) {
		xero_send_manually_payment( $order_id );
	}
}

/**
 * Send payment to Xero manually
 *
 * @param $order_id
 *
 * @return string
 */
function xero_send_manually_payment( $order_id ) {
	include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
	global $wpdb;
	$xeroDebugTable    = $wpdb->prefix . 'xeroom_debug';
	$exportErrorXero   = $errorMessage = "";
	$xeroTime          = date( 'Y-m-d H:i:s' );
	$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
	$getApiCredentials = $wpdb->get_results( $query );

	if ( $getApiCredentials ) {
		$xeroApiKey    = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
		$xeroApiSecret = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
		$salesAccount  = intval( $getApiCredentials[0]->sales_account );
		$BankCode      = intval( $getApiCredentials[0]->bank_code );

		$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );
		if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
			$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
		}

		$woo_gateway_payment_sending = get_xero_option( 'xero_woo_gateway_payment_send' );
		if ( $woo_gateway_payment_sending && ! array_key_exists( 'xero_' . $oder_gateway_code . '_payment_auto_send', $woo_gateway_payment_sending ) ) {
			return;
		}

		if ( ! defined( 'BASE_PATH' ) ) {
			define( 'BASE_PATH', dirname( __FILE__ ) );
		}

		if ( ! defined( 'XERO_KEY' ) ) {
			define( 'XERO_KEY', $xeroApiKey );
		}

		if ( ! defined( 'XERO_SECRET' ) ) {
			define( 'XERO_SECRET', $xeroApiSecret );
		}

		if ( ! defined( 'PUBLIC_KEY' ) ) {
			define( 'PUBLIC_KEY', '' );
		}

		if ( ! defined( 'PRIVATE_KEY' ) ) {
			define( 'PRIVATE_KEY', '' );
		}

		if ( ! defined( 'FORMAT' ) ) {
			define( 'FORMAT', 'json' );
		}

		$xero_timezone = new DateTimeZone( 'UTC' );
		$xero_time     = date( 'Y-m-d H:i:s' );
		$xero_date     = new DateTime( $xero_time, $xero_timezone );

		$oauth2        = get_xero_option( 'xero_oauth_options' );
		$xero          = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );
		$order_details = get_post( $order_id );

		$order            = new WC_Order( $order_id );
		$transaction_info = '';
		if ( $order->get_transaction_id() ) {
			$transaction_info = mb_substr( $order->get_transaction_id(), - 7, null, 'UTF-8' );
		}

		$new_payment = array(
			array(
				"Invoice"   => array(
					"InvoiceNumber" => $order_details->post_content
				),
				"Account"   => array(
					"Code" => $BankCode
				),
				"Date"      => $xero_date,
				"Amount"    => $order_details->post_mime_type,
				"Reference" => strtoupper( $oder_gateway_code ) . $transaction_info
			)
		);
		$paymentXero = $xero->Payments( $new_payment );

		$order = new WC_Order( $order_id );

		if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
			$errorMessage = $errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] . ". Please Create your bank account on xero and update bank code in xeroom setting's bank code.";
			returnErrorMessageByXero( $order_id, $errD, $xeroTime, 'productOrder' );

			// Add order note for failing to send data Xero
			$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

			$xero_order_note = apply_filters(
				'xero_add_fail_message_order',
				$xero_order_note,
				$order
			);

			$order->add_order_note( $xero_order_note );
		} else if ( isset( $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
			$errD = array();
			$errD = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
			for ( $e = 0; $e < count( $errD ); $e ++ ) {
				$errorMessage = $paymentXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
				returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, 'productOrder' );
			}

			// Add order note for failing to send data Xero
			$xero_order_note = __( 'Payment sending to Xero by Xeroom failed', 'xeroom' );

			$xero_order_note = apply_filters(
				'xero_add_fail_message_order',
				$xero_order_note,
				$order
			);

			$order->add_order_note( $xero_order_note );
		} else {
			$errorMessage = __( 'Payment sent successfully to Xero', 'xeroom' );
			update_post_meta( $order_id, 'xeroom_payment_sent', 'Sent to Xero' );

			// Add order note for successfully sent data to Xero
			$xero_order_note = __( 'The Payment has been successfully sent to Xero using Xeroom', 'xeroom' );

			$xero_order_note = apply_filters(
				'xero_add_success_message_order',
				$xero_order_note,
				$order
			);

			$order->add_order_note( $xero_order_note );
			update_post_meta( $order_id, 'xeroom_order_sent', 'Sent to Xero' );

			update_xero_option( 'xero_connection_status', 'active' );
			update_xero_option( 'xero_connection_status_message', '' );
		}
	} else {
		$mMessage = "Xeroom credentials are not set or invalid, Please contact Xeroom support team.";
		returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, 'productOrder' );
	}

	return $errorMessage;
}

/**
 * Is category associated
 *
 * @param $i
 * @param $category_id
 *
 * @return bool
 */
function xeroom_is_associated_category( $i, $category_id ) {
	$is_associated = false;

	$associated_product_categories = get_xero_option( 'xero_associate_product_categories' );

	if ( $associated_product_categories ) {
		foreach ( $associated_product_categories as $key => $value ) {
			if ( $i == $key && is_array( $value ) && array_key_exists( $category_id, $value ) ) {
				$is_associated = true;
			}
		}
	}

	return $is_associated;
}

/**
 * Get associated account to category
 *
 * @param $i
 * @param $category_id
 *
 * @return bool
 */
function xeroom_account_associated_category( $i, $category_id ) {
	$account_no = '';

	$associated_product_categories = get_xero_option( 'xero_associate_product_categories' );

	if ( $associated_product_categories ) {
		foreach ( $associated_product_categories as $key => $value ) {
			if ( $i == $key ) {
				$account_no = $value[ $category_id ];
			}
		}
	}

	return $account_no;
}

/**
 * Is shipping zone associated
 *
 * @param $i
 * @param $zone_id
 *
 * @return bool
 */
function xeroom_is_associated_shipping( $i, $zone_id ) {
	$is_associated = false;

	$associated_shipping_zones = get_xero_option( 'xero_associate_shipping_zones' );

	if ( $associated_shipping_zones ) {
		foreach ( $associated_shipping_zones as $key => $value ) {
			if ( $i == $key && is_array( $value ) && array_key_exists( $zone_id, $value ) ) {
				$is_associated = true;
			}
		}
	}

	return $is_associated;
}

/**
 * Fetch shipping account
 *
 * @param $i
 * @param $shipping_id
 *
 * @return string
 */
function xeroom_fetch_associated_shipping( $i, $shipping_id ) {
	$shipping_no = '';

	$associated_shipping = get_xero_option( 'xero_associate_shipping_zones' );

	if ( $associated_shipping ) {
		foreach ( $associated_shipping as $key => $value ) {
			if ( $i == $key ) {
				$shipping_no = $value[ $shipping_id ];
			}
		}
	}

	return $shipping_no;
}

/**
 * Fetch Associated Category Xero Account
 *
 * @param $categories
 *
 * @return string
 */
function xeroom_product_category_associated( $categories ) {
	$account_no                    = '';
	$associated_product_categories = get_xero_option( 'xero_associate_product_categories' );

	if ( $associated_product_categories ) {
		$key_val = array();
		foreach ( $associated_product_categories as $key => $value ) {
			if ( ! empty( $value ) ) {
				foreach ( $value as $sec => $acc ) {
					$key_val[ $sec ] = esc_attr( $acc );
				}
			}
		}

		if ( ! empty( $categories ) ) {
			foreach ( $categories as $keyc => $cval ) {
				if ( is_array( $key_val ) && array_key_exists( $keyc, $key_val ) ) {
					$account_no = $key_val[ $keyc ];
					break;
				}
			}
		}

	}

	return $account_no;
}

/**
 * Fetch Associated Geography Xero Account
 *
 * @param $shipping_zone
 *
 * @return string
 */
function xeroom_invoice_geography_zone( $shipping_zone ) {
	$account_no = '';

	$associated_shipping = get_xero_option( 'xero_associate_shipping_zones' );

	if ( $associated_shipping ) {
		foreach ( $associated_shipping as $key => $value ) {
			if ( is_array( $value ) && array_key_exists( $shipping_zone, $value ) ) {
				$account_no = $value[ $shipping_zone ];
			}
		}
	}

	return $account_no;
}

/**
 * Fetch first product category
 *
 * @param $product_id
 *
 * @return array
 */
function fetch_product_category_id( $product_id ) {
	$product_cat_id = array();
	$terms          = get_the_terms( $product_id, 'product_cat' );

	if ( $terms ) {
		foreach ( $terms as $term ) {
			$product_cat_id[ $term->term_id ] = $term->term_id;
		}
	}

	return $product_cat_id;
}

/**
 * Fetch shipping zone id
 *
 * @param $shipping_id
 *
 * @return null|string
 */
function xeroom_fetch_zone_id( $shipping_id ) {
	global $wpdb;
	$method_id = $wpdb->get_var( $wpdb->prepare( "SELECT zone_id FROM {$wpdb->prefix}woocommerce_shipping_zone_methods WHERE instance_id = %d", $shipping_id ) );

	return $method_id;
}

add_action( 'woocommerce_order_actions', 'xeroom_add_order_generate_credit_note' );
/**
 * Add "Generate Credit Note" to Order actions
 *
 * @param $actions
 *
 * @return mixed
 */
function xeroom_add_order_generate_credit_note( $actions ) {
	$credit_note = get_xero_option( 'xero_generate_credit_note' );

	if ( ! $credit_note ) {
		return $actions;
	}

	// add "Generate Credit Note" custom action
	$actions['xeroom_generate_credit_note'] = __( 'Generate Credit Note', 'xeroom' );

	return $actions;
}

add_action( 'woocommerce_order_action_xeroom_generate_credit_note', 'xeroom_process_generate_credit_note' );
/**
 * Generate Credit Note in Xero
 *
 * @param $order
 */
function xeroom_process_generate_credit_note( $order ) {
	global $wpdb;

	$errorType = "orderProduct";

	$order_id = $order;
	if ( is_object( $order ) ) {
		$order_id = $order->ID;
	}

	$credit_note = get_xero_option( 'xero_generate_credit_note' );

	if ( ! $credit_note ) {
		return;
	}

	// Check if is a Cart redirect - if comes from PayPal
	$url        = ( isset( $_SERVER['HTTPS'] ) && $_SERVER['HTTPS'] === 'on' ? "https" : "http" ) . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
	$url_search = parse_url( $url );

	$cart_url       = wc_get_page_permalink( 'cart' );
	$woo_url_search = parse_url( $cart_url );

	if ( $url_search['path'] == $woo_url_search['path'] ) {
		return;
	}

	$order = new WC_Order( $order_id );

	$country  = new WC_Countries();
	$orderTax = new WC_Tax();

	//define Tables
	$exportErrorXero = $xero_send_error = "";
	$xeroDebugTable  = $wpdb->prefix . 'xeroom_debug';
	$xeroLicActive   = $wpdb->prefix . "xeroom_license_key_status";
	$taxTableName    = $wpdb->prefix . "xeroom_tax";
	$xeroTime        = date( 'Y-m-d H:i:s' );

	//License Key
	$sql               = "SELECT * FROM " . $xeroLicActive . " WHERE id=1";
	$xeroLicensekeyAct = $wpdb->get_results( $sql );
	$active            = sanitize_text_field( $xeroLicensekeyAct[0]->status );
	$lic_key           = sanitize_text_field( $xeroLicensekeyAct[0]->license_key );
	$lic_method        = sanitize_text_field( $xeroLicensekeyAct[0]->xero_method );

	// Credit Note Status
	$default_credit_status       = 'SUBMITTED';
	$get_xero_credit_note_status = get_xero_option( 'xero_credit_note_status' );
	if ( $get_xero_credit_note_status ) {
		$default_credit_status = 'AUTHORISED';
	}

	//Check License of plugin
	if ( $active == 'active' ) {
		$license_checked = get_xero_option( 'xero_license_status' );
//		$url          = "https://www.xeroom.com/apidata.php?license_key=" . $lic_key;
//		$result       = wp_remote_fopen( esc_url( $url ) );
//		$validLicense = json_decode( $result, true );

		if ( $license_checked != 'expired' ) {

			//Items Details
			$totalItems    = $order->get_item_count();
			$orderCurency  = $order->get_currency();
			$usedCupons    = "Coupon used " . implode( ", ", $order->get_used_coupons() );
			$shippingPrice = $order->calculate_shipping();
			$totalDiscount = $order->get_total_discount();

			// User Address
			$shipAddress     = $order->get_address();
			$allCountry      = $country->get_countries();
			$countryFullName = $allCountry[ $shipAddress['country'] ];
			$allState        = $country->get_states( $shipAddress['country'] );
			$stateFullName   = '';
			if ( $allState ) {
				$stateFullName = $allState[ $shipAddress['state'] ];
			}

			// Prices include tax amount
			$included_tax = false;
			if ( 'yes' === get_option( 'woocommerce_prices_include_tax' ) && wc_tax_enabled() ) {
				$included_tax = true;
			}

			// Tax Rate Used
			$getTaxRate     = array(
				'country'   => $shipAddress['country'],
				'state'     => $shipAddress['state'],
				'city'      => $shipAddress['city'],
				'postcode'  => $shipAddress['postcode'],
				'tax_class' => ''
			);
			$taxRatePercent = $orderTax->find_rates( $getTaxRate );
			if ( count( $taxRatePercent ) != 0 && wc_tax_enabled() ) {
				foreach ( $taxRatePercent as $taxKey => $taxValue ) {
					$taxValue = $taxValue;
				}
				$taxOnWholeCart = $taxValue;
			} else {
				$taxOnWholeCart = array();
			}

			// Xero Connectivity Api Credentials------------
			$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
			$getApiCredentials = $wpdb->get_results( $query );
			$xeroApiKey        = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
			$xeroApiSecret     = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
			$salesAccount      = intval( $getApiCredentials[0]->sales_account );
			$BankCode          = intval( $getApiCredentials[0]->bank_code );
			$TaxCode           = sanitize_text_field( $getApiCredentials[0]->tax_code );
			$AssetCode         = intval( $getApiCredentials[0]->asset_code );
			$SoldCode          = intval( $getApiCredentials[0]->sold_code );
			$StockMaster       = sanitize_text_field( $getApiCredentials[0]->stock_master );
			$productMaster     = sanitize_text_field( $getApiCredentials[0]->product_master );

			$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );

			if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
				$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
			}

			/**
			 * Get shipping cost code
			 */
			$shippingCode = get_xero_option( 'xero_default_shipping_costs_code' );

			/**
			 * Get Use Extra Sales Accounts - Shipping
			 */
			$use_extra_sales_accounts = get_xero_option( 'xero_use_extra_sales_account' );
			if ( $order->get_shipping_methods() && $use_extra_sales_accounts && 'geography_zones' == $use_extra_sales_accounts ) {
				$zone_id = xero_fetch_order_zone_id( $order );
				if ( ! empty( $zone_id ) ) {
					$shipping_associated_code = xeroom_invoice_geography_zone( $zone_id );
					if ( ! empty( $shipping_associated_code ) ) {
						$salesAccount = $shipping_associated_code;
					}
				}
			}

			include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
			if ( ! defined( 'BASE_PATH' ) ) {
				define( 'BASE_PATH', dirname( __FILE__ ) );
			}

			if ( ! defined( 'XERO_KEY' ) ) {
				define( 'XERO_KEY', $xeroApiKey );
			}

			if ( ! defined( 'XERO_SECRET' ) ) {
				define( 'XERO_SECRET', $xeroApiSecret );
			}

			if ( ! defined( 'PUBLIC_KEY' ) ) {
				define( 'PUBLIC_KEY', '' );
			}

			if ( ! defined( 'PRIVATE_KEY' ) ) {
				define( 'PRIVATE_KEY', '' );
			}

			if ( ! defined( 'FORMAT' ) ) {
				define( 'FORMAT', 'json' );
			}
			$oauth2 = get_xero_option( 'xero_oauth_options' );
			$xero   = new Xero( $xeroApiKey, $xeroApiSecret, PUBLIC_KEY, PRIVATE_KEY, 'json', $oauth2 );

			// Xero Connectivity Api Credentials------------
			$ordersList   = $order->get_items();
			$orderDetails = array();
			$oKeyValue    = array();
			$countOrder   = 0;

			$fetch_tax_method           = get_xero_option( 'xero_tax_method' );
			$fetch_saved_tax            = get_xero_option( 'xero_tax_methods' );
			$fetch_taxes_association    = get_xero_option( 'xero_taxes_association' );
			$shipping_price_code        = get_xero_option( 'xero_shipping_price_code' );
			$shipping_price_description = get_xero_option( 'xero_shipping_price_description' );

			$xero_shipping_price_code = 'shipping_price';
			if ( $shipping_price_code ) {
				$xero_shipping_price_code = esc_attr( $shipping_price_code );
			}

			$xero_shipping_price_description = 'Shipping Price';
			if ( $shipping_price_description ) {
				$xero_shipping_price_description = esc_attr( $shipping_price_description );
			}

			$product_tax        = 'standard';
			$item_tax_rate_id   = '';
			$item_tax_rate_rate = 0;

			foreach ( $ordersList as $singleorderskey => $singleorders ) {
				$orderDetails[ $countOrder ]['name']          = $singleorders['name'];
				$orderDetails[ $countOrder ]['_product_id']   = $singleorders['product_id'];
				$orderDetails[ $countOrder ]['_variation_id'] = $singleorders['variation_id'];
				$orderDetails[ $countOrder ]['_qty']          = $singleorders['quantity'];
				$orderDetails[ $countOrder ]['item_price']    = $order->get_item_total( $singleorders, false, true );

				// Fetch Order Item Tax
				if ( $fetch_tax_method && wc_tax_enabled() ) {
					$simple_tax = get_post_meta( $singleorders['product_id'], '_tax_class', true );

					if ( $simple_tax ) {
						$product_tax = $simple_tax;
					} else {
                                $product_tax        = 'standard';
                            }
                   
					switch ( $fetch_tax_method ) {
						case "xero_simple_tax" :
							if ( $fetch_saved_tax && is_array( $fetch_saved_tax ) && array_key_exists( 'xero_' . $product_tax . '_taxmethods', $fetch_saved_tax ) ) {
								$orderDetails[ $countOrder ]['tax_class'] = xero_tax_type_code( $fetch_saved_tax[ 'xero_' . $product_tax . '_taxmethods' ] );
								$orderDetails[ $countOrder ]['total_tax'] = xero_fetch_tax_complex_amount( $singleorders->get_taxes() );
							}
							break;
						case "xero_complex_tax" :
							$tax_data = $singleorders->get_taxes();
							foreach ( $order->get_taxes() as $tax_item ) {
								$rate_id        = $tax_item->get_rate_id();
								$item_total_tax = $tax_data['total'][ $rate_id ];
								if ( isset( $item_total_tax ) && '' !== $item_total_tax ) {
									$item_tax_rate_id = $rate_id;
								}
							}

							if ( $fetch_taxes_association && is_array( $fetch_taxes_association ) && array_key_exists( $item_tax_rate_id, $fetch_taxes_association ) ) {
								$orderDetails[ $countOrder ]['tax_class'] = xero_tax_type_code( $fetch_taxes_association[ $item_tax_rate_id ] );
								$orderDetails[ $countOrder ]['total_tax'] = xero_fetch_tax_complex_amount( $singleorders->get_taxes() );
							}
							break;
					}
				}

				if ( 'w' == $productMaster ) {
					if ( wc_tax_enabled() ) {
						if( ! $fetch_tax_method ) {
						    $tax_data         = $singleorders->get_taxes();
						    $item_tax_rate_id = $tax_name = '';
						    foreach ( $order->get_taxes() as $tax_item ) {
							    $rate_id = $tax_item->get_rate_id();

							    if ( ! empty( $tax_data['total'][ $rate_id ] ) ) {
								    $item_tax_rate_rate = $tax_data['total'][ $rate_id ];
								    $tax_name           = xero_fetch_tax_rate_name( $rate_id );
								    if ( isset( $item_tax_rate_rate ) ) {
									    $item_tax_rate_id = xero_tax_type_code( $tax_name );
								    }
							    }
						    }

						    if ( ! empty( $item_tax_rate_id ) ) {
							    $orderDetails[ $countOrder ]['tax_class'] = $item_tax_rate_id;
							    if ( ! empty( $item_tax_rate_id ) ) {
								    $orderDetails[ $countOrder ]['total_tax'] = ! empty( $item_tax_rate_rate ) ? $item_tax_rate_rate : 0;
							    }
						    }
                        }
					} else {
						$orderDetails[ $countOrder ]['tax_class'] = 'NONE';
						$orderDetails[ $countOrder ]['total_tax'] = 0;
					}
				}

				$countOrder ++;
			}

			$oderList  = array();
			$uniqueTax = "";
			// Create new tax here
			if ( count( $taxOnWholeCart ) != 0 ) {
				$getAllTax = $xero->TaxRates();
				xeroom_check_connection_message( $getAllTax, $order_id );
				$taxRate = $taxOnWholeCart['rate'];
				if ( isset( $getAllTax['TaxRates']['TaxRate'] ) ) {
					foreach ( $getAllTax['TaxRates']['TaxRate'] as $allTaxesNow ) {
						if ( $allTaxesNow['Name'] == $TaxCode ) {
							$uniqueTax      = $allTaxesNow['TaxType'];
							$displayTaxRate = $allTaxesNow['DisplayTaxRate'];
							break;
						}
					}
				}

				if ( $uniqueTax == "" ) {
					foreach ( $getAllTax['TaxRates']['TaxRate'] as $allTaxesNow ) {
						if ( $allTaxesNow['TaxComponents']['TaxComponent']['Rate'] == $taxRate ) {
							$uniqueTax      = $allTaxesNow['TaxType'];
							$displayTaxRate = $allTaxesNow['DisplayTaxRate'];
							break;
						}
					}
				}
				if ( $uniqueTax == "" ) {
					$taxName     = "Default Xeroom Sales Tax";
					$query       = "SELECT * FROM `" . $taxTableName . "` WHERE `tax_rate` ='" . $taxRate . "'";
					$getTaxRates = $wpdb->get_results( $query );
					if ( count( $getTaxRates ) > 0 ) {
						$uniqueTax = $getTaxRates[0]->tax_type;
					} else {
						$xero_rate = array(
							"TaxRate" => array(
								"Name"          => "$taxName",
								"ReportTaxType" => 'OUTPUT',
								"TaxComponents" => array(
									"TaxComponent" => array(
										"Name" => "VAT",
										"Rate" => $taxRate
									)
								)
							)
						);
						$taxResult = $xero->TaxRates( $xero_rate );
						if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
							$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
							returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
						} else if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
							$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
							for ( $er = 0; $er < count( $errD ); $er ++ ) {
								$errorMessage = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $er ]['Message'];
								returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
							}
						} else if ( $taxResult['Status'] == "OK" ) {
							$uniqueTax = $taxResult['TaxRates']['TaxRate']['TaxType'];
							$wpdb->insert( $taxTableName,
								array(
									'tax_name' => sanitize_text_field( $taxResult['TaxRates']['TaxRate']['Name'] ),
									'tax_rate' => sanitize_text_field( $taxRate ),
									'tax_type' => sanitize_text_field( $taxResult['TaxRates']['TaxRate']['TaxType'] )
								)
							);
							update_xero_option( 'xero_connection_status', 'active' );
							update_xero_option( 'xero_connection_status_message', '' );
						} else {
							returnErrorMessageByXero( $order_id, $taxResult, $xeroTime, $errorType );
						}
					}
				}
			}
			// End Create new tax here
			for ( $i = 0; $i < count( $orderDetails ); $i ++ ) {
				/**
				 * Add product to associated category
				 */
				if ( $use_extra_sales_accounts && 'product_categories' == $use_extra_sales_accounts ) {
					$product_categories = fetch_product_category_id( $orderDetails[ $i ]['_product_id'] );

					$category_associated_account = xeroom_product_category_associated( $product_categories );
					if ( ! empty( $category_associated_account ) ) {
						$salesAccount = $category_associated_account;
					}
				}

				if ( isset( $orderDetails[ $i ]['_variation_id'] ) && $orderDetails[ $i ]['_variation_id'] != 0 ) {
					$unitPrice = get_post_meta( $orderDetails[ $i ]['_variation_id'] );
					$itemId    = $orderDetails[ $i ]['_variation_id'];
					$product   = wc_get_product( $orderDetails[ $i ]['_variation_id'] );

					$allProductAttr = "";

					if ( ! empty( $product->get_attribute_summary() ) ) {
						$allProductAttr .= ', ' . $product->get_attribute_summary();
					}

					if ( empty( $product->get_attribute_summary() ) ) {
						$productAttributes = get_post_meta( $orderDetails[ $i ]['_product_id'], '_product_attributes' );

						if ( $productAttributes ) {
							foreach ( $productAttributes[0] as $key => $value ) {
								$attributeName  = get_taxonomy( $value['name'] );
								$attributeName  = $attributeName->labels->name;
								$slugValue      = get_term_by( 'slug', $attributeName->name, $value['name'] );
								$allProductAttr .= ', ' . $attributeName . ': ' . $slugValue->name;
							}
						}
					}
				} else {
					$unitPrice = get_post_meta( $orderDetails[ $i ]['_product_id'] );
					$itemId    = $orderDetails[ $i ]['_product_id'];
					$product   = wc_get_product( $orderDetails[ $i ]['_product_id'] );
				}

				if ( $unitPrice['_sku'][0] != "" ) {
					$sku = $unitPrice['_sku'][0];
				} else {
					$sku = $orderDetails[ $i ]['_product_id'];
				}

				$sku = xeroom_reduce_sku_length( $sku );

				if ( isset( $unitPrice['_sale_price'][0] ) && $unitPrice['_sale_price'][0] != "" ) {
					$unitPriceNow = $unitPrice['_sale_price'][0];
					if ( $allProductAttr == "" ) {
						$allProductAttr = ', Full Price: ' . $unitPrice['_regular_price'][0];
					} else {
						$allProductAttr .= ', Full Price: ' . $unitPrice['_regular_price'][0];
					}
				} else {
					$unitPriceNow = $unitPrice['_regular_price'][0];
				}

				$checkwithoutStock = $xero->Items( $sku );
				xeroom_check_connection_message( $checkwithoutStock, $order_id );

				$unitPriceNow = $orderDetails[ $i ]['item_price'];
//				$unitPriceNow = $product->get_price_excluding_tax();
				$unitPriceNow = number_format( (float) $unitPriceNow, 2, '.', '' );
				if ( $allProductAttr != "" ) {
					$descriptionName = $orderDetails[ $i ]['name'] . $allProductAttr;
					if ( $productMaster == "w" ) {
						$oderList[ $i ]['Description'] = $orderDetails[ $i ]['name'] . $allProductAttr;
					}
				} else {
					$descriptionName = $orderDetails[ $i ]['name'];
					if ( $productMaster == "w" ) {
						$oderList[ $i ]['Description'] = $orderDetails[ $i ]['name'];
					}
				}

				$oderList[ $i ]['Quantity'] = $orderDetails[ $i ]['_qty'];
				if ( $productMaster == "w" || $productMaster == "n" ) {
					$oderList[ $i ]['UnitAmount'] = $unitPriceNow;
				}
				$oderList[ $i ]['ItemCode']     = $sku;
				$oderList[ $i ]['DiscountRate'] = 0;
				if ( $uniqueTax != "" ) {
					$oderList[ $i ]['TaxType'] = $uniqueTax;
				}
				$oderList[ $i ]['AccountCode'] = $salesAccount;
				$post_details                  = get_post( $orderDetails[ $i ]['_product_id'] );
				$xItemName                     = substr( $post_details->post_title, 0, 48 );
				if ( $xItemName == "" ) {
					$xItemName = "No Name_" . rand();
				}

				// Add tax Rate name
				if ( '' != $orderDetails[ $i ]['tax_class'] ) {
					$oderList[ $i ]['TaxType'] = $orderDetails[ $i ]['tax_class'];
					if ( $productMaster == "w" ) {
						$oderList[ $i ]['TaxAmount'] = $orderDetails[ $i ]['total_tax'];
					}
				}

				$xItemDesc  = $post_details->post_content;
				$xItemStock = $unitPrice['_stock'][0];
				/* Stock Master */
				if ( $StockMaster == "w" ) {
					if ( $xItemStock == 0 || $xItemStock == "" ) {
						$createItemWithoutStock = array(
							"Item"            => array(
								"Code"        => $sku,
								"Name"        => $xItemName,
								"Description" => $descriptionName
							),
							"SalesDetails"    => array(
								"UnitPrice"   => $unitPriceNow,
								"AccountCode" => $salesAccount
							),
							"PurchaseDetails" => array(
								"UnitPrice"       => $unitPriceNow,
								"COGSAccountCode" => $SoldCode
							)
						);

						if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
							$xero_desc = $descriptionName;
							if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
								$xero_desc = xero_product_description( $checkwithoutStock );
							}
							$createItemWithoutStock['Item']['Description'] = $xero_desc;

							$createItemWithoutStock['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
							$createItemWithoutStock['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
							$createItemWithoutStock['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
						}

						$withoutStockXero = $xero->Items( $createItemWithoutStock );
						if ( isset( $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
							$errD = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
							returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
						} else if ( isset( $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
							$errD = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
							for ( $e = 0; $e < count( $errD ); $e ++ ) {
								$errorMessage = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
								returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
							}
						}

						$oderList[ $i ]['Description'] = $descriptionName;
						if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
							$xero_desc = $descriptionName;
							if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
								$xero_desc = xero_product_description( $checkwithoutStock );
							}
							$oderList[ $i ]['Description'] = $xero_desc;
							$oderList[ $i ]['UnitAmount']  = xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;;
						}

					} else {
						$newItemsCode = array(
							"Item" => array(
								"Code"                      => $sku,
								"Name"                      => $xItemName,
								"Description"               => $descriptionName,
								"PurchaseDescription"       => $xItemName,
								"IsTrackedAsInventory"      => true,
								"IsSold"                    => true,
								"IsPurchased"               => true,
								"InventoryAssetAccountCode" => $AssetCode,
								"QuantityOnHand"            => $xItemStock,
								"TotalCostPool"             => $xItemStock * $unitPriceNow,
								"InventoryAssetAccountCode" => $AssetCode,
								"SalesDetails"              => array(
									"UnitPrice"   => $unitPriceNow,
									"AccountCode" => $salesAccount
								),
								"PurchaseDetails"           => array(
									"UnitPrice"       => $unitPriceNow,
									"COGSAccountCode" => $SoldCode
								)
							)
						);

						if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
							$xero_desc = '-';
							if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
								$xero_desc = xero_product_description( $checkwithoutStock );
							}
							$newItemsCode['Item']['Description']                  = $xero_desc;
							$newItemsCode['Item']['Name']                         = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
							$newItemsCode['Item']['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
							$newItemsCode['Item']['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
						}

						$createItem = $xero->Items( $newItemsCode );
						if ( isset( $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
							$errD = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
							returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
						} else if ( isset( $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
							$errD = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
							for ( $e = 0; $e < count( $errD ); $e ++ ) {
								$errorMessage = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
								returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
							}
						}
					}
				} else if ( $StockMaster == "x" ) {
					$xeroInventryStock = $xero->Items( $sku );
					if ( is_array( $xeroInventryStock ) ) {
						$simpleItemCode = array(
							"Item"            => array(
								"Code"        => $sku,
								"Name"        => $xItemName,
								"Description" => $descriptionName
							),
							"SalesDetails"    => array(
								"UnitPrice"   => $unitPriceNow,
								"AccountCode" => $salesAccount
							),
							"PurchaseDetails" => array(
								"UnitPrice"       => $unitPriceNow,
								"COGSAccountCode" => $SoldCode
							)
						);

						if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
							$xero_desc = '-';
							if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
								$xero_desc = xero_product_description( $checkwithoutStock );
							}
							$simpleItemCode['Item']['Description']          = $xero_desc;
							$simpleItemCode['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
							$simpleItemCode['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
							$simpleItemCode['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
						}

						$simpleXeroItem = $xero->Items( $simpleItemCode );
						if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
							$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
							returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
						} else if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
							$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
							for ( $e = 0; $e < count( $errD ); $e ++ ) {
								$errorMessage = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
								returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
							}
						}

						$oderList[ $i ]['Description'] = $descriptionName;
						if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
							$xero_desc = '-';
							if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
								$xero_desc = xero_product_description( $checkwithoutStock );
							}
							$oderList[ $i ]['Description'] = $xero_desc;
							$oderList[ $i ]['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
						}

					} else {
						$simpleItemCode = array(
							"Item"            => array(
								"Code"        => $sku,
								"Name"        => $xItemName,
								"Description" => $descriptionName
							),
							"SalesDetails"    => array(
								"UnitPrice"   => $unitPriceNow,
								"AccountCode" => $salesAccount
							),
							"PurchaseDetails" => array(
								"UnitPrice"       => $unitPriceNow,
								"COGSAccountCode" => $SoldCode
							)
						);

						if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
							$xero_desc = '-';
							if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
								$xero_desc = xero_product_description( $checkwithoutStock );
							}
							$simpleItemCode['Item']['Description']          = $xero_desc;
							$simpleItemCode['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
							$simpleItemCode['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
							$simpleItemCode['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
						}

						$simpleXeroItem = $xero->Items( $simpleItemCode );
						if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
							$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
							returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
						} else if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
							$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
							for ( $e = 0; $e < count( $errD ); $e ++ ) {
								$errorMessage = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
								returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
							}
						}
						$oderList[ $i ]['Description'] = $descriptionName;
						if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
							$xero_desc = '-';
							if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
								$xero_desc = xero_product_description( $checkwithoutStock );
							}
							$oderList[ $i ]['Description'] = $xero_desc;
							$oderList[ $i ]['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
						}
					}
				} else {
					$simpleItemsCode = array(
						"Item"            => array(
							"Code"        => $sku,
							"Name"        => $xItemName,
							"Description" => $descriptionName
						),
						"SalesDetails"    => array(
							"UnitPrice"   => $unitPriceNow,
							"AccountCode" => $salesAccount
						),
						"PurchaseDetails" => array(
							"UnitPrice"       => $unitPriceNow,
							"COGSAccountCode" => $SoldCode
						)
					);

					if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
						$xero_desc = '-';
						if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
							$xero_desc = xero_product_description( $checkwithoutStock );
						}
						$simpleItemsCode['Item']['Description']          = $xero_desc;
						$simpleItemsCode['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
						$simpleItemsCode['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
						$simpleItemsCode['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
					}

					$againSimpleXero = $xero->Items( $simpleItemsCode );
					if ( isset( $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
						$errD = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
						returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
					} else if ( isset( $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
						$errD = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
						for ( $e = 0; $e < count( $errD ); $e ++ ) {
							$errorMessage = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
							returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
						}
					}
				}
				if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
					$newItemsCodeShip = array(
						"Item" => array(
							"Code"        => $xero_shipping_price_code,
							"Description" => $xero_shipping_price_description
						)
					);
					$xero->Items( $newItemsCodeShip );
				}
				if ( $totalDiscount != 0 ) {
					$newItemsCodeDis = array(
						"Item" => array(
							"Code"        => "discount_price",
							"Description" => "Discount Price"
						)
					);
					$xero->Items( $newItemsCodeDis );
				}

				if ( $order->get_items( array( 'fee' ) ) ) {
					$newItemsCodeDis = array(
						"Item" => array(
							"Code"        => "fee_price",
							"Description" => "Fee Price"
						)
					);
					$xero->Items( $newItemsCodeDis );
				}
			}

			// Fetch shippinh tax class
			$shipping_tax = array();
			if ( $fetch_tax_method && wc_tax_enabled() ) {
				$shipping_tax_class = 'standard';
				foreach ( $order->get_items( array( 'shipping' ) ) as $item ) {
					if ( '' != $item->get_tax_class() ) {
						$shipping_tax_class = $item->get_tax_class();
					}
				}

				$simple_tax = $order->get_shipping_tax();

				if ( $simple_tax ) {
					$product_tax = $simple_tax;
				} else {
                                $product_tax        = 'standard';
                            }

				switch ( $fetch_tax_method ) {
					case "xero_simple_tax" :
						if ( $fetch_saved_tax && is_array( $fetch_saved_tax ) && array_key_exists( 'xero_' . $shipping_tax_class . '_taxmethods', $fetch_saved_tax ) ) {
							$shipping_tax['tax_class'] = xero_tax_type_code( $fetch_saved_tax[ 'xero_' . $shipping_tax_class . '_taxmethods' ] );
						}
						break;
					case "xero_complex_tax" :
						if ( $fetch_taxes_association && is_array( $fetch_taxes_association ) && array_key_exists( $product_tax, $fetch_taxes_association ) ) {
							$shipping_tax['tax_class'] = xero_tax_type_code( $fetch_taxes_association[ $product_tax ] );
						}
						break;
				}

				if ( 'inherit' == $shipping_tax_class ) {
					$shipping_class            = reset( $orderDetails );
					$shipping_tax['tax_class'] = $shipping_class['tax_class'];
				}
			} else {
				$shipping_tax['tax_class'] = 'NONE';
			}

			if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
				$oderList[ count( $orderDetails ) ]['Description']  = $xero_shipping_price_description;
				$oderList[ count( $orderDetails ) ]['Quantity']     = 1;
				$oderList[ count( $orderDetails ) ]['UnitAmount']   = $shippingPrice;
				$oderList[ count( $orderDetails ) ]['DiscountRate'] = 0;
				$oderList[ count( $orderDetails ) ]['ItemCode']     = $xero_shipping_price_code;
				if ( $taxOnWholeCart['shipping'] == "yes" ) {
					// Add tax Rate name
//					if ( $uniqueTax != "" ) {
//						$oderList[ count( $orderDetails ) ]['TaxType'] = $uniqueTax;
//					}
				}
				if ( ! empty( $shipping_tax ) ) {
					$oderList[ count( $orderDetails ) ]['TaxType'] = $shipping_tax['tax_class'];
				}
				$oderList[ count( $orderDetails ) ]['AccountCode'] = $shippingCode && '' != $shippingCode ? $shippingCode : $salesAccount;
			}
			if ( $lic_method != "trail" ) {
				if ( $totalDiscount != 0 ) {
					if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
						$nextCount = count( $orderDetails ) + 1;
					} else {
						$nextCount = count( $orderDetails );
					}
					$oderList[ $nextCount ]['Description']  = $usedCupons;
					$oderList[ $nextCount ]['Quantity']     = 1;
					$oderList[ $nextCount ]['UnitAmount']   = - $totalDiscount;
					$oderList[ $nextCount ]['DiscountRate'] = 0;
					$oderList[ $nextCount ]['ItemCode']     = "discount_price";
					// Add tax Rate name
					if ( '' != $orderDetails[ $i ]['tax_class'] ) {
						$oderList[ count( $orderDetails ) ]['TaxType'] = $orderDetails[ $i ]['tax_class'];
					}
//					if ( $uniqueTax != "" ) {
//						$oderList[ $nextCount ]['TaxType'] = $uniqueTax;
//					}
					$oderList[ $nextCount ]['AccountCode'] = $shippingCode && '' != $shippingCode ? $shippingCode : $salesAccount;
				}
			}

			// Fee tax
			$fee_tax = array();
			if ( $fetch_tax_method && wc_tax_enabled() ) {
				$fee_class            = reset( $orderDetails );
				$fee_tax['tax_class'] = $fee_class['tax_class'];
			}

			// Add fee if exists
			if ( $order->get_items( array( 'fee' ) ) ) {
				$fee_count = count( $oderList );
				foreach ( $order->get_items( array( 'fee' ) ) as $item_type => $item ) {
					$oderList[ $fee_count ]['Description'] = "Fee Price";
					$oderList[ $fee_count ]['Quantity']    = 1;
					$oderList[ $fee_count ]['UnitAmount']  = $order->get_item_total( $item, false, true );;
					$oderList[ $fee_count ]['DiscountRate'] = 0;
					$oderList[ $fee_count ]['ItemCode']     = "fee_price";
					if ( ! empty( $fee_tax ) ) {
						$oderList[ $fee_count ]['TaxType'] = $fee_tax['tax_class'];
					}

					$oderList[ $fee_count ]['AccountCode'] = $salesAccount;
					$fee_count ++;
				}
			}

			$lineItemsArray = array();
			for ( $i = 0; $i < count( $oderList ); $i ++ ) {
				$lineItemsArray[] = array( "LineItem" => array( $oderList[ $i ] ) );
			}

//			$currentDate       = xeroom_invoice_date( $order );
			$currentDate       = date( 'Y-m-d H:i:s' );
			$xero_contact_name = get_xero_option( 'xero_contact_name' );
			$xero_address_info = get_xero_option( 'xero_address_info' );

			if ( 'xeroom_use_company' == $xero_contact_name ) {
				$xero_add_contact_name = $shipAddress['company'];
			} elseif ( 'xeroom_use_email' == $xero_contact_name ) {
				$xero_add_contact_name = $shipAddress['email'];
			} else {
				$xero_add_contact_name = $shipAddress['first_name'] . ' ' . $shipAddress['last_name'];
			}

			if ( $lic_method != "trail" ) {
				$credit_note = array(
					array(
						"Type"            => "ACCRECCREDIT",
						"Contact"         => array(
							"Name"         => $xero_add_contact_name,
							"FirstName"    => $shipAddress['first_name'],
							"LastName "    => $shipAddress['last_name'],
							"EmailAddress" => $shipAddress['email'],
//							"Addresses"    => array(
//								"Address" => array(
//									array(
//										"AddressType"  => "POBOX",
//										"AttentionTo"  => "",
//										"AddressLine1" => $shipAddress['company'],
//								        "AddressLine2" => $shipAddress['address_1'],
//								        "AddressLine3" => $shipAddress['address_2'],
//										"City"         => $shipAddress['city'],
//										"Region"       => $stateFullName,
//										"Country"      => $countryFullName,
//										"PostalCode"   => $shipAddress['postcode']
//									)
//								)
//							),
							"Phones"       => array(
								"Phone" => array(
									"PhoneType"   => "MOBILE",
									"PhoneNumber" => $shipAddress['phone']
								)
							)
						),
						"Date"            => $currentDate,
						"Status"          => $default_credit_status,
						"CurrencyCode"    => $order->get_currency(),
						"LineAmountTypes" => "Exclusive",
						"Reference"       => $order_id,
						"LineItems"       => $lineItemsArray
					)
				);

				if ( 'xeroom_use_woo_address' == $xero_address_info ) {
					$new_invoice[0]['Contact']['Addresses']['Address'] = array(
						array(
							"AddressType"  => base64_decode( 'UE9CT1g=' ),
							"AttentionTo"  => "",
							"AddressLine1" => ( $xero_contact_name && 'xeroom_use_company' != $xero_contact_name ) ? $shipAddress['company'] : '',
							"AddressLine2" => $shipAddress['address_1'],
							"AddressLine3" => $shipAddress['address_2'],
							"City"         => $shipAddress['city'],
							"Region"       => $stateFullName,
							"Country"      => $countryFullName,
							"PostalCode"   => $shipAddress['postcode']
						)
					);
				}
			} else {
				$credit_note = array(
					array(
						"Type"            => "ACCRECCREDIT",
						"Contact"         => array(
							"Name"         => $xero_add_contact_name,
							"FirstName"    => $shipAddress['first_name'],
							"LastName "    => $shipAddress['last_name'],
							"EmailAddress" => $shipAddress['email'],
							"Addresses"    => array(
								"Address" => array(
									array(
										"AddressType"  => "POBOX",
										"AttentionTo"  => "Created using demo version of Xeroom",
										"AddressLine1" => "",
										"AddressLine2" => "",
										"AddressLine3" => "",
										"City"         => "",
										"Region"       => "",
										"Country"      => "",
										"PostalCode"   => ""
									)
								)
							),
							"Phones"       => array(
								"Phone" => array(
									"PhoneType"   => "MOBILE",
									"PhoneNumber" => $shipAddress['phone']
								)
							)
						),
						"Date"            => $currentDate,
						"Status"          => $default_credit_status,
						"CurrencyCode"    => $orderCurency,
						"LineAmountTypes" => "Exclusive",
						"Reference"       => $order_id,
						"LineItems"       => $lineItemsArray
					)
				);
			}

			if ( $license_checked != 'expired' ) {
				$creditNotesCreated = $xero->CreditNotes( $credit_note );

				if ( isset( $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
					$errD = $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
					returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
				} else if ( isset( $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
					$errD = $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
					for ( $e = 0; $e < count( $errD ); $e ++ ) {
						$errorMessage = $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
						returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
					}
				} else if ( $creditNotesCreated['Status'] != "OK" ) {
					returnErrorMessageByXero( $order_id, 'Credit Note was not created', $xeroTime, $errorType );
				}
				// Allocate Credit Note to Invoice No.
				if ( $creditNotesCreated['Status'] == "OK" ) {
					update_post_meta( $order_id, 'xeroom_cred_note_generated', 1 );
					update_post_meta( $order_id, 'xeroom_cred_note_id', $creditNotesCreated['CreditNotes']['CreditNote']['CreditNoteID'] );

					$message = sprintf( __( 'Xero Credit Note Generated.', 'xeroom' ) );
					$order->add_order_note( $message );

					$total_amount = $order->get_total();

					$alocateCreditNote = array(
						"Allocation" => array(
							"Invoice"       => array(
								"InvoiceID" => get_post_field( 'post_content_filtered', $order_id ),
							),
							"AppliedAmount" => $total_amount,
							"Date"          => date( 'Y-m-d' ),
						),
					);

					$sendCreditNoteAllocation = $xero->CreditNotes( $alocateCreditNote, array( 'Allocations' => $creditNotesCreated['CreditNotes']['CreditNote']['CreditNoteID'] ) );

					if ( isset( $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
						$errD = $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
						returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
					} else if ( isset( $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
						$errD = $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
						for ( $e = 0; $e < count( $errD ); $e ++ ) {
							$errorMessage = $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
							returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
						}
					} else if ( $sendCreditNoteAllocation['Status'] != "OK" ) {
						returnErrorMessageByXero( $order_id, 'Credit Note was not Allocated', $xeroTime, $errorType );
					}

					if ( $sendCreditNoteAllocation['Status'] == "OK" ) {
						update_post_meta( $order_id, 'xeroom_cred_note_allocated', 1 );

						$message = sprintf( __( 'Xero Credit Note Allocated to Invoice.', 'xeroom' ) );
						$order->add_order_note( $message );
					}

					update_xero_option( 'xero_connection_status', 'active' );
					update_xero_option( 'xero_connection_status_message', '' );
				}
			}
		} else {
			$mMessage = "Your license has expired, please contact xeroom customer support team.";
			returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
		}
	} else {
		$mMessage = "Please use a premium or trail license!";
		returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
	}
}

add_action( 'woocommerce_order_status_cancelled', 'xeroom_send_credit_note_on_order_cancellation' );
/**
 * Send Credit Note on Order Cancellation
 *
 * @param $order_id
 */
function xeroom_send_credit_note_on_order_cancellation( $order_id ) {
	if ( get_post_meta( $order_id, 'xeroom_payment_sent', true ) ) {
		xeroom_process_generate_credit_note( $order_id );
	}
}

/**
 * Determine if a given string ends with a given substring.
 *
 * @param $haystack
 * @param $needles
 *
 * @return bool
 */
function xeroom_ends_with( $haystack, $needles ) {
	foreach ( (array) $needles as $needle ) {
		if ( (string) $needle === substr( $haystack, - strlen( $needle ) ) ) {
			return true;
		}
	}

	return false;
}

/**
 * Return Xero Bank Account setting
 *
 * @param $order_gateway
 *
 * @return int
 */
function xeroom_get_bank_account_code( $order_gateway ) {
	$gateway_account = 0;
	$order_gateway   = esc_attr( $order_gateway );
	$woo_gateway     = get_xero_option( 'xero_woo_gateway' );

	if ( $woo_gateway ) {
		$gateway_account = $woo_gateway['xero_default_payment'];
		if ( is_array( $woo_gateway ) && array_key_exists( 'xero_' . $order_gateway . '_payment', $woo_gateway ) && $woo_gateway[ 'xero_' . $order_gateway . '_payment' ] > 0 ) {
			$gateway_account = $woo_gateway[ 'xero_' . $order_gateway . '_payment' ];
		}
	}

	return $gateway_account;
}

/**
 * Display Xero Settings tabs
 *
 * @param string $current ]
 */
function xero_settings_tabs( $current = 'general' ) {
	$tabs = array( 'general' => 'General', 'taxes' => 'Taxes', 'sync' => 'Global Inventory Sync' );
	echo '<div id="icon-themes" class="icon32"><br></div>';
	echo '<h2 class="nav-tab-wrapper">';
	foreach ( $tabs as $tab => $name ) {
		$class = ( $tab == $current ) ? ' nav-tab-active' : '';
		if ( ! wc_tax_enabled() && 'taxes' == $tab ) {
			continue;
		}
		echo "<a class='nav-tab$class' href='?page=add_xero_api_fields&tab=$tab'>$name</a>";
	} ?>
    <div class="button-primary xero-primery" onclick="submitLicKey();" style="margin-left: 325px !important;"><?php echo __( 'Submit', 'xeroom' ); ?></div>
	<?php echo '</h2>';
}

/**
 * Generate Xero Setting Taxes table
 *
 * @param $display_standard_tax
 */
function generate_xero_settings_tax_table( $display_standard_tax ) {
	$xero_taxes_association = get_xero_option( 'xero_taxes_association' );
	echo '<table class="wc_tax_rates wc_input_table widefat">';
	?>
    <thead>
    <tr>
        <th width="8%"><?php _e( 'Country&nbsp;code', 'woocommerce' ); ?></th>
        <th width="8%"><?php _e( 'State code', 'woocommerce' ); ?></th>
        <th width="8%"><?php _e( 'Rate&nbsp;%', 'woocommerce' ); ?></th>
        <th width="8%"><?php _e( 'Tax name', 'woocommerce' ); ?></th>
        <th width="8%"><?php _e( 'Shipping', 'woocommerce' ); ?></th>
        <th><?php _e( 'Postcode / ZIP', 'woocommerce' ); ?></th>
        <th><?php _e( 'City', 'woocommerce' ); ?></th>
        <th width="8%"><?php _e( 'Xero Tax Name', 'woocommerce' ); ?></th>
    </tr>
    </thead>
	<?php
	$xero_value = '';
	foreach ( $display_standard_tax as $taxes ) {
		if ( $xero_taxes_association && is_array( $xero_taxes_association ) && array_key_exists( $taxes->tax_rate_id, $xero_taxes_association ) ) {
			$xero_value = $xero_taxes_association[ $taxes->tax_rate_id ];
		}

		echo '<tr>';
		echo sprintf( '<td>%s</td>', $taxes->tax_rate_country );
		echo sprintf( '<td>%s</td>', $taxes->tax_rate_state );
		echo sprintf( '<td>%s</td>', $taxes->tax_rate );
		echo sprintf( '<td>%s</td>', $taxes->tax_rate_name );
		echo sprintf( '<td>%s</td>', $taxes->tax_rate_shipping );
		echo sprintf( '<td>%s</td>', ( isset( $taxes->postcode[0] ) ) ? $taxes->postcode[0] : '' );
		echo sprintf( '<td>%s</td>', ( isset( $taxes->city[0] ) ) ? $taxes->city[0] : '' );
		echo sprintf( '<td><input type="text" name="xero_name_tax_%d" value="%s" placeholder="%s" /></td>', $taxes->tax_rate_id, esc_attr( $xero_value ), __( 'Insert Xero Tax' ) );
		echo '</tr>';
	}
	echo '<tr><th colspan="8">&nbsp;</th></tr>';
	echo '</table>';
}

/**
 * Fetch rate_id for Xero Complex Tax Association
 *
 * @param $taxes
 *
 * @return int|string
 */
function xero_fetch_tax_complex_rate_id( $taxes ) {
	$rate_id = '';
	if ( ! empty( $taxes ) ) {
		foreach ( $taxes as $tax ) {
			if ( ! empty( $tax ) ) {
				foreach ( $tax as $rate => $nah ) {
					$rate_id = $rate;
					break;
				}
			}
		}
	}

	return $rate_id;
}

/**
 * Fetch total amount for Xero Complex Tax Association
 *
 * @param $taxes
 *
 * @return string
 */
function xero_fetch_tax_complex_amount( $taxes ) {
	$amount = 0;
	if ( ! empty( $taxes ) ) {
		foreach ( $taxes as $tax ) {
			if ( ! empty( $tax ) ) {
				foreach ( $tax as $rate => $val ) {
					if ( ! empty( $val ) ) {
						$amount = $val;
						break;
					}
				}
			}
		}
	}

	return $amount;
}

/**
 * Fetch Tax Type by Tax Name
 *
 * @param $tax_name
 *
 * @return string
 */
function xero_tax_type_code( $tax_name ) {
	$tax_type  = '';
	$getAllTax = get_xero_option( 'xero_defined_tax_methods' );
	if ( $getAllTax ) {
		foreach ( $getAllTax as $taxname ) {
			if ( $tax_name == $taxname['Name'] ) {
				$tax_type = $taxname['TaxType'];
				break;
			}
		}
	}

	return $tax_type;
}

/**
 * Fetch Tax Amount by Tax Name
 *
 * @param $tax_name
 *
 * @return string
 */
function xero_tax_amount_by_name( $tax_name ) {
	$tax_type  = '';
	$getAllTax = get_xero_option( 'xero_defined_tax_methods' );
	if ( $getAllTax ) {
		foreach ( $getAllTax as $taxname ) {
			if ( $tax_name == $taxname['Name'] ) {
				$tax_type = $taxname['EffectiveRate'];
				break;
			}
		}
	}

	return $tax_type;
}

/**
 * Fetch Xero Product Name
 *
 * @param $response
 *
 * @return string
 */
function xero_product_name( $response ) {
	$description = '';

	if ( ! empty( $response ) && is_array( $response ) && array_key_exists( 'Items', $response ) && isset( $response['Items']['Item']['Name'] ) ) {
		$description = $response['Items']['Item']['Name'];
	}

	return $description;
}

/**
 * Fetch Xero Product Description
 *
 * @param $response
 *
 * @return string
 */
function xero_product_description( $response ) {
	$description = '';

	if ( ! empty( $response ) && is_array( $response ) && array_key_exists( 'Items', $response ) && isset( $response['Items']['Item']['Description'] ) ) {
		$description = $response['Items']['Item']['Description'];
	}

	return $description;
}

/**
 * Fetch Xero Product Purchase
 *
 * @param $response
 *
 * @return string
 */
function xero_product_purchase_description( $response ) {
	$description = '';

	if ( ! empty( $response ) && is_array( $response ) && array_key_exists( 'Items', $response ) && isset( $response['Items']['Item']['PurchaseDescription'] ) ) {
		$description = $response['Items']['Item']['PurchaseDescription'];
	}

	return $description;
}

/**
 * Fetch Xero Product Purchase Unit Price
 *
 * @param $response
 *
 * @return string
 */
function xero_product_purchase_unit_price( $response ) {
	$unit_price = '';

	if ( ! empty( $response ) && is_array( $response ) && array_key_exists( 'Items', $response ) && isset( $response['Items']['Item']['PurchaseDetails']['UnitPrice'] ) ) {
		$unit_price = $response['Items']['Item']['PurchaseDetails']['UnitPrice'];
	}

	return $unit_price;
}

/**
 * Fetch Xero Product Sales Unit Price
 *
 * @param $response
 *
 * @return string
 */
function xero_product_sales_unit_price( $response ) {
	$unit_price = '';

	if ( ! empty( $response ) && is_array( $response ) && array_key_exists( 'Items', $response ) && isset( $response['Items']['Item']['SalesDetails']['UnitPrice'] ) ) {
		$unit_price = $response['Items']['Item']['SalesDetails']['UnitPrice'];
	}

	return $unit_price;
}

add_action( 'admin_enqueue_scripts', 'my_admin_enqueue' );
/**
 * Enqueue Admin Xero Style
 *
 * @param $hook_suffix
 */
function my_admin_enqueue( $hook_suffix ) {
	if ( 'toplevel_page_add_xero_api_fields' == $hook_suffix || 'xeroom-setting_page_xeroom_debug_page' == $hook_suffix || 'xeroom-setting_page_xeroom_export_woo_xero' == $hook_suffix || 'xeroom-setting_page_xeroom_log_woo_xero' == $hook_suffix ) {
		$apicsspath = esc_url( XEROOM_CSS_PATH . 'xeroom_style.css' );
		wp_enqueue_style( "xeroom_style", $apicsspath );
	}
}

add_filter( 'woocommerce_tax_round', 'pr14475_changed_rounding' );
/**
 * Add rounding fix
 *
 * @param $in
 *
 * @return float
 */
function pr14475_changed_rounding( $in ) {
	return round( $in, wc_get_price_decimals() );
}

add_filter( 'woocommerce_calc_tax', 'pr14475_rounding_taxes' );
/**
 * @param $taxes
 *
 * @return array
 */
function pr14475_rounding_taxes( $taxes ) {
	$taxes = array_map( 'pr14475_rounding', $taxes );

	return $taxes;
}

;
/**
 * @param $in
 *
 * @return float
 */
function pr14475_rounding( $in ) {
	return round( $in, wc_get_price_decimals() );
}

add_action( 'woocommerce_tax_rate_added', 'xeroom_save_update_tax_rates' );
add_action( 'woocommerce_tax_rate_updated', 'xeroom_save_update_tax_rates' );
/**
 * Create or update Xero Tax Rates if Master is set to WOO
 *
 * @param $rate_id
 * @param $tax_rate
 */
function xeroom_save_update_tax_rates() {
	global $wpdb;

	$xeroCredentialTable    = $wpdb->prefix . "xeroom_credentials";
	$sql                    = "SELECT xero_api_key, xero_api_secret, product_master FROM " . $xeroCredentialTable . " WHERE id=1";
	$xeroCredentialsFromTbl = $wpdb->get_results( $sql );

	$xeroApiKey    = $xeroCredentialsFromTbl[0]->xero_api_key;
	$xeroApiSecret = $xeroCredentialsFromTbl[0]->xero_api_secret;
	$productMaster = $xeroCredentialsFromTbl[0]->product_master;

	if ( 'w' == $productMaster ) {
		$errorType = "orderProduct";
		$xeroTime  = date( 'Y-m-d H:i:s' );
		// Fetch and Save Xero Tax rates if Synch Inventory Master is set on Woocommerce
		include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );

		if ( ! defined( 'BASE_PATH' ) ) {
			define( 'BASE_PATH', dirname( __FILE__ ) );
		}

		if ( ! defined( 'XERO_KEY' ) ) {
			define( 'XERO_KEY', $xeroApiKey );
		}

		if ( ! defined( 'XERO_SECRET' ) ) {
			define( 'XERO_SECRET', $xeroApiSecret );
		}

		if ( ! defined( 'PUBLIC_KEY' ) ) {
			define( 'PUBLIC_KEY', '' );
		}

		if ( ! defined( 'PRIVATE_KEY' ) ) {
			define( 'PRIVATE_KEY', '' );
		}

		if ( ! defined( 'FORMAT' ) ) {
			define( 'FORMAT', 'json' );
		}

		$oauth2   = get_xero_option( 'xero_oauth_options' );
		$xero_api = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );

		$rates = $wpdb->get_results( "SELECT tax_rate_name,tax_rate FROM `{$wpdb->prefix}woocommerce_tax_rates`" );

		if ( $rates ) {
			foreach ( $rates as $rate ) {
				$xero_rate = array(
					"TaxRate" => array(
						"Name"          => $rate->tax_rate_name,
						"Status"        => "ACTIVE",
						"ReportTaxType" => 'OUTPUT',
						"TaxComponents" => array(
							"TaxComponent" => array(
								"Name" => $rate->tax_rate_name,
								"Rate" => $rate->tax_rate
							)
						)
					)
				);
				$taxResult = $xero_api->TaxRates( $xero_rate, array( 'TaxRate' => true ) );

				if ( is_array( $taxResult ) ) {
					if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
						$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
						returnErrorMessageByXero( 0, $errD, $xeroTime, $errorType );
					} else if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
						$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
						for ( $er = 0; $er < count( $errD ); $er ++ ) {
							$errorMessage = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $er ]['Message'];
							returnErrorMessageByXero( 0, $errorMessage, $xeroTime, $errorType );
						}
					}
				} else {
					returnErrorMessageByXero( 0, "Adding Woocommerce taxes to Xero failed.", $xeroTime, $errorType );
					update_xero_option( 'xero_connection_status_message', 'Adding Woocommerce taxes to Xero failed.' );
					update_xero_option( 'xero_connection_status', 'failed' );

					return;
				}
			}

			// Update Option entry
			$getAllTax = $xero_api->TaxRates();
			if ( $getAllTax && is_array( $getAllTax ) && array_key_exists( 'TaxRates', $getAllTax ) ) {
				if ( isset( $getAllTax['TaxRates']['TaxRate'] ) ) {
					if ( is_array( $getAllTax['TaxRates']['TaxRate'] ) ) {
						$save_taxes = array();
						foreach ( $getAllTax['TaxRates']['TaxRate'] as $xero_tax ) {
							array_push(
								$save_taxes,
								array(
									'Name'          => $xero_tax['Name'],
									'TaxType'       => $xero_tax['TaxType'],
									'EffectiveRate' => $xero_tax['EffectiveRate'],
								)
							);
						}
						update_xero_option( 'xero_defined_tax_methods', $save_taxes );
					} else {
						update_xero_option( 'xero_defined_tax_methods', $getAllTax['TaxRates']['TaxRate'] );
					}
				}
				update_xero_option( 'xero_connection_status', 'active' );
				update_xero_option( 'xero_connection_status_message', '' );
			}
		}
	}
}

/**
 * Fetch tax rate name
 *
 * @param $rate_id
 *
 * @return null|string
 */
function xero_fetch_tax_rate_name( $rate_id ) {
	global $wpdb;
	$tax_rate_name = $wpdb->get_var( $wpdb->prepare( "SELECT tax_rate_name FROM {$wpdb->prefix}woocommerce_tax_rates WHERE tax_rate_id = %d", $rate_id ) );

	return $tax_rate_name;
}

/**
 * Does the Invoice exists in Xero
 *
 * @param $order_id
 * @param $xeroApiKey
 * @param $xeroApiSecret
 *
 * @return bool
 */
function xero_invoice_exists( $order_id, $xeroApiKey, $xeroApiSecret ) {
	$order_details = get_post( $order_id );

	if ( '' != trim( $order_details->post_content ) ) {
		include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
		if ( ! defined( 'BASE_PATH' ) ) {
			define( 'BASE_PATH', dirname( __FILE__ ) );
		}

		if ( ! defined( 'XERO_KEY' ) ) {
			define( 'XERO_KEY', $xeroApiKey );
		}

		if ( ! defined( 'XERO_SECRET' ) ) {
			define( 'XERO_SECRET', $xeroApiSecret );
		}

		if ( ! defined( 'PUBLIC_KEY' ) ) {
			define( 'PUBLIC_KEY', '' );
		}

		if ( ! defined( 'PRIVATE_KEY' ) ) {
			define( 'PRIVATE_KEY', '' );
		}

		if ( ! defined( 'FORMAT' ) ) {
			define( 'FORMAT', 'json' );
		}

		$oauth2   = get_xero_option( 'xero_oauth_options' );
		$xero_api = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );

		$invoice_info = array(
			array(
				"InvoiceNumber" => $order_details->post_content
			)
		);

		$xero_check = $xero_api->Invoices( $invoice_info );
		xeroom_check_connection_message( $xero_check, $order_id );

		if ( $xero_check && ! $xero_check['ErrorNumber'] ) {
			return true;
		} else {
			return false;
		}
	} else {
		return false;
	}
}

/**
 * Fetch order zone id
 *
 * @param $order
 *
 * @return mixed
 */
function xero_fetch_order_zone_id( $order ) {
	$country  = strtoupper( wc_clean( $order->get_billing_country() ) );
	$state    = strtoupper( wc_clean( $order->get_billing_state() ) );
	$postcode = wc_normalize_postcode( wc_clean( $order->get_billing_postcode() ) );

	$package = array(
		'destination' => array(
			'country'  => $country,
			'state'    => $state,
			'postcode' => $postcode,
		)
	);

	$data_store = WC_Data_Store::load( 'shipping-zone' );

	return $data_store->get_zone_id_from_package( $package );
}


add_action( 'xeroom_sync_schedule', 'xeroom_sync_products_stock' );
/**
 * Sync Products Stock
 */
function xeroom_sync_products_stock() {
	$sync_data = get_xero_option( 'sync_stock_data' );

	if ( ! empty( $sync_data ) ) {
		$sync_master = esc_attr( $sync_data['sync_master'] );
		$batch_size  = absint( $sync_data['batch_sync_size'] );
		$errorType   = "orderProduct";
		$xeroTime    = date( 'Y-m-d H:i:s' );

		global $wpdb;
		$xeroCredentialTable    = $wpdb->prefix . "xeroom_credentials";
		$sql                    = "SELECT xero_api_key, xero_api_secret, asset_code, product_master FROM " . $xeroCredentialTable . " WHERE id=1";
		$xeroCredentialsFromTbl = $wpdb->get_results( $sql );
		$xeroApiKey             = $xeroCredentialsFromTbl[0]->xero_api_key;
		$xeroApiSecret          = $xeroCredentialsFromTbl[0]->xero_api_secret;
		$asset_code             = $xeroCredentialsFromTbl[0]->asset_code;
		$product_master         = sanitize_text_field( $xeroCredentialsFromTbl[0]->product_master );

		include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
		if ( ! defined( 'BASE_PATH' ) ) {
			define( 'BASE_PATH', dirname( __FILE__ ) );
		}

		if ( ! defined( 'XERO_KEY' ) ) {
			define( 'XERO_KEY', $xeroApiKey );
		}

		if ( ! defined( 'XERO_SECRET' ) ) {
			define( 'XERO_SECRET', $xeroApiSecret );
		}

		if ( ! defined( 'PUBLIC_KEY' ) ) {
			define( 'PUBLIC_KEY', '' );
		}

		if ( ! defined( 'PRIVATE_KEY' ) ) {
			define( 'PRIVATE_KEY', '' );
		}

		if ( ! defined( 'FORMAT' ) ) {
			define( 'FORMAT', 'json' );
		}

		$oauth2   = get_xero_option( 'xero_oauth_options' );
		$xero_api = new Xero( XERO_KEY, XERO_SECRET, PUBLIC_KEY, PRIVATE_KEY, FORMAT, $oauth2 );

		$xero_items = $xero_api->Items();

		if ( ! empty( $xero_items ) && is_array( $xero_items ) && array_key_exists( 'Items', $xero_items ) ) {
			$woo_products = xeroom_fetch_products();

			switch ( $sync_master ) {
				case 'w':
					include( plugin_dir_path( __FILE__ ) . 'sync/woo-to-xero.php' );
					break;
				case 'x':
					include( plugin_dir_path( __FILE__ ) . 'sync/xero-to-woo.php' );
					break;
			}

			if ( count( $woo_products ) < $batch_size ) {
				xeroom_reset_products_sync();
			}
		} else {
			$exportErrorXero = '';
			if ( isset( $xero_items['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
				$errD = $xero_items['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
				returnErrorMessageByXero( 0, $errD, $xeroTime, $errorType );
				$exportErrorXero .= $errD;
			} else if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
				$errD = $xero_items['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
				for ( $e = 0; $e < count( $errD ); $e ++ ) {
					$errorMessage = $xero_items['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
					returnErrorMessageByXero( 0, $errorMessage, $xeroTime, $errorType );
					$exportErrorXero .= $errorMessage;
				}
			} else {
				$exportErrorXero = __( 'Please wait before retrying the Xero api' );
			}
			xeroom_generate_sync_log( array( 'xeroom_error' => maybe_serialize( $exportErrorXero ) ) );
		}
	}

}

/**
 * Generate Product Stock Sync Log
 *
 * @param $log_data
 * @param string $type
 *
 * @throws PHPExcel_Exception
 * @throws PHPExcel_Reader_Exception
 * @throws PHPExcel_Writer_Exception
 */
function xeroom_generate_sync_log( $log_data, $type = '' ) {
	if ( empty( $log_data ) || ! is_array( $log_data ) ) {
		return;
	}
	require_once( XEROOM_ROOT_PATH . 'library/Classes/PHPExcel.php' );
	require_once( XEROOM_ROOT_PATH . 'library/Classes/excel_reader.php' );

	ob_clean();
	$xeroomExcelFile = 'product-sync-list-' . date( "Ymd-H:i:s" ) . '-' . $type . '.xlsx';
	$objPHPExcel     = new PHPExcel();

	$objPHPExcel->getProperties()->setCreator( "WooCommerce" )->setLastModifiedBy( "WooCommerce" )->setTitle( "Office 2007 XLSX Product Sync List Document" )->setSubject( "Office 2007 XLSX Product Sync List Document" )->setDescription( "Product Sync List." )->setKeywords( "office 2007 openxml php" )->setCategory( "Product Sync List File" );

	if ( is_array( $log_data ) && array_key_exists( 'xeroom_error', $log_data ) ) {
		$objPHPExcel->setActiveSheetIndex( 0 )
		            ->setCellValue( 'A1', 'Error' )
		            ->setCellValue( 'B1', $log_data['xeroom_error'] );
	} else {
		$objPHPExcel->setActiveSheetIndex( 0 )
		            ->setCellValue( 'A1', 'S.No' )
		            ->setCellValue( 'B1', 'Product SKU' )
		            ->setCellValue( 'C1', 'Stock' )
		            ->setCellValue( 'D1', 'Sync Status' );

		$it = 2;
		foreach ( $log_data as $item ) {
			$objPHPExcel->setActiveSheetIndex( 0 )
			            ->setCellValue( "A" . $it, $it - 1 )
			            ->setCellValue( "B" . $it, $item['sku'] )
			            ->setCellValue( "C" . $it, $item['stock'] )
			            ->setCellValue( "D" . $it, $item['status'] );
			$it ++;
		}
	}

	$objPHPExcel->getActiveSheet()->setTitle( 'Xero Product Sync List' );
	$objPHPExcel->setActiveSheetIndex( 0 );
	$objWriter = PHPExcel_IOFactory::createWriter( $objPHPExcel, 'Excel2007' );
	$objWriter->save( XEROOM_LOG_PATH . $xeroomExcelFile );
}

/**
 * Fetch Woo Product Stock & SKU
 * who is not sync yet
 *
 * @return array
 */
function xeroom_fetch_products() {
	$sync_data  = get_xero_option( 'sync_stock_data' );
	$batch_size = 100;
	if ( ! empty( $sync_data ) ) {
		$batch_size = absint( $sync_data['batch_sync_size'] );
	}

	$args = array(
		'post_type'      => 'product',
		'posts_per_page' => $batch_size,
		'post_status'    => 'publish',
		'meta_query'     => array(
			'relation' => 'OR',
			array(
				'key'     => 'wpr_stock_updated',
				'compare' => 'NOT EXISTS',
			),
			array(
				'key'     => 'wpr_stock_updated',
				'value'   => 0,
				'compare' => '=',
			),
		)
	);

	$get_products = new WP_Query( $args );

	$product_data = array();
	if ( $get_products->get_posts() ) {
		foreach ( $get_products->get_posts() as $post ) {
			$product = wc_get_product( $post->ID );

			if ( $product->is_type( 'variable' ) ) {
				$variations    = $product->get_available_variations();
				$variable_data = array();
				foreach ( $variations as $variation ) {
					$variable_data['product_id']    = $variation['variation_id'];
					$variation_obj                  = new WC_Product_variation( $variation['variation_id'] );
					$variable_data['product_stock'] = $variation_obj->get_stock_quantity();
					$variable_data['product_sku']   = $variation_obj->get_sku();

					array_push( $product_data, $variable_data );
				}
			}

			array_push(
				$product_data,
				array(
					'product_id'    => $post->ID,
					'product_stock' => $product->get_stock_quantity(),
					'product_sku'   => $product->get_sku(),
				)
			);
		}
	}
	wp_reset_query();

	return $product_data;
}

/**
 * Fetch Active Tracking
 *
 * @param $tracking_response
 *
 * @return array|bool
 */
function xeroom_fetch_tracking_categories( $tracking_response ) {
	if ( null === $tracking_response ) {
		return false;
	}

	if ( ! isset( $tracking_response['TrackingCategories'] ) ) {
		return false;
	}

	if ( ! is_array( $tracking_response ) && ! array_key_exists( 'TrackingCategories', $tracking_response ) && ! empty( $tracking_response['TrackingCategories']['TrackingCategory'] ) ) {
		return false;
	}

	$trackings = array();
	$cat_num   = count( $tracking_response['TrackingCategories']['TrackingCategory'] );

	for ( $u = 0; $u < $cat_num; $u ++ ) {
		if ( isset( $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Status'] ) && 'ACTIVE' == $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Status'] ) {
			if ( ! empty( $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Options']['Option'] ) ) {
				for ( $o = 0; $o < count( $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Options']['Option'] ); $o ++ ) {
					if ( isset( $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Options']['Option'][ $o ]['Status'] ) && 'ACTIVE' == $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Options']['Option'][ $o ]['Status'] ) {
						$trackings[ $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Name'] ][] = $tracking_response['TrackingCategories']['TrackingCategory'][ $u ]['Options']['Option'][ $o ]['Name'];
					}
				}
			}
		}
	}

	return $trackings;
}

add_action( 'wp_ajax_get_orders_for_xero', 'xeroom_get_orders_for_xero' );
/**
 * Fetch orders ID's
 */
function xeroom_get_orders_for_xero() {
	$batch_size     = ! empty( $_REQUEST['orders_no'] ) ? $_REQUEST['orders_no'] : 100;
	$xeroom_Payment = $_REQUEST['payment'];

	if ( 'Paid' == $xeroom_Payment ) {
		$xeroomOrderStatus = array( 'wc-completed' );
	} else {
		$xeroomOrderStatus = array( 'wc-on-hold', 'wc-processing', 'wc-pending' );
	}

	$args  = array(
		'posts_per_page' => absint( $batch_size ),
		'post_type'      => wc_get_order_types(),
		'post_status'    => $xeroomOrderStatus,
		'orderby'        => 'ID',
		'order'          => 'ASC'
	);
	$query = new WP_Query( $args );

	$order_ids = array();
	if ( $query->posts ) {
		foreach ( $query->posts as $key => $xeroomOrdersValue ) {
			$order_ids[] = $xeroomOrdersValue->ID;
		}
	}

	echo wp_json_encode( $order_ids );
	wp_die();
}

add_action( 'woocommerce_order_refunded', 'xero_order_refunded', 10, 2 );
/**
 * Refund the Invoice
 *
 * @param $order_id
 * @param $refund_id
 */
function xero_order_refunded( $order_id, $refund_id ) {
	global $wpdb;

	if ( ! get_post_meta( $order_id, 'xeroom_payment_sent', true ) ) {
		$xeroCredentialTable    = $wpdb->prefix . "xeroom_credentials";
		$sql                    = "SELECT xero_api_key, xero_api_secret, asset_code, product_master FROM " . $xeroCredentialTable . " WHERE id=1";
		$xeroCredentialsFromTbl = $wpdb->get_results( $sql );
		$xeroApiKey             = $xeroCredentialsFromTbl[0]->xero_api_key;
		$xeroApiSecret          = $xeroCredentialsFromTbl[0]->xero_api_secret;

		if ( xero_invoice_exists( $order_id, $xeroApiKey, $xeroApiSecret ) ) {
			$order_details = get_post( $order_id );
			$oauth2        = get_xero_option( 'xero_oauth_options' );
			$xero_api      = new Xero( $xeroApiKey, $xeroApiSecret, '', '', 'json', $oauth2 );

			$invoice_info = array(
				array(
					"InvoiceNumber" => $order_details->post_content,
					"Status"        => 'VOIDED',
				)
			);

			$xero_check = $xero_api->Invoices( $invoice_info );
			xeroom_check_connection_message( $xero_check, $order_id );

			if ( $xero_check && ! $xero_check['ErrorNumber'] ) {

			} else {

			}
		}
	} else {
		$errorType = "orderProduct";
		$order     = new WC_Order( $order_id );

		$country  = new WC_Countries();
		$orderTax = new WC_Tax();

		//define Tables
		$exportErrorXero = $xero_send_error = "";
		$xeroDebugTable  = $wpdb->prefix . 'xeroom_debug';
		$xeroLicActive   = $wpdb->prefix . "xeroom_license_key_status";
		$taxTableName    = $wpdb->prefix . "xeroom_tax";
		$xeroTime        = date( 'Y-m-d H:i:s' );

		//License Key
		$sql               = "SELECT * FROM " . $xeroLicActive . " WHERE id=1";
		$xeroLicensekeyAct = $wpdb->get_results( $sql );
		$active            = sanitize_text_field( $xeroLicensekeyAct[0]->status );
		$lic_key           = sanitize_text_field( $xeroLicensekeyAct[0]->license_key );
		$lic_method        = sanitize_text_field( $xeroLicensekeyAct[0]->xero_method );

		// Credit Note Status
		$default_credit_status = 'AUTHORISED';

		//Check License of plugin
		if ( $active == 'active' ) {
			$license_checked = get_xero_option( 'xero_license_status' );

			if ( $license_checked != 'expired' ) {
				$totalItems    = $order->get_item_count();
				$orderCurency  = $order->get_currency();
				$usedCupons    = "Coupon used " . implode( ", ", $order->get_used_coupons() );
				$shippingPrice = $order->calculate_shipping();
				$totalDiscount = $order->get_total_discount();

				// User Address
				$shipAddress     = $order->get_address();
				$allCountry      = $country->get_countries();
				$countryFullName = $allCountry[ $shipAddress['country'] ];
				$allState        = $country->get_states( $shipAddress['country'] );
				$stateFullName   = '';
				if ( $allState ) {
					$stateFullName = $allState[ $shipAddress['state'] ];
				}

				// Prices include tax amount
				$included_tax = false;
				if ( 'yes' === get_option( 'woocommerce_prices_include_tax' ) && wc_tax_enabled() ) {
					$included_tax = true;
				}

				// Tax Rate Used
				$getTaxRate     = array(
					'country'   => $shipAddress['country'],
					'state'     => $shipAddress['state'],
					'city'      => $shipAddress['city'],
					'postcode'  => $shipAddress['postcode'],
					'tax_class' => ''
				);
				$taxRatePercent = $orderTax->find_rates( $getTaxRate );
				if ( count( $taxRatePercent ) != 0 && wc_tax_enabled() ) {
					foreach ( $taxRatePercent as $taxKey => $taxValue ) {
						$taxValue = $taxValue;
					}
					$taxOnWholeCart = $taxValue;
				} else {
					$taxOnWholeCart = array();
				}

				// Xero Connectivity Api Credentials------------
				$query             = "SELECT *  FROM `" . $wpdb->prefix . "xeroom_credentials` WHERE `id`=1";
				$getApiCredentials = $wpdb->get_results( $query );
				$xeroApiKey        = sanitize_text_field( $getApiCredentials[0]->xero_api_key );
				$xeroApiSecret     = sanitize_text_field( $getApiCredentials[0]->xero_api_secret );
				$salesAccount      = intval( $getApiCredentials[0]->sales_account );
				$BankCode          = intval( $getApiCredentials[0]->bank_code );
				$TaxCode           = sanitize_text_field( $getApiCredentials[0]->tax_code );
				$AssetCode         = intval( $getApiCredentials[0]->asset_code );
				$SoldCode          = intval( $getApiCredentials[0]->sold_code );
				$StockMaster       = sanitize_text_field( $getApiCredentials[0]->stock_master );
				$productMaster     = sanitize_text_field( $getApiCredentials[0]->product_master );

				$oder_gateway_code = get_post_meta( $order_id, '_payment_method', true );

				if ( xeroom_get_bank_account_code( $oder_gateway_code ) > 0 ) {
					$BankCode = xeroom_get_bank_account_code( $oder_gateway_code );
				}

				/**
				 * Get shipping cost code
				 */
				$shippingCode = get_xero_option( 'xero_default_shipping_costs_code' );

				/**
				 * Get Use Extra Sales Accounts - Shipping
				 */
				$use_extra_sales_accounts = get_xero_option( 'xero_use_extra_sales_account' );
				if ( $order->get_shipping_methods() && $use_extra_sales_accounts && 'geography_zones' == $use_extra_sales_accounts ) {
					$zone_id = xero_fetch_order_zone_id( $order );
					if ( ! empty( $zone_id ) ) {
						$shipping_associated_code = xeroom_invoice_geography_zone( $zone_id );
						if ( ! empty( $shipping_associated_code ) ) {
							$salesAccount = $shipping_associated_code;
						}
					}
				}

				include_once( XEROOM_ROOT_PATH . "library/xeroom_indexManager.php" );
				if ( ! defined( 'BASE_PATH' ) ) {
					define( 'BASE_PATH', dirname( __FILE__ ) );
				}

				if ( ! defined( 'XERO_KEY' ) ) {
					define( 'XERO_KEY', $xeroApiKey );
				}

				if ( ! defined( 'XERO_SECRET' ) ) {
					define( 'XERO_SECRET', $xeroApiSecret );
				}

				if ( ! defined( 'PUBLIC_KEY' ) ) {
					define( 'PUBLIC_KEY', '' );
				}

				if ( ! defined( 'PRIVATE_KEY' ) ) {
					define( 'PRIVATE_KEY', '' );
				}

				if ( ! defined( 'FORMAT' ) ) {
					define( 'FORMAT', 'json' );
				}
				$oauth2 = get_xero_option( 'xero_oauth_options' );
				$xero   = new Xero( $xeroApiKey, $xeroApiSecret, PUBLIC_KEY, PRIVATE_KEY, 'json', $oauth2 );

				// Xero Connectivity Api Credentials------------
				$ordersList   = $order->get_items();
				$orderDetails = array();
				$oKeyValue    = array();
				$countOrder   = 0;

				$fetch_tax_method           = get_xero_option( 'xero_tax_method' );
				$fetch_saved_tax            = get_xero_option( 'xero_tax_methods' );
				$fetch_taxes_association    = get_xero_option( 'xero_taxes_association' );
				$shipping_price_code        = get_xero_option( 'xero_shipping_price_code' );
				$shipping_price_description = get_xero_option( 'xero_shipping_price_description' );

				$xero_shipping_price_code = 'shipping_price';
				if ( $shipping_price_code ) {
					$xero_shipping_price_code = esc_attr( $shipping_price_code );
				}

				$xero_shipping_price_description = 'Shipping Price';
				if ( $shipping_price_description ) {
					$xero_shipping_price_description = esc_attr( $shipping_price_description );
				}

				$product_tax        = 'standard';
				$item_tax_rate_id   = '';
				$item_tax_rate_rate = 0;

				$refund = new WC_Order_Refund( $refund_id );

				if ( $refund ) {
					foreach ( $refund->get_items() as $item_id => $singleorders ) {
						$orderDetails[ $countOrder ]['name']          = $singleorders['name'];
						$orderDetails[ $countOrder ]['_product_id']   = $singleorders['product_id'];
						$orderDetails[ $countOrder ]['_variation_id'] = $singleorders['variation_id'];
						$orderDetails[ $countOrder ]['_qty']          = abs( $singleorders['quantity'] );
						$orderDetails[ $countOrder ]['item_price']    = abs( $order->get_item_total( $singleorders, false, true ) );

						// Fetch Order Item Tax
						if ( $fetch_tax_method && wc_tax_enabled() ) {
							$simple_tax = get_post_meta( $singleorders['product_id'], '_tax_class', true );

							if ( $simple_tax ) {
								$product_tax = $simple_tax;
							} else {
                                $product_tax        = 'standard';
                            }
							switch ( $fetch_tax_method ) {
								case "xero_simple_tax" :
									if ( $fetch_saved_tax && is_array( $fetch_saved_tax ) && array_key_exists( 'xero_' . $product_tax . '_taxmethods', $fetch_saved_tax ) ) {
										$orderDetails[ $countOrder ]['tax_class'] = xero_tax_type_code( $fetch_saved_tax[ 'xero_' . $product_tax . '_taxmethods' ] );
										$orderDetails[ $countOrder ]['total_tax'] = abs( xero_fetch_tax_complex_amount( $singleorders->get_taxes() ) );
									}
									break;
								case "xero_complex_tax" :
									$tax_data = $singleorders->get_taxes();
									foreach ( $order->get_taxes() as $tax_item ) {
										$rate_id        = $tax_item->get_rate_id();
										$item_total_tax = $tax_data['total'][ $rate_id ];
										if ( isset( $item_total_tax ) && '' !== $item_total_tax ) {
											$item_tax_rate_id = $rate_id;
										}
									}

									if ( $fetch_taxes_association && is_array( $fetch_taxes_association ) && array_key_exists( $item_tax_rate_id, $fetch_taxes_association ) ) {
										$orderDetails[ $countOrder ]['tax_class'] = xero_tax_type_code( $fetch_taxes_association[ $item_tax_rate_id ] );
										$orderDetails[ $countOrder ]['total_tax'] = abs( xero_fetch_tax_complex_amount( $singleorders->get_taxes() ) );
									}
									break;
							}
						}

						if ( 'w' == $productMaster ) {
							if ( wc_tax_enabled() ) {
								 if( ! $fetch_tax_method ) {
								    $tax_data         = $singleorders->get_taxes();
								    $item_tax_rate_id = $tax_name = '';
								    foreach ( $order->get_taxes() as $tax_item ) {
									    $rate_id = $tax_item->get_rate_id();

									    if ( ! empty( $tax_data['total'][ $rate_id ] ) ) {
										    $item_tax_rate_rate = $tax_data['total'][ $rate_id ];
										    $tax_name           = xero_fetch_tax_rate_name( $rate_id );
										    if ( isset( $item_tax_rate_rate ) ) {
											    $item_tax_rate_id = xero_tax_type_code( $tax_name );
										    }
									    }
								    }

								    if ( ! empty( $item_tax_rate_id ) ) {
									    $orderDetails[ $countOrder ]['tax_class'] = $item_tax_rate_id;
									    if ( ! empty( $item_tax_rate_id ) ) {
										    $orderDetails[ $countOrder ]['total_tax'] = ! empty( $item_tax_rate_rate ) ? abs( $item_tax_rate_rate ) : 0;
									    }
								    }
                                }
							} else {
								$orderDetails[ $countOrder ]['tax_class'] = 'NONE';
								$orderDetails[ $countOrder ]['total_tax'] = 0;
							}
						}

						$countOrder ++;
					}

					$oderList  = array();
					$uniqueTax = "";
					// Create new tax here
					if ( count( $taxOnWholeCart ) != 0 ) {
						$getAllTax = $xero->TaxRates();
						xeroom_check_connection_message( $getAllTax, $order_id );
						$taxRate = $taxOnWholeCart['rate'];
						if ( isset( $getAllTax['TaxRates']['TaxRate'] ) ) {
							foreach ( $getAllTax['TaxRates']['TaxRate'] as $allTaxesNow ) {
								if ( $allTaxesNow['Name'] == $TaxCode ) {
									$uniqueTax      = $allTaxesNow['TaxType'];
									$displayTaxRate = $allTaxesNow['DisplayTaxRate'];
									break;
								}
							}
						}

						if ( $uniqueTax == "" ) {
							foreach ( $getAllTax['TaxRates']['TaxRate'] as $allTaxesNow ) {
								if ( $allTaxesNow['TaxComponents']['TaxComponent']['Rate'] == $taxRate ) {
									$uniqueTax      = $allTaxesNow['TaxType'];
									$displayTaxRate = $allTaxesNow['DisplayTaxRate'];
									break;
								}
							}
						}
						if ( $uniqueTax == "" ) {
							$taxName     = "Default Xeroom Sales Tax";
							$query       = "SELECT * FROM `" . $taxTableName . "` WHERE `tax_rate` ='" . $taxRate . "'";
							$getTaxRates = $wpdb->get_results( $query );
							if ( count( $getTaxRates ) > 0 ) {
								$uniqueTax = $getTaxRates[0]->tax_type;
							} else {
								$xero_rate = array(
									"TaxRate" => array(
										"Name"          => "$taxName",
										"ReportTaxType" => 'OUTPUT',
										"TaxComponents" => array(
											"TaxComponent" => array(
												"Name" => "VAT",
												"Rate" => $taxRate
											)
										)
									)
								);
								$taxResult = $xero->TaxRates( $xero_rate );
								if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
								} else if ( isset( $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $er = 0; $er < count( $errD ); $er ++ ) {
										$errorMessage = $taxResult['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $er ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
									}
								} else if ( $taxResult['Status'] == "OK" ) {
									$uniqueTax = $taxResult['TaxRates']['TaxRate']['TaxType'];
									$wpdb->insert( $taxTableName,
										array(
											'tax_name' => sanitize_text_field( $taxResult['TaxRates']['TaxRate']['Name'] ),
											'tax_rate' => sanitize_text_field( $taxRate ),
											'tax_type' => sanitize_text_field( $taxResult['TaxRates']['TaxRate']['TaxType'] )
										)
									);

									update_xero_option( 'xero_connection_status', 'active' );
									update_xero_option( 'xero_connection_status_message', '' );
								} else {
									returnErrorMessageByXero( $order_id, $taxResult, $xeroTime, $errorType );
								}
							}
						}
					}

					for ( $i = 0; $i < count( $orderDetails ); $i ++ ) {
						/**
						 * Add product to associated category
						 */
						if ( $use_extra_sales_accounts && 'product_categories' == $use_extra_sales_accounts ) {
							$product_categories = fetch_product_category_id( $orderDetails[ $i ]['_product_id'] );

							$category_associated_account = xeroom_product_category_associated( $product_categories );
							if ( ! empty( $category_associated_account ) ) {
								$salesAccount = $category_associated_account;
							}
						}

						if ( isset( $orderDetails[ $i ]['_variation_id'] ) && $orderDetails[ $i ]['_variation_id'] != 0 ) {
							$unitPrice = get_post_meta( $orderDetails[ $i ]['_variation_id'] );
							$itemId    = $orderDetails[ $i ]['_variation_id'];
							$product   = wc_get_product( $orderDetails[ $i ]['_variation_id'] );

							$allProductAttr = "";

							if ( ! empty( $product->get_attribute_summary() ) ) {
								$allProductAttr .= ', ' . $product->get_attribute_summary();
							}

							if ( empty( $product->get_attribute_summary() ) ) {
								$productAttributes = get_post_meta( $orderDetails[ $i ]['_product_id'], '_product_attributes' );

								if ( $productAttributes ) {
									foreach ( $productAttributes[0] as $key => $value ) {
										$attributeName  = get_taxonomy( $value['name'] );
										$attributeName  = $attributeName->labels->name;
										$slugValue      = get_term_by( 'slug', $attributeName->name, $value['name'] );
										$allProductAttr .= ', ' . $attributeName . ': ' . $slugValue->name;
									}
								}
							}
						} else {
							$unitPrice = get_post_meta( $orderDetails[ $i ]['_product_id'] );
							$itemId    = $orderDetails[ $i ]['_product_id'];
							$product   = wc_get_product( $orderDetails[ $i ]['_product_id'] );
						}

						if ( $unitPrice['_sku'][0] != "" ) {
							$sku = $unitPrice['_sku'][0];
						} else {
							$sku = $orderDetails[ $i ]['_product_id'];
						}

						$sku = xeroom_reduce_sku_length( $sku );

						if ( isset( $unitPrice['_sale_price'][0] ) && $unitPrice['_sale_price'][0] != "" ) {
							$unitPriceNow = $unitPrice['_sale_price'][0];
							if ( $allProductAttr == "" ) {
								$allProductAttr = ', Full Price: ' . $unitPrice['_regular_price'][0];
							} else {
								$allProductAttr .= ', Full Price: ' . $unitPrice['_regular_price'][0];
							}
						} else {
							$unitPriceNow = $unitPrice['_regular_price'][0];
						}

						$checkwithoutStock = $xero->Items( $sku );
						xeroom_check_connection_message( $checkwithoutStock, $order_id );

						$unitPriceNow = $orderDetails[ $i ]['item_price'];
//				$unitPriceNow = $product->get_price_excluding_tax();
						$unitPriceNow = number_format( (float) $unitPriceNow, 2, '.', '' );
						if ( $allProductAttr != "" ) {
							$descriptionName = $orderDetails[ $i ]['name'] . $allProductAttr;
							if ( $productMaster == "w" ) {
								$oderList[ $i ]['Description'] = $orderDetails[ $i ]['name'] . $allProductAttr;
							}
						} else {
							$descriptionName = $orderDetails[ $i ]['name'];
							if ( $productMaster == "w" ) {
								$oderList[ $i ]['Description'] = $orderDetails[ $i ]['name'];
							}
						}

						$oderList[ $i ]['Quantity'] = $orderDetails[ $i ]['_qty'];
						if ( $productMaster == "w" || $productMaster == "n" ) {
							$oderList[ $i ]['UnitAmount'] = $unitPriceNow;
						}
						$oderList[ $i ]['ItemCode']     = $sku;
						$oderList[ $i ]['DiscountRate'] = 0;
						if ( $uniqueTax != "" ) {
							$oderList[ $i ]['TaxType'] = $uniqueTax;
						}
						$oderList[ $i ]['AccountCode'] = $salesAccount;
						$post_details                  = get_post( $orderDetails[ $i ]['_product_id'] );
						$xItemName                     = substr( $post_details->post_title, 0, 48 );
						if ( $xItemName == "" ) {
							$xItemName = "No Name_" . rand();
						}

						// Add tax Rate name
						if ( '' != $orderDetails[ $i ]['tax_class'] ) {
							$oderList[ $i ]['TaxType'] = $orderDetails[ $i ]['tax_class'];
							if ( $productMaster == "w" ) {
								$oderList[ $i ]['TaxAmount'] = $orderDetails[ $i ]['total_tax'];
							}
						}

						$xItemDesc  = $post_details->post_content;
						$xItemStock = $unitPrice['_stock'][0];
						/* Stock Master */
						if ( $StockMaster == "w" ) {
							if ( $xItemStock == 0 || $xItemStock == "" ) {
								$createItemWithoutStock = array(
									"Item"            => array(
										"Code"        => $sku,
										"Name"        => $xItemName,
										"Description" => $descriptionName
									),
									"SalesDetails"    => array(
										"UnitPrice"   => $unitPriceNow,
										"AccountCode" => $salesAccount
									),
									"PurchaseDetails" => array(
										"UnitPrice"       => $unitPriceNow,
										"COGSAccountCode" => $SoldCode
									)
								);

								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = $descriptionName;
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$createItemWithoutStock['Item']['Description'] = $xero_desc;

									$createItemWithoutStock['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
									$createItemWithoutStock['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									$createItemWithoutStock['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
								}

								$withoutStockXero = $xero->Items( $createItemWithoutStock );
								if ( isset( $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
								} else if ( isset( $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $e = 0; $e < count( $errD ); $e ++ ) {
										$errorMessage = $withoutStockXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
									}
								}

								$oderList[ $i ]['Description'] = $descriptionName;
								if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = $descriptionName;
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$oderList[ $i ]['Description'] = $xero_desc;
									$oderList[ $i ]['UnitAmount']  = xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;;
								}

							} else {
								$newItemsCode = array(
									"Item" => array(
										"Code"                      => $sku,
										"Name"                      => $xItemName,
										"Description"               => $descriptionName,
										"PurchaseDescription"       => $xItemName,
										"IsTrackedAsInventory"      => true,
										"IsSold"                    => true,
										"IsPurchased"               => true,
										"InventoryAssetAccountCode" => $AssetCode,
										"QuantityOnHand"            => $xItemStock,
										"TotalCostPool"             => $xItemStock * $unitPriceNow,
										"InventoryAssetAccountCode" => $AssetCode,
										"SalesDetails"              => array(
											"UnitPrice"   => $unitPriceNow,
											"AccountCode" => $salesAccount
										),
										"PurchaseDetails"           => array(
											"UnitPrice"       => $unitPriceNow,
											"COGSAccountCode" => $SoldCode
										)
									)
								);

								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$newItemsCode['Item']['Description']                  = $xero_desc;
									$newItemsCode['Item']['Name']                         = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
									$newItemsCode['Item']['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									$newItemsCode['Item']['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
								}

								$createItem = $xero->Items( $newItemsCode );
								if ( isset( $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
								} else if ( isset( $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $e = 0; $e < count( $errD ); $e ++ ) {
										$errorMessage = $createItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
									}
								}
							}
						} else if ( $StockMaster == "x" ) {
							$xeroInventryStock = $xero->Items( $sku );
							if ( is_array( $xeroInventryStock ) ) {
								$simpleItemCode = array(
									"Item"            => array(
										"Code"        => $sku,
										"Name"        => $xItemName,
										"Description" => $descriptionName
									),
									"SalesDetails"    => array(
										"UnitPrice"   => $unitPriceNow,
										"AccountCode" => $salesAccount
									),
									"PurchaseDetails" => array(
										"UnitPrice"       => $unitPriceNow,
										"COGSAccountCode" => $SoldCode
									)
								);

								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$simpleItemCode['Item']['Description']          = $xero_desc;
									$simpleItemCode['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
									$simpleItemCode['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									$simpleItemCode['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
								}

								$simpleXeroItem = $xero->Items( $simpleItemCode );
								if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
								} else if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $e = 0; $e < count( $errD ); $e ++ ) {
										$errorMessage = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
									}
								}

								$oderList[ $i ]['Description'] = $descriptionName;
								if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$oderList[ $i ]['Description'] = $xero_desc;
									$oderList[ $i ]['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
								}

							} else {
								$simpleItemCode = array(
									"Item"            => array(
										"Code"        => $sku,
										"Name"        => $xItemName,
										"Description" => $descriptionName
									),
									"SalesDetails"    => array(
										"UnitPrice"   => $unitPriceNow,
										"AccountCode" => $salesAccount
									),
									"PurchaseDetails" => array(
										"UnitPrice"       => $unitPriceNow,
										"COGSAccountCode" => $SoldCode
									)
								);

								if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$simpleItemCode['Item']['Description']          = $xero_desc;
									$simpleItemCode['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
									$simpleItemCode['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
									$simpleItemCode['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
								}

								$simpleXeroItem = $xero->Items( $simpleItemCode );
								if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
									$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
									returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
								} else if ( isset( $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
									$errD = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
									for ( $e = 0; $e < count( $errD ); $e ++ ) {
										$errorMessage = $simpleXeroItem['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
										returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
									}
								}
								$oderList[ $i ]['Description'] = $descriptionName;
								if ( $productMaster == "x" && isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
									$xero_desc = '-';
									if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
										$xero_desc = xero_product_description( $checkwithoutStock );
									}
									$oderList[ $i ]['Description'] = $xero_desc;
									$oderList[ $i ]['UnitAmount']  = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
								}
							}
						} else {
							$simpleItemsCode = array(
								"Item"            => array(
									"Code"        => $sku,
									"Name"        => $xItemName,
									"Description" => $descriptionName
								),
								"SalesDetails"    => array(
									"UnitPrice"   => $unitPriceNow,
									"AccountCode" => $salesAccount
								),
								"PurchaseDetails" => array(
									"UnitPrice"       => $unitPriceNow,
									"COGSAccountCode" => $SoldCode
								)
							);

							if ( isset( $checkwithoutStock['Items']['Item']['ItemID'] ) ) {
								$xero_desc = '-';
								if ( ! empty( xero_product_description( $checkwithoutStock ) ) ) {
									$xero_desc = xero_product_description( $checkwithoutStock );
								}
								$simpleItemsCode['Item']['Description']          = $xero_desc;
								$simpleItemsCode['Item']['Name']                 = '' != xero_product_name( $checkwithoutStock ) ? xero_product_name( $checkwithoutStock ) : $xItemName;
								$simpleItemsCode['SalesDetails']['UnitPrice']    = '' != xero_product_sales_unit_price( $checkwithoutStock ) ? xero_product_sales_unit_price( $checkwithoutStock ) : $unitPriceNow;
								$simpleItemsCode['PurchaseDetails']['UnitPrice'] = '' != xero_product_purchase_unit_price( $checkwithoutStock ) ? xero_product_purchase_unit_price( $checkwithoutStock ) : $unitPriceNow;
							}

							$againSimpleXero = $xero->Items( $simpleItemsCode );
							if ( isset( $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
								$errD = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
								returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
							} else if ( isset( $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
								$errD = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
								for ( $e = 0; $e < count( $errD ); $e ++ ) {
									$errorMessage = $againSimpleXero['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
									returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
								}
							}
						}
						if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
							$newItemsCodeShip = array(
								"Item" => array(
									"Code"        => $xero_shipping_price_code,
									"Description" => $xero_shipping_price_description
								)
							);
							$xero->Items( $newItemsCodeShip );
						}
						if ( $totalDiscount != 0 ) {
							$newItemsCodeDis = array(
								"Item" => array(
									"Code"        => "discount_price",
									"Description" => "Discount Price"
								)
							);
							$xero->Items( $newItemsCodeDis );
						}

						if ( $order->get_items( array( 'fee' ) ) ) {
							$newItemsCodeDis = array(
								"Item" => array(
									"Code"        => "fee_price",
									"Description" => "Fee Price"
								)
							);
							$xero->Items( $newItemsCodeDis );
						}
					}

					// Fetch shippinh tax class
					$shipping_tax = array();
					if ( $fetch_tax_method && wc_tax_enabled() ) {
						$shipping_tax_class = 'standard';
						foreach ( $order->get_items( array( 'shipping' ) ) as $item ) {
							if ( '' != $item->get_tax_class() ) {
								$shipping_tax_class = $item->get_tax_class();
							}
						}

						$simple_tax = $order->get_shipping_tax();

						if ( $simple_tax ) {
							$product_tax = $simple_tax;
						} else {
                                $product_tax        = 'standard';
                            }

						switch ( $fetch_tax_method ) {
							case "xero_simple_tax" :
								if ( $fetch_saved_tax && is_array( $fetch_saved_tax ) && array_key_exists( 'xero_' . $shipping_tax_class . '_taxmethods', $fetch_saved_tax ) ) {
									$shipping_tax['tax_class'] = xero_tax_type_code( $fetch_saved_tax[ 'xero_' . $shipping_tax_class . '_taxmethods' ] );
								}
								break;
							case "xero_complex_tax" :
								if ( $fetch_taxes_association && is_array( $fetch_taxes_association ) && array_key_exists( $product_tax, $fetch_taxes_association ) ) {
									$shipping_tax['tax_class'] = xero_tax_type_code( $fetch_taxes_association[ $product_tax ] );
								}
								break;
						}

						if ( 'inherit' == $shipping_tax_class ) {
							$shipping_class            = reset( $orderDetails );
							$shipping_tax['tax_class'] = $shipping_class['tax_class'];
						}
					} else {
						$shipping_tax['tax_class'] = 'NONE';
					}

					if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
						$oderList[ count( $orderDetails ) ]['Description']  = $xero_shipping_price_description;
						$oderList[ count( $orderDetails ) ]['Quantity']     = 1;
						$oderList[ count( $orderDetails ) ]['UnitAmount']   = $shippingPrice;
						$oderList[ count( $orderDetails ) ]['DiscountRate'] = 0;
						$oderList[ count( $orderDetails ) ]['ItemCode']     = $xero_shipping_price_code;

						if ( ! empty( $shipping_tax ) ) {
							$oderList[ count( $orderDetails ) ]['TaxType'] = $shipping_tax['tax_class'];
						}
						$oderList[ count( $orderDetails ) ]['AccountCode'] = $shippingCode && '' != $shippingCode ? $shippingCode : $salesAccount;
					}
					if ( $lic_method != "trail" ) {
						if ( $totalDiscount != 0 ) {
							if ( $order->get_shipping_method() && $shippingPrice > 0 ) {
								$nextCount = count( $orderDetails ) + 1;
							} else {
								$nextCount = count( $orderDetails );
							}
							$oderList[ $nextCount ]['Description']  = $usedCupons;
							$oderList[ $nextCount ]['Quantity']     = 1;
							$oderList[ $nextCount ]['UnitAmount']   = - $totalDiscount;
							$oderList[ $nextCount ]['DiscountRate'] = 0;
							$oderList[ $nextCount ]['ItemCode']     = "discount_price";
							// Add tax Rate name
							if ( '' != $orderDetails[ $i ]['tax_class'] ) {
								$oderList[ count( $orderDetails ) ]['TaxType'] = $orderDetails[ $i ]['tax_class'];
							}
							$oderList[ $nextCount ]['AccountCode'] = $shippingCode && '' != $shippingCode ? $shippingCode : $salesAccount;
						}
					}

					// Fee tax
					$fee_tax = array();
					if ( $fetch_tax_method && wc_tax_enabled() ) {
						$fee_class            = reset( $orderDetails );
						$fee_tax['tax_class'] = $fee_class['tax_class'];
					}

					// Add fee if exists
					if ( $order->get_items( array( 'fee' ) ) ) {
						$fee_count = count( $oderList );
						foreach ( $order->get_items( array( 'fee' ) ) as $item_type => $item ) {
							$oderList[ $fee_count ]['Description'] = "Fee Price";
							$oderList[ $fee_count ]['Quantity']    = 1;
							$oderList[ $fee_count ]['UnitAmount']  = $order->get_item_total( $item, false, true );;
							$oderList[ $fee_count ]['DiscountRate'] = 0;
							$oderList[ $fee_count ]['ItemCode']     = "fee_price";
							if ( ! empty( $fee_tax ) ) {
								$oderList[ $fee_count ]['TaxType'] = $fee_tax['tax_class'];
							}

							$oderList[ $fee_count ]['AccountCode'] = $salesAccount;
							$fee_count ++;
						}
					}

					$lineItemsArray = array();
					for ( $i = 0; $i < count( $oderList ); $i ++ ) {
						$lineItemsArray[] = array( "LineItem" => array( $oderList[ $i ] ) );
					}

//					$currentDate       = xeroom_invoice_date( $order );
					$currentDate       = date( 'Y-m-d H:i:s' );
					$xero_contact_name = get_xero_option( 'xero_contact_name' );

					if ( 'xeroom_use_company' == $xero_contact_name ) {
						$xero_add_contact_name = $shipAddress['company'];
					} elseif ( 'xeroom_use_email' == $xero_contact_name ) {
						$xero_add_contact_name = $shipAddress['email'];
					} else {
						$xero_add_contact_name = $shipAddress['first_name'] . ' ' . $shipAddress['last_name'];
					}

					if ( $lic_method != "trail" ) {
						$credit_note = array(
							array(
								"Type"            => "ACCRECCREDIT",
								"Contact"         => array(
									"Name"         => $xero_add_contact_name,
									"FirstName"    => $shipAddress['first_name'],
									"LastName "    => $shipAddress['last_name'],
									"EmailAddress" => $shipAddress['email'],
									"Addresses"    => array(
										"Address" => array(
											array(
												"AddressType"  => "POBOX",
												"AttentionTo"  => "",
												"AddressLine1" => $shipAddress['address_1'],
												"AddressLine2" => $shipAddress['address_2'],
												"City"         => $shipAddress['city'],
												"Region"       => $stateFullName,
												"Country"      => $countryFullName,
												"PostalCode"   => $shipAddress['postcode']
											)
										)
									),
									"Phones"       => array(
										"Phone" => array(
											"PhoneType"   => "MOBILE",
											"PhoneNumber" => $shipAddress['phone']
										)
									)
								),
								"Date"            => $currentDate,
								"Status"          => $default_credit_status,
								"CurrencyCode"    => $order->get_currency(),
								"LineAmountTypes" => "Exclusive",
								"Reference"       => $order_id,
								"LineItems"       => $lineItemsArray
							)
						);
					} else {
						$credit_note = array(
							array(
								"Type"            => "ACCRECCREDIT",
								"Contact"         => array(
									"Name"         => $xero_add_contact_name,
									"FirstName"    => $shipAddress['first_name'],
									"LastName "    => $shipAddress['last_name'],
									"EmailAddress" => $shipAddress['email'],
									"Addresses"    => array(
										"Address" => array(
											array(
												"AddressType"  => "POBOX",
												"AttentionTo"  => "Created using demo version of Xeroom",
												"AddressLine1" => "",
												"AddressLine2" => "",
												"AddressLine3" => "",
												"City"         => "",
												"Region"       => "",
												"Country"      => "",
												"PostalCode"   => ""
											)
										)
									),
									"Phones"       => array(
										"Phone" => array(
											"PhoneType"   => "MOBILE",
											"PhoneNumber" => $shipAddress['phone']
										)
									)
								),
								"Date"            => $currentDate,
								"Status"          => $default_credit_status,
								"CurrencyCode"    => $orderCurency,
								"LineAmountTypes" => "Exclusive",
								"Reference"       => $order_id,
								"LineItems"       => $lineItemsArray
							)
						);
					}

					if ( $license_checked != 'expired' ) {
						$creditNotesCreated = $xero->CreditNotes( $credit_note );

						if ( isset( $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
							$errD = $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
							returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
						} else if ( isset( $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
							$errD = $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
							for ( $e = 0; $e < count( $errD ); $e ++ ) {
								$errorMessage = $creditNotesCreated['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
								returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
							}
						} else if ( $creditNotesCreated['Status'] != "OK" ) {
							returnErrorMessageByXero( $order_id, 'Credit Note was not created', $xeroTime, $errorType );
						}
						// Allocate Credit Note to Invoice No.
						if ( $creditNotesCreated['Status'] == "OK" ) {
							update_post_meta( $order_id, 'xeroom_cred_note_generated', 1 );
							update_post_meta( $order_id, 'xeroom_cred_note_id', $creditNotesCreated['CreditNotes']['CreditNote']['CreditNoteID'] );

							$message = sprintf( __( 'Xero Credit Note Generated.', 'xeroom' ) );
							$order->add_order_note( $message );

							$total_amount = $order->get_total();

							$alocateCreditNote = array(
								"Allocation" => array(
									"Invoice"       => array(
										"InvoiceID" => get_post_field( 'post_content_filtered', $order_id ),
									),
									"AppliedAmount" => $total_amount,
									"Date"          => date( 'Y-m-d' ),
								),
							);

							$sendCreditNoteAllocation = $xero->CreditNotes( $alocateCreditNote, array( 'Allocations' => $creditNotesCreated['CreditNotes']['CreditNote']['CreditNoteID'] ) );

							if ( isset( $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
								$errD = $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
								returnErrorMessageByXero( $order_id, $errD, $xeroTime, $errorType );
							} else if ( isset( $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
								$errD = $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
								for ( $e = 0; $e < count( $errD ); $e ++ ) {
									$errorMessage = $sendCreditNoteAllocation['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $e ]['Message'];
									returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $errorType );
								}
							} else if ( $sendCreditNoteAllocation['Status'] != "OK" ) {
								returnErrorMessageByXero( $order_id, 'Credit Note was not Allocated', $xeroTime, $errorType );
							}

							if ( $sendCreditNoteAllocation['Status'] == "OK" ) {
								update_post_meta( $order_id, 'xeroom_cred_note_allocated', 1 );

								$message = sprintf( __( 'Xero Credit Note Allocated to Invoice.', 'xeroom' ) );
								$order->add_order_note( $message );
							}

							update_xero_option( 'xero_connection_status', 'active' );
							update_xero_option( 'xero_connection_status_message', '' );
						}
					}
				}
			} else {
				$mMessage = "Your license has expired, please contact xeroom customer support team.";
				returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
			}
		} else {
			$mMessage = "Please use a premium or trail license!";
			returnErrorMessageByXero( $order_id, $mMessage, $xeroTime, $errorType );
		}
	}
}

/**
 * Save log error entry
 *
 * @param $response
 * @param int $order_id
 * @param string $error_type
 */
function xeroom_check_connection_message( $response, $order_id = 0, $error_type = 'connection' ) {
	if ( ! empty( $response ) ) {
		$xeroTime = date( 'Y-m-d H:i:s' );
		if ( isset( $response['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'] ) ) {
			$errD = $response['Elements']['DataContractBase']['ValidationErrors']['ValidationError']['Message'];
			returnErrorMessageByXero( $order_id, $errD, $xeroTime, $error_type );
		} else if ( isset( $response['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0] ) ) {
			$errD = $response['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][0];
			if ( ! empty( $errD ) ) {
				for ( $er = 0; $er < count( $errD ); $er ++ ) {
					$errorMessage = $response['Elements']['DataContractBase']['ValidationErrors']['ValidationError'][ $er ]['Message'];
					returnErrorMessageByXero( $order_id, $errorMessage, $xeroTime, $error_type );
				}
			}
		} else if ( isset( $response['Status'] ) && "OK" == $response['Status'] ) {
			update_xero_option( 'xero_connection_status', 'active' );
			update_xero_option( 'xero_connection_status_message', '' );
		} else {
			returnErrorMessageByXero( $order_id, $response, $xeroTime, $error_type );
		}
	}
}

/**
 * Add order suffix on manual resending
 *
 * @param $order_id
 *
 * @return string
 */
function xeroom_add_invoice_suffix( $order_id ) {
	$existing_no   = get_post_meta( $order_id, 'xeroom_invoice_resend_suffix', true );
	$return_suffix = 'A';

	if ( $existing_no ) {
		$letterAscii = ord( $existing_no );
		$letterAscii ++;
		$return_suffix = chr( $letterAscii );
	}

	update_post_meta( $order_id, 'xeroom_invoice_resend_suffix', $return_suffix );

	return $return_suffix;
}

/**
 * Reduce SKU length to Xero max size
 *
 * @param $sku
 *
 * @return mixed
 */
function xeroom_reduce_sku_length( $sku ) {
	if ( strlen( $sku ) > 30 ) {
		$sku = 'SKUFAIL_' . substr( $sku, 0, 20 );
	}

	return $sku;
}
